package com.hb.proj.auth.controller;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.auth.service.AuthService;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.xframework.util.SessionThreadLocal;
import com.hb.xframework.util.SessionUser;


@Controller
@RequestMapping("/**/auth")
public class AuthController {

	@Autowired
	private AuthService service;
	
	/**
	 * 主页菜单
	 * @param response
	 */
	@RequestMapping("/loadMyMenus")
	public void loadMyMenus(HttpServletResponse response){
		try{
			SessionUser su=SessionThreadLocal.getSessionUser();
			if(su==null){
				JsonOutUtils.returnError(response, "缺少登录信息");
				return ;
			}
			List<Map<String,Object>> menus=null;
			if(su.isSysAdmin()){
				menus=service.loadAllMenus();
			}
			else{
				menus=service.loadMyMenus(su.getUserId());
			}
			JsonOutUtils.returnOkWithData(response, menus);
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "获取菜单服务出错");
		}
		
	}
	
	
	/**
	 * 可分配的权限节点（菜单,功能点暂时不需要）
	 * @param response
	 */
	@RequestMapping("/loadAuthNodes")
	public void loadAuthNodes(HttpServletResponse response){
		List<Map<String,Object>> nodes=service.loadAllMenuNodes();
		if(nodes==null||nodes.size()==0){
			JsonOutUtils.returnError(response, "未找到数据");
			return;
		}
		/*List<Map<String,Object>> fnodes=service.loadAllFunNodes();  //暂时不需要对功能点进行分配
		if(fnodes!=null){
			nodes.addAll(fnodes);
		}*/
		JsonOutUtils.returnOkWithData(response, nodes);
	}
	
	@RequestMapping("/loadUsAuth")
	public void loadUsAuth(String userId,HttpServletResponse response){
		if(StringUtils.isEmpty(userId)){
			JsonOutUtils.returnError(response, "缺少用户信息");
			return;
		}
		JsonOutUtils.returnOkWithData(response, service.loadUserAuth(userId));
	}
	
	
	@RequestMapping("/save")
	public void save(String userId,String authIds,HttpServletResponse response){
		if(StringUtils.isEmpty(userId)||StringUtils.isEmpty(authIds)){
			JsonOutUtils.returnError(response, "缺少必要参数");
			return;
		}
		service.saveUserAuth(userId, authIds);
		JsonOutUtils.returnOk(response);
	}
}
