package com.hb.proj.auth.controller;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.auth.service.MenuService;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.RequestUtils;
import com.hb.xframework.util.JsonParamHandler;
import com.hb.xframework.util.SessionThreadLocal;
import com.hb.xframework.util.SessionUser;

@Controller
@RequestMapping("/**/menu")
public class MenuController {

	@Autowired
	private MenuService service;
	
	@RequestMapping("/get")
	public void get(String menuId,HttpServletResponse response){
		if(menuId==null){
			JsonOutUtils.returnError(response, "菜单编号不能为空，操作取消");
			return ;
		}
		Map<String,Object>  menu=service.getMenu(menuId);
		JsonOutUtils.returnOkWithData(response, menu);
	}
	
	@RequestMapping("/add")
	public void add(HttpServletRequest request,HttpServletResponse response){
		Map<String,Object> args=RequestUtils.getParams(request);
		if(StringUtils.isEmpty((String)args.get("menuName"))){
			JsonOutUtils.returnError(response, "缺少菜单名称");
			return ;
		}
		args.remove("oldFatherMenuId");
		if(StringUtils.isEmpty(args.get("fatherMenuId"))){
			args.put("fatherMenuId", "0");
		}
		if(StringUtils.isEmpty(args.get("displayNum"))){
			args.put("displayNum", 0);
		}
		else{
			args.put("displayNum", Integer.parseInt((String)args.get("displayNum")));
		}
		String newAssistCode = service.generateLeveledCode((String)args.get("fatherMenuId"));
		if(newAssistCode==null){
			JsonOutUtils.returnError(response, "指定的父菜单不存在");
			return ;
		}
		args.put("assistCode",newAssistCode);
		SessionUser su=SessionThreadLocal.getSessionUser();
		args.put("creator",su!=null?su.getUserName():"unknow");
		String newMenuId=service.addMenu(args);
		JsonOutUtils.returnOkWithData(response, newMenuId);
	}
	
	
	@RequestMapping("/update")
	public void update(HttpServletRequest request,HttpServletResponse response){
		Map<String,Object> args=RequestUtils.getParams(request);
		if(StringUtils.isEmpty(args.get("fatherMenuId"))){
			args.put("fatherMenuId", "0");
		}
		String oldFatherMenuId=(String)args.remove("oldFatherMenuId");
		String fatherMenuId=(String)args.get("fatherMenuId");
		if (!oldFatherMenuId.equals(fatherMenuId)) { // 隶属关系变动后，修正assistCode
			String newAssistCode =service.generateLeveledCode(fatherMenuId);
			if(newAssistCode==null){
				JsonOutUtils.returnError(response, "指定的父菜单不存在");
				return ;
			}
			service.updateMenu(args,newAssistCode);
		}
		else{
			service.updateMenu(args,null);
		}
		JsonOutUtils.returnOk(response);
	}
	
	
	
	@RequestMapping("/delete")
	public void delete(String menuId,HttpServletResponse response){
		if(menuId==null){
			JsonOutUtils.returnError(response, "菜单编号不能为空，操作取消");
			return ;
		}
		service.deleteMenu(menuId);
		JsonOutUtils.returnOk(response);
	}
	
	/**
	 * 获得菜单子节点（ztree格式）
	 * @param superId
	 * @param targetDepth
	 * @param response
	 */
	@RequestMapping("/treeNode")
	public void treeNode(String superId,HttpServletResponse response){
		List<Map<String,Object>> nodes=service.queryTreeNodeBySuperId(StringUtils.isEmpty(superId)?"0":superId);
		if(nodes==null){
			return ;
		}
		JsonOutUtils.preprocessTreeNodes(nodes, null);
		JsonOutUtils.writeToResponse(response, JsonParamHandler.getJson(nodes));
	}
	
}
