package com.hb.proj.auth.service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.xframework.dao.core.SpringJdbcDAO;
import com.hb.xframework.dao.util.UUIDHexGenerator;
import com.hb.xframework.util.PinyinFromHanzi;

@Service
public class MenuService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	
	public List<Map<String,Object>> queryTreeNodeBySuperId(String fatherMenuId) {
		StringBuilder sql=new StringBuilder();
		sql.append("select m.menu_id id,m.menu_name name,m.father_menu_id p_id,menu_link, ");
		sql.append("(select count(1) from t_menu cm where cm.father_menu_id=m.menu_id) child_count");
		sql.append(" from t_menu  m where m.father_menu_id=?");
		sql.append(" order by m.display_num");
		return dao.queryForListMap(sql.toString(),fatherMenuId);
	}
	
	public Map<String,Object> getMenu(String menuId){
		StringBuilder sql=new StringBuilder(100);
		sql.append("select menu.*,");
		sql.append("(select sm.menu_name from t_menu sm where sm.menu_id=menu.father_menu_id) father_menu_name");
		sql.append(" from t_menu menu where menu.menu_id=?");
		return dao.queryForSingleMap(sql.toString(), menuId);
	}
	
	public String  addMenu(Map<String,Object> menu){
		UUIDHexGenerator uuid=new UUIDHexGenerator();
		menu.put("createTime",new Date());
		menu.put("menuId",uuid.generate());
		if(StringUtils.isEmpty((String)menu.get("fatherMenuId"))){
			menu.put("fatherMenuId", "0");
		}
		if(menu.get("displayNum")==null){
			Number num=getMenuMaxDispNum((String)menu.get("fatherMenuId"));
			menu.put("displayNum",num==null?0:(num.intValue()+1));
		}
		dao.insert(menu, "t_menu");
		
		//增加菜单的时候，自动增加浏览功能项
		Map<String,Object>  fun=new HashMap<String,Object>();
		fun.put("functionId", uuid.generate());
		fun.put("createTime", new Date());
		fun.put("menuId",menu.get("menuId"));
		fun.put("creator", menu.get("creator"));
		fun.put("functionCode", PinyinFromHanzi.getPinyin((String)menu.get("menuName")).toUpperCase()+"_BROWSE");
		fun.put("functionName", "浏览");
		
		dao.insert(fun, "t_function");
		
		return (String)menu.get("menuId");
	}
	
	public boolean deleteMenu(String menuId){
		String sql="delete from t_menu where menu_id=?";
		dao.getJdbcTemplate().update(sql, menuId);
		
		//级联删除分配给所有用户的该的功能
		sql="delete from t_user_function where function_id in (select function_id from t_function where menu_id=?)";
		dao.getJdbcTemplate().update(sql,menuId);
				
		sql="delete from t_function where menu_id=?";
		dao.getJdbcTemplate().update(sql,menuId);
		return true;
	}
	
	
	
	public boolean updateMenu(Map<String,Object> menu,String newAssistCode){
		if(StringUtils.isNotEmpty(newAssistCode)){
			updateAssistCode((String)menu.get("assistCode"),newAssistCode);
			menu.put("assistCode",newAssistCode);
		}
		dao.update(menu, "t_menu", "menu_id");
		return true;
	}
	
	/**
	 * 获得指定级别下的最大assistCode
	 * @param superId
	 * @return
	 */
	public String generateLeveledCode(String superId) {
		String superAssistCode=null;
		if("0".equals(superId)){  //顶级父节点
			superAssistCode="C";
		}
		else{
			Map<String,Object> superMenu=getMenu(superId);
			if(superMenu==null){
				return null;
			}
			superAssistCode=(String)superMenu.get("assistCode");
			if(StringUtils.isEmpty(superAssistCode)){
				superAssistCode="C";
			}
		}
		String subMaxAssistCode =getMaxSubAssistCode(superId);
		if (subMaxAssistCode == null || "null".equalsIgnoreCase(subMaxAssistCode)) { // 没有子编码时
			return superAssistCode + "000";
		}
		String subSerial = subMaxAssistCode.replaceFirst(superAssistCode, ""); // 子编码流水号

		int nextNum = Integer.parseInt(subSerial) + 1;
		subSerial = String.format("%1$03d", nextNum);
		return superAssistCode + subSerial;
	}
	
	private String getMaxSubAssistCode(String superId) {
		String sql = "select max(assist_code) assist_code from t_menu where father_menu_id=?";
		Map<String,Object> rst=dao.queryForSingleMap(sql, superId);
		return rst!=null?(String)rst.get("assistCode"):null;
	}
	
	private boolean updateAssistCode(String assistCode, String newAssistCode){
		StringBuilder sql = new StringBuilder();
		sql.append("update t_menu set assist_code=replace(assist_code,?,?)");
		sql.append(" where assist_code like ?");
		dao.getJdbcTemplate().update(sql.toString(), assistCode, newAssistCode, assistCode + "%");
		return true;
	}
	
	private Number getMenuMaxDispNum(String fatherMenuId){
		String sql="select max(m.display_num) num from t_menu m where m.father_menu_id=?";
		Map<String,Object> rst=dao.queryForSingleMap(sql, fatherMenuId);
		return rst!=null?((Number)rst.get("num")):null;
	}
}
