package com.hb.proj.car.controller;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.car.service.CarOilMonitorService;
import com.hb.proj.car.service.OilMeterService;
import com.hb.proj.excel.ExcelUtils;
import com.hb.proj.utils.DataAuthUtils;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.LayGridJsonUtils;
import com.hb.proj.utils.MapUtils;
import com.hb.proj.utils.MySessionUser;
import com.hb.proj.utils.OkhttpUtils;
import com.hb.proj.utils.RequestUtils;
import com.hb.proj.utils.RptMonthUtil;
import com.hb.xframework.dao.util.PageModel;
import com.hb.xframework.util.DateUtil;
import com.hb.xframework.util.SessionThreadLocal;
import com.hb.xframework.util.SysConfigUtil;

@Controller
@RequestMapping("/**/oil")
public class CarOilMonitorController {

	@Autowired
	private CarOilMonitorService  service;
	
	@Autowired
	private OilMeterService  meterService;
	
	/**
	 * 油量当前数据--安装检测用
	 * @param request
	 * @param page
	 * @param limit
	 * @param response
	 */
	@RequestMapping("/checkCurrentOil")
	public void checkCurrentOil(HttpServletRequest request,Integer page,Integer limit,HttpServletResponse response){
		try{
			
			Map<String,Object> args=RequestUtils.getParams(request);
			PageModel<Map<String,Object>> pagedData=service.queryCarOil(args,null,page, limit);
			LayGridJsonUtils.wirte(response, pagedData.getData(),pagedData.getTotalRow(),"yyyy-MM-dd HH:mm:ss");
		}
		catch(Exception e){
			e.printStackTrace();
			LayGridJsonUtils.error(response, "服务出现错误");
		}
		
	}
	
	/**
	 * 直接从宏电平台获取数据，用于安装检测
	 * @param request
	 * @param dtuNum
	 * @param response
	 */
	@RequestMapping("/loadOilFromHd")
	public void loadOilFromHd(HttpServletRequest request,String dtuNum,HttpServletResponse response){
		try{
			if(StringUtils.isEmpty(dtuNum)){
				JsonOutUtils.returnError(response, "未提供设备识别号");
				return;
			}
			Map<String,String> reqArgs=new HashMap<String,String>();
			reqArgs.put("vins", dtuNum);
			reqArgs.put("queryDate", DateUtil.format(new Date(), "yyyy-MM-dd"));
			String resp=OkhttpUtils.post(SysConfigUtil.getConfig("hdOilAPI"), reqArgs);
			if(StringUtils.isEmpty(resp)){
				JsonOutUtils.returnError(response,"未查询到油量数据:");
				return;
			}
			else{
				/*List<OilCurrentDTO> oilDats=JSON.parseObject(resp,new TypeReference<List<OilCurrentDTO>>() {});
				if(oilDats==null||oilDats.size()==0){
					JsonOutUtils.returnError(response,"未查询到油量数据:");
					return;
				}*/
				JsonOutUtils.returnOkWithData(response, resp);
			}
			
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
		
	}
	
	
	/**
	 * 油量采集数据查询-表格显示
	 * @param mountId
	 * @param startDate
	 * @param endDate
	 * @param page
	 * @param limit
	 * @param response
	 */
	@RequestMapping("/queryOilCollect")
	public void queryOilCollect(String mountId,String startDate,String endDate,Integer page,Integer limit,HttpServletResponse response){
		try{
			if(StringUtils.isEmpty(mountId)){
				LayGridJsonUtils.error(response, "未指定油箱编号");
				return;
			}
			if(StringUtils.isEmpty(startDate)||StringUtils.isEmpty(endDate)){
				startDate=DateUtil.format(new Date(), "yyyy-MM-dd");
				endDate=startDate+" 23:59:59";
				startDate=startDate+" 00:00:00";
			}
			PageModel<Map<String,Object>> pagedData=service.queryOilCollect(mountId,DateUtil.parse(startDate),DateUtil.parse(endDate), page,limit);
			LayGridJsonUtils.wirte(response, pagedData.getData(),pagedData.getTotalRow(),"yyyy-MM-dd HH:mm:ss");
		}
		catch(Exception e){
			e.printStackTrace();
			LayGridJsonUtils.error(response, "服务出现错误");
		}
		
	}
	
	/**
	 * 油量采集数据查询-绘制曲线
	 * @param mountId
	 * @param startDate
	 * @param endDate
	 * @param response
	 */
	@RequestMapping("/queryOilCollectCurve")
	public void queryOilCollectCurve(String mountId,String startDate,String endDate,HttpServletResponse response){
		try{
			if(StringUtils.isEmpty(mountId)){
				JsonOutUtils.returnError(response, "未指定油箱编号");
				return;
			}
			if(StringUtils.isEmpty(startDate)||StringUtils.isEmpty(endDate)){
				startDate=DateUtil.format(new Date(), "yyyy-MM-dd");
				endDate=startDate+" 23:59:59";
				startDate=startDate+" 00:00:00";
			}
			Date sd=DateUtil.parse(startDate),ed=DateUtil.parse(endDate);
			List<Map<String,Object>> oildatas=service.queryOilCollectForCurve(mountId,sd,ed);
			if(oildatas==null||oildatas.size()==0){
				JsonOutUtils.returnError(response, "未找到数据");
				return;
			}
			List<Map<String,Object>> groupCurves=new ArrayList<Map<String,Object>>();
			groupCurves.add(buildCurveData("oil", "油量",oildatas));
			Map<String,Object> rtnCurve=MapUtils.builder("title","油量曲线","yTitle","体积/液位","yUnit","L/0.1mm","xFixedMin",sd.getTime(),"curves",groupCurves);
			JsonOutUtils.returnOkWithData(response, rtnCurve);
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
		
	}
	
	/**
	 * 油量采集数据查询-绘制曲线(双油箱同时绘制)
	 * @param carId
	 * @param startDate
	 * @param endDate
	 * @param response
	 */
	@RequestMapping("/queryOilCollectDblCurve")
	public void queryOilCollectDblCurve(String carId,String startDate,String endDate,HttpServletResponse response){
		try{
			if(StringUtils.isEmpty(carId)){
				JsonOutUtils.returnError(response, "未指定车辆");
				return;
			}
			List<Map<String,Object>> oilboxs=meterService.loadCarBoxs(carId);
			if(oilboxs==null||oilboxs.size()==0){
				JsonOutUtils.returnError(response, "该车辆未配置油量计设备");
				return;
			}
			
			if(StringUtils.isEmpty(startDate)||StringUtils.isEmpty(endDate)){
				startDate=DateUtil.format(new Date(), "yyyy-MM-dd");
				endDate=startDate+" 23:59:59";
				startDate=startDate+" 00:00:00";
			}
			Date sd=DateUtil.parse(startDate),ed=DateUtil.parse(endDate);
			
			
			List<Map<String,Object>> groupCurves=new ArrayList<Map<String,Object>>();
			List<Map<String,Object>> oildatas=null;
			for(Map<String,Object> itm : oilboxs){
				oildatas=service.queryOilCollectForCurve((String)itm.get("val"),sd,ed);
				if(oildatas!=null&&oildatas.size()>0){
					groupCurves.add(buildCurveData("oil"+itm.get("txt"), "油量-"+itm.get("txt"),oildatas));
				}
				
			}
			
			
			Map<String,Object> rtnCurve=MapUtils.builder("title","油量曲线","yTitle","体积/液位","yUnit","L/0.1mm","xFixedMin",sd.getTime(),"curves",groupCurves);
			JsonOutUtils.returnOkWithData(response, rtnCurve);
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
		
	}
	
	
	
	private Map<String,Object>  buildCurveData(String id,String title,List<Map<String,Object>> datas){
		 List<String> times=new ArrayList<String>();
		 List<Object> vals=new ArrayList<Object>();
		 for(Map<String,Object> d : datas){
			 times.add(DateUtil.format((Date)d.get("dataTime"), "yyyy-MM-dd HH:mm:ss"));
			 vals.add(d.get("data"));
		 }
		 return MapUtils.builder("id",id,"title",title,"xVals",times,"yVals",vals);
	}
	
	/**
	 * 供项目部通过车辆查询油量异常数据
	 * @param request
	 * @param page
	 * @param limit
	 * @param response
	 */
	@RequestMapping("/queryOilException")
	public void queryOilException(HttpServletRequest request,Integer page,Integer limit,HttpServletResponse response){
		try{
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			if(su==null){  //测试暂时不用
				LayGridJsonUtils.error(response, "缺少登录信息");
				return;
		    }
			Map<String,Object> args=RequestUtils.getParams(request);
			String startDate=request.getParameter("startDate");
			String endDate=request.getParameter("endDate");
			if(StringUtils.isEmpty(startDate)&&StringUtils.isEmpty(endDate)){
				String now=DateUtil.format((new Date()),"yyyy-MM-dd");
				startDate=now+" 00:00:00";
				endDate=now+" 23:59:59";
				
			}
			else if(StringUtils.isNotEmpty(startDate)&&StringUtils.isNotEmpty(endDate)){
				startDate+=" 00:00:00";
				endDate+=" 23:59:59";
			}
			else if(StringUtils.isNotEmpty(startDate)){
				endDate=startDate+" 23:59:59";
				startDate+=" 00:00:00";
			}
			else if(StringUtils.isNotEmpty(endDate)){
				startDate=endDate+" 00:00:00";
				endDate+=" 23:59:59";
			}
			
			args.put("startDate", startDate);
			args.put("endDate", endDate);
			PageModel<Map<String,Object>> pagedData=service.queryOilException(args,DataAuthUtils.getAuthOrgAssistCode(su),page, limit);
			LayGridJsonUtils.wirte(response, pagedData.getData(),pagedData.getTotalRow(),"yyyy-MM-dd HH:mm:ss");
		}
		catch(Exception e){
			e.printStackTrace();
			LayGridJsonUtils.error(response, "服务出现错误");
		}
		
	}
	
	
	@RequestMapping("/queryOilMonitorRpt")
	public void queryOilMonitorRpt(HttpServletRequest request,Integer page,Integer limit,HttpServletResponse response){
		try{
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			if(su==null){  //测试暂时不用
				LayGridJsonUtils.error(response, "缺少登录信息");
				return;
		    }
			Map<String,Object> args=RequestUtils.getParams(request);
			if(StringUtils.isEmpty(request.getParameter("rptMonth"))){
				args.put("rptMonth", RptMonthUtil.getDefaultMonth());
			}
			PageModel<Map<String,Object>> pagedData=service.queryOilMonitorRpt(args,DataAuthUtils.getAuthOrgAssistCode(su), page,limit);
			LayGridJsonUtils.wirte(response, pagedData.getData(),pagedData.getTotalRow(),"yyyy-MM-dd HH:mm:ss");
		}
		catch(Exception e){
			e.printStackTrace();
			LayGridJsonUtils.error(response, "服务出现错误");
		}
		
	}
	
	@RequestMapping("/exportMonitorRpt")
	public void exportMonitorRpt(HttpServletRequest request,HttpServletResponse response){
		try{
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			if(su==null){  //测试暂时不用
				JsonOutUtils.writeToResponse(response, "<script>alert('缺少登录信息');window.close();</script>","text/html;charset=UTF-8");
				return;
		    }
			
			Map<String,Object> args=RequestUtils.getParams(request);
			if(StringUtils.isEmpty(request.getParameter("rptMonth"))){
				args.put("rptMonth", RptMonthUtil.getDefaultMonth());
			}
			
			PageModel<Map<String,Object>> pagedData=service.queryOilMonitorRpt(args,DataAuthUtils.getAuthOrgAssistCode(su), 1,10000);
			if(pagedData==null||pagedData.getData()==null||pagedData.getData().size()==0){
				JsonOutUtils.writeToResponse(response, "<script>alert('暂时无数据可导出');window.close();</script>","text/html;charset=UTF-8");
				return;
			}
			String templateName=File.separator+"excel"+File.separator+"oilMonitorRpt.xls";
			Map<String,Object> context=new HashMap<String,Object>();
			context.put("ds", pagedData.getData());
			Workbook wb=MonitorRptExportBound.bound(context,ExcelUtils.reader(templateName,request.getSession().getServletContext()));
			ExcelUtils.exportExcel(response, "油量监测统计", wb);
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.writeToResponse(response, "<script>alert('导出数据出现错误');window.close();</script>","text/html;charset=UTF-8");
		}
	}
}
