package com.hb.proj.car.controller;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.car.service.CarSumRptService;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.RptOuter;
import com.hb.xframework.util.DateUtil;
/**
 * 柴油车汇总统计
 */
@Controller
@RequestMapping("/**/rpt")
public class CoilTypeSumRptController {

	@Autowired
	private CarSumRptService service;
	
	
	@RequestMapping("/checkLackRptData")
	public void checkLackRptData(String rptYear,Integer rptQuarter,String rptMonth,HttpServletResponse response){
		try{
			if(!StringUtils.isBlank(rptMonth)){
				JsonOutUtils.returnOkWithData(response, service.checkLackRptData(rptMonth));
				return;
			}
			else if(rptQuarter!=null&&!StringUtils.isBlank(rptYear)){
				Calendar  ca=Calendar.getInstance();
				int month=ca.get(Calendar.MONTH)+1;
				int nowQuarter=(int)Math.ceil(month/3.0);
				if(rptQuarter==nowQuarter&&rptYear.equals(String.valueOf(ca.get(Calendar.YEAR)))){  //当前季度
					JsonOutUtils.returnOkWithData(response, service.checkLackRptData(DateUtil.format(ca.getTime(), "yyyy-MM")));
					return;
				}
			}
			else if(!StringUtils.isBlank(rptYear)){
				Date now=new Date();
				if(rptYear.equals(DateUtil.format(now, "yyyy"))){  //当年
					JsonOutUtils.returnOkWithData(response, service.checkLackRptData(DateUtil.format(now, "yyyy-MM")));
					return;
				}
			}
			JsonOutUtils.returnOk(response);
			
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	/**
	 * 柴油车分类耗油统计
	 * @param rptYear
	 * @param rptQuarter  季度
	 * @param request
	 * @param response
	 */
	@RequestMapping("/loadCoilCarSum")
	public void loadCoilCarSum(String rptYear,Integer rptQuarter,HttpServletRequest request,HttpServletResponse response){
		try{
			if(StringUtils.isEmpty(rptYear)){
				rptYear=DateUtil.format(new Date(), "yyyy");
			}
			if(rptQuarter==null){
				Calendar  ca=Calendar.getInstance();
				int month=ca.get(Calendar.MONTH)+1;
				rptQuarter=(int)Math.ceil(month/3.0);
			}
			int emth=rptQuarter*3,smth=emth-2;
			
			String startMonth=rptYear+"-"+String.format("%02d", smth);
			String endMonth=rptYear+"-"+String.format("%02d", emth);
			
			String[] numName={"一","二","三","四"};
			String rptTitle=rptYear+"年"+numName[rptQuarter-1]+"季度";
			Map<String,Object> rootMap=new HashMap<String,Object>();
			rootMap.put("rptTitle", rptTitle);
			
			List<Map<String,Object>>  orgs=service.loadAllSubOrgs();
			List<Map<String,Object>>  coilTypes=service.loadCoilCarTypes();
			rootMap.put("orgDS", orgs); //所有项目部
			rootMap.put("carTypeDS", coilTypes); //柴油车类型
			rootMap.put("carCountDS", service.loadCoilCarCountRpt(startMonth,endMonth)); //柴油车按类型、单位统计数量
			rootMap.put("coilCostDS", service.loadCoilCarRealCost(startMonth,endMonth)); //柴油车按类型、单位统计耗油量
			
			rootMap.put("formulaDS",getFormulaDS(coilTypes,orgs));
			
			RptOuter.clearTemplateCache();
			RptOuter.write(request,response,rootMap,"coilCarSortRpt.ftl");
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	//报表中公式的生成
	private Map<String,String> getFormulaDS(List<Map<String,Object>> coilTypes,List<Map<String,Object>> orgs){
		Map<String,String>  ds=new HashMap<String,String>();
		char cn=0;
		//横向合计公式
		String exp=null,exp2=null;
		for(int j=0,len2=coilTypes.size();j<len2;j++){
			exp=""; //车辆数合计
			exp2=""; //耗油量合计
			for(int i=0,len=orgs.size();i<len;i++){
				cn=(char)(i*2+65+4);  //从E列开始
				exp+="+"+String.valueOf(cn)+""+(j+3);
				cn=(char)(i*2+65+5);  //从F列开始
				exp2+="+"+String.valueOf(cn)+""+(j+3);
			}
			ds.put((String)coilTypes.get(j).get("codeId"), exp.substring(1));
			ds.put((coilTypes.get(j).get("codeId")+"_coil"), exp2.substring(1));
			
			
		}
		
		//纵向合计公式
		int typeStartIdx=3,typeEndIdx=coilTypes.size()+typeStartIdx-1;
		for(int i=0,len=orgs.size();i<len;i++){
			cn=(char)(i*2+65+4);  //从E列开始,车辆数
			ds.put((String)orgs.get(i).get("orgId"), String.format("sum(%s%d:%s%d)", cn,typeStartIdx,cn,typeEndIdx));
			cn=(char)(i*2+65+5);  //从F列开始,耗油数
			ds.put((orgs.get(i).get("orgId")+"_coil"),String.format("sum(%s%d:%s%d)", cn,typeStartIdx,cn,typeEndIdx));
		}
		
		//总合计
		ds.put("totalCar",String.format("sum(%s%d:%s%d)", "C",typeStartIdx,"C",typeEndIdx));
		ds.put("totalCoil",String.format("sum(%s%d:%s%d)", "D",typeStartIdx,"D",typeEndIdx));
		
		return ds;
	}
	
	
}
