package com.hb.proj.car.controller;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.car.service.CarRptUtils;
import com.hb.proj.car.service.CarSumRptService;
import com.hb.proj.car.service.ERPKLSumRptService;
import com.hb.proj.model.CarConsume;
import com.hb.proj.model.OtherConsume;
import com.hb.proj.model.OtherConsumeWrapVO;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.RptOuter;
import com.hb.xframework.util.DateUtil;
import com.hb.xframework.util.MapUtils;

@Controller
@RequestMapping("/**/rpt")
public class ERPKLSumRptController {

	@Autowired
	private ERPKLSumRptService  service;
	
	@Autowired
	private CarSumRptService baseService;
	
	/**
	 * 统计指定年份各月份的考核里程、行驶里程。目前统计的是全分公司
	 * @param request
	 * @param response
	 */
	@RequestMapping("/rptMile")
	public void rptMile(String year,HttpServletRequest request,HttpServletResponse response){
		try{
			if(StringUtils.isEmpty(year)){
				Calendar ca=Calendar.getInstance();
				year=String.valueOf(ca.get(Calendar.YEAR));
			}
			
			List<Map<String,Object>>  rptdatas=new ArrayList<Map<String,Object>>(12);
		
			String[] mthNames={"一月","二月","三月","四月","五月","六月","七月","八月","九月","十月","十一月","十二月"};
			for(int i=1;i<=12;i++){
				rptdatas.add(MapUtils.build("month",mthNames[i-1]));
			}
			List<Map<String,Object>> miles=service.rptCheckAndTravelMile(year);
			if(miles!=null&&miles.size()>0){
				int monthNum=0;
				for(Map<String,Object> itm : miles){
					monthNum=Integer.parseInt((String)itm.get("rptDate"));
					rptdatas.get(monthNum-1).putAll(itm);
				}
			}
			
			String rptTitle=year+"年";
			Map<String,Object> rootMap=new HashMap<String,Object>();
			rootMap.put("rptTitle", rptTitle);
			rootMap.put("datas", rptdatas); 
			RptOuter.clearTemplateCache();
			RptOuter.write(request,response,rootMap,"mileYearRpt.ftl");
			
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	/**
	 * ERP昆仑卡汇总表(统计数据来源于其它油耗表，录入的各单位各月份的合计数据，不在通过单车数据累加2020.9.15)
	 * @param rptMonth
	 * @param request
	 * @param response
	 */
	@RequestMapping("/loadERPKLSum")
	public void loadERPKLSum(String rptMonth,HttpServletRequest request,HttpServletResponse response){
		try{
			if(StringUtils.isEmpty(rptMonth)){
				rptMonth=DateUtil.format(new Date(), "yyyy-MM");
			}
			String startMonth=(rptMonth.split("-"))[0]+"-01";
			Map<String,CarConsume> thisMonthIdx=builderIndex(service.loadERPKLSumMile(rptMonth, rptMonth));
			Map<String,CarConsume> sumRptIdx=builderIndex(service.loadERPKLSumMile(startMonth, rptMonth));
			
			Map<String,OtherConsumeWrapVO> otherMonth=builderOtherIndex(service.loadOtherConsume(rptMonth, rptMonth));
			Map<String,OtherConsumeWrapVO> otherSum=builderOtherIndex(service.loadOtherConsume(startMonth, rptMonth));
			
			String rptMonthTitle=rptMonth.replace("-", "年")+"月";
			Map<String,Object> rootMap=new HashMap<String,Object>();
			rootMap.put("rptMonthTitle", rptMonthTitle);
			rootMap.put("monthCar", thisMonthIdx); //当月统计数据集
			rootMap.put("sumCar", sumRptIdx);  //累计统计数据集
			rootMap.put("monthOth", otherMonth);  //当月统计数据集
			rootMap.put("sumOth", otherSum);   //累计统计数据集
			
			List<Map<String,Object>> orgs=baseService.loadAllSubOrgs();
			rootMap.put("orgDS", orgs); //所有项目部
			
			
			RptOuter.clearTemplateCache();
			RptOuter.write(request,response,rootMap,"erpKlSum.ftl");
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	

	private Map<String,CarConsume> builderIndex(List<CarConsume> sumData){
		Map<String,CarConsume> index=new HashMap<String,CarConsume>();
		CarConsume total=new CarConsume();
		for(CarConsume car : sumData){
			index.put(car.getBelongOrg(), car);
			addCarConsume(total,car);
		}
		
		index.put("合计", total);
		return index;
	}
	
	private void addCarConsume(CarConsume t,CarConsume d){
	
		//t.setCheckMile(CarRptUtils.addDouble(t.getCheckMile(),d.getCheckMile()));
		//t.setTravelMile(CarRptUtils.addDouble(t.getTravelMile(),d.getTravelMile()));
		t.setEngineCostKg(CarRptUtils.addDouble(t.getEngineCostKg(),d.getEngineCostKg()));
		t.setEngineWorkHour(CarRptUtils.addDouble(t.getEngineWorkHour(),d.getEngineWorkHour()));
	}
	
	

	/**
	 * 报表中需要显示汽油、柴油总量,freemarker表达式无法满足，后台程序进行计算
	 * @param sumData
	 * @return
	 */
	private Map<String,OtherConsumeWrapVO> builderOtherIndex(List<OtherConsume> sumData){
		Map<String,OtherConsumeWrapVO> index=new HashMap<String,OtherConsumeWrapVO>();
		OtherConsume total=new OtherConsume();
		for(OtherConsume other : sumData){
			index.put(other.getOrgId(), new OtherConsumeWrapVO(other));
			addOtherConsume(total,other);
		}
		
		index.put("合计", new OtherConsumeWrapVO(total));
		return index;
	}
	
	private void addOtherConsume(OtherConsume d1,OtherConsume d2){
		d1.setOilCountKg(CarRptUtils.addDouble(d1.getOilCountKg(), d2.getOilCountKg()));
		d1.setCoilCountKg(CarRptUtils.addDouble(d1.getCoilCountKg(), d2.getCoilCountKg()));
		d1.setOilErpCountKg(CarRptUtils.addDouble(d1.getOilErpCountKg(), d2.getOilErpCountKg()));
		d1.setCoilErpCountKg(CarRptUtils.addDouble(d1.getCoilErpCountKg(), d2.getCoilErpCountKg()));
		
		d1.setOilKlCountKg(CarRptUtils.addDouble(d1.getOilKlCountKg(), d2.getOilKlCountKg()));
		d1.setCoilKlCountKg(CarRptUtils.addDouble(d1.getCoilKlCountKg(), d2.getCoilKlCountKg()));
		
		d1.setOilKlMoney(CarRptUtils.addDouble(d1.getOilKlMoney(), d2.getOilKlMoney()));
		d1.setCoilKlMoney(CarRptUtils.addDouble(d1.getCoilKlMoney(), d2.getCoilKlMoney()));
	}
}
