package com.hb.proj.car.controller;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.car.service.ERPKLSumRptService;
import com.hb.proj.car.service.EnergySumRptService;
import com.hb.proj.car.service.MaterialERPService;
import com.hb.proj.model.EnergyWaterSumVO;
import com.hb.proj.model.EnergyWaterSumVOUtil;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.RptOuter;
import com.hb.xframework.util.MapUtils;
/**
 * 辽河分公司能源消耗台帐
 */
@Controller
@RequestMapping("/**/rpt")
public class EnergyCostAccountController {
	
	@Autowired
	private EnergySumRptService service;
	
	@Autowired
	private ERPKLSumRptService  othService;
	
	@Autowired
	private MaterialERPService materService;
	
	/**
	 * 考核里程、行驶里程、定额汽油、柴油取自单车油耗，其余取自数据校正表
	 * @param year
	 * @param request
	 * @param response
	 */
	@RequestMapping("/rptEnergyCost")
	public void rptEnergyCost(String year,HttpServletRequest request,HttpServletResponse response){
		try{
			if(StringUtils.isEmpty(year)){
				Calendar ca=Calendar.getInstance();
				year=String.valueOf(ca.get(Calendar.YEAR));
			}
			List<Map<String,Object>>  mths=new ArrayList<Map<String,Object>>(12);
			
			String[] mthNames={"一月","二月","三月","四月","五月","六月","七月","八月","九月","十月","十一月","十二月"};
			Map<String,Object> perMth=null;
			for(int i=1;i<=12;i++){
				perMth=MapUtils.build("month",mthNames[i-1],"monthNum",String.format("%02d",i));
				addSumVerticalFormula(i,perMth);
				mths.add(perMth);
			}
			
			//按月统计分组油水电气
			Map<String,EnergyWaterSumVO> energyWaters=EnergyWaterSumVOUtil.buildForCostAccount(service.erpMonthGrp(year), service.otherMonthGrp(year), service.wpgSumRpt(year), service.wpgCorrectSumRpt(year));
			
			//按月统计考核里程、行驶里程
			List<Map<String,Object>> miles=othService.rptCarMileAndQuota(year);
			
			//油耗(ERP累计+昆仑累计)、水电气 分月累计(上报月数据+月校正数据)  直接各月相加，不用数据校正总的累计
			
			
			
			String rptTitle=year+"年";
			Map<String,Object> rootMap=new HashMap<String,Object>();
			rootMap.put("rptTitle", rptTitle);
			rootMap.put("rptSubTitle", rptTitle+"12月30日");
			rootMap.put("months", mths); 
			rootMap.put("ds",energyWaters);
			
			rootMap.put("miles",builderMileIndex(miles));
			
			RptOuter.clearTemplateCache();
			RptOuter.write(request,response,rootMap,"energyCostAccount.ftl");
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	private Map<String,Map<String,Object>> builderMileIndex(List<Map<String,Object>> miles){
		if(miles==null||miles.size()==0){
			return null;
		}
		Map<String,Map<String,Object>> idx=new HashMap<String,Map<String,Object>>();
		for(Map<String,Object> m : miles){
			idx.put((String)m.get("rptDate"), m);
		}
		return idx;
		
	}
	
	
	
	
	//
	private void addSumVerticalFormula(int month,Map<String,Object> holder){
		String clm="CDEFGHIJKLMNOPQRSTU";
		char cn=0;
		for(int i=0,len=clm.length();i<len;i++){
			cn=clm.charAt(i);
			holder.put(String.valueOf(cn), builderFormual(cn,month));
		}	
		
		
		
	}
	
	private String builderFormual(char clm,int month){
		StringBuilder rtn=new StringBuilder();
		for(int i=1;i<=month;i++){
			rtn.append("+"+clm+(i*2-1));
		}
		return rtn.substring(1);
	}
}
