package com.hb.proj.car.controller;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.car.service.EnergySumRptService;
import com.hb.proj.car.service.MaterialERPService;
import com.hb.proj.car.service.YearSumCompareService;
import com.hb.proj.input.service.WorkloadService;
import com.hb.proj.model.CarConsumeMulRptVO;
import com.hb.proj.model.ERP;
import com.hb.proj.model.EnergyWaterCompareVO;
import com.hb.proj.model.EnergyWaterRatioCompareVO;
import com.hb.proj.model.OtherConsume;
import com.hb.proj.model.Workload;
import com.hb.proj.model.Wpg;
import com.hb.proj.model.WpgCorrect;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.RptOuter;
import com.hb.xframework.util.SessionThreadLocal;
/**
 * 用能用水累计同比
 */
@Controller
@RequestMapping("/**/rpt")
public class EnergyWaterCompareController {
	
	@Autowired
	private YearSumCompareService compService;
	
	@Autowired
	private MaterialERPService erpService;
	
	@Autowired
	private EnergySumRptService service;
	
	@Autowired
	private WorkloadService wkservice;
	
	/**
	 *  报表E7第二表：万元比值同比  中的建筑面积上报保存
	 *  目前是保存在workload表中，报表是按年统计，而workload表是按月存储记录的。所以建筑面积数据只存储一条，且固定在1月份。
	 *  便于任何月份都能查询到
	 *  2022.5.26  改为按查询月份分开存储
	 * @param year
	 * @param month
	 * @param builderArea
	 * @param prebuilderArea
	 * @param response
	 */
	@RequestMapping("/saveWYRatioCompare")
	public void saveWYRatioCompare(String year,Integer month,String builderArea,String prebuilderArea,HttpServletResponse response){
		if(StringUtils.isEmpty(year)||month==null){
			JsonOutUtils.returnError(response, "缺少年份或月份");
			return;
		}
		String monthstr=String.format("%02d",month);
		String rptMonth=year+"-"+monthstr;
		String preprtMonth=String.valueOf(Integer.parseInt(year)-1)+"-"+monthstr;
		String us=SessionThreadLocal.getSessionUser().getUserName();
		if(!StringUtils.isEmpty(builderArea)){
			erpService.saveBuilderArea(rptMonth, Double.parseDouble(builderArea), us);
		}
		if(!StringUtils.isEmpty(prebuilderArea)){
			erpService.saveBuilderArea(preprtMonth, Double.parseDouble(prebuilderArea), us);
		}
		JsonOutUtils.returnOk(response);
	}
	
	/**
	 * 报表E7第二表：万元比值同比
	 * @param year
	 * @param month  累计同比月份
	 * @param response
	 */
	@RequestMapping("/yearWYRatioCompare")
	public void yearWYRatioCompare(String year,Integer month,HttpServletRequest request,HttpServletResponse response){
		try{
			//String month="12";
			Calendar ca=Calendar.getInstance();
			if(month==null||month.intValue()<1||month.intValue()>12){
				month=ca.get(Calendar.MONTH)+1; //默认自然月当月
			}
			if(StringUtils.isEmpty(year)){
				year=String.valueOf(ca.get(Calendar.YEAR));
			}
			/*if(Integer.parseInt(year)==ca.get(Calendar.YEAR)){ //等于当前年份,根据当前日期决定用本月还是上月，否则为12月份
				
				month=String.format("%02d",RptMonthUtil.getFullMonthNum(ca));
			}*/
			String monthstr=String.format("%02d",month);
			String preyear=String.valueOf(Integer.parseInt(year)-1);
			String startMonth=year+"-01",endMonth=year+"-"+monthstr;
			String preStartMonth=preyear+"-01",preEndMonth=preyear+"-"+monthstr;
			
			EnergyWaterRatioCompareVO  ratioVO=builderVO(service,startMonth,endMonth);
			EnergyWaterRatioCompareVO  preratioVO=builderVO(service,preStartMonth,preEndMonth);
			
			Map<String,Object> rootMap=new HashMap<String,Object>();
			rootMap.put("ratio", ratioVO);
			rootMap.put("preratio", preratioVO);
			
			RptOuter.clearTemplateCache();
			RptOuter.write(request,response,rootMap,"yearWYRatioCompare.ftl");
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	
	
	private EnergyWaterRatioCompareVO  builderVO(EnergySumRptService service,String startMonth,String endMonth){
		ERP erp=service.erpMonthSum(startMonth,endMonth);
		OtherConsume  kl=service.otherMonthSum(startMonth, endMonth);
		erp=ERPKlSumUtil.erpPlusKlMonth(kl, erp);
		
		Wpg wpg=service.wpgMonthSum(startMonth,endMonth);
		WpgCorrect wpgcrt=service.wpgCorrectMonthSum(startMonth,endMonth);
		wpg=ERPKlSumUtil.processWpgSum(wpg,wpgcrt);
		CarConsumeMulRptVO consume=service.carOilMonthSum(startMonth, endMonth);
		//MonitorRemainOil remainOil=service.carOilPeriodDiff(startMonth, endMonth);  //2022.7.20 不考虑期末余油的计算
		//consume.adjust(remainOil);
		Workload  wk=service.workloadMonthSum(startMonth,endMonth);
		Workload  wkpart=service.getBuilderAreaYearLast(endMonth);
		wk.setBuilderArea(wkpart!=null?wkpart.getBuilderArea():0);  //建筑面积比较特殊，按年的统计报表中只需要统计月份之前最新的记录即可2022.9.14
		return new EnergyWaterRatioCompareVO(erp,wpg,wk,consume);
		
	}
	
	
	/**
	 * 用能用水年度对比
	 * @param year
	 * @param month  累计同比月份
	 * @param request
	 * @param response
	 */
	@RequestMapping("/yearSumCompare")
	public void yearSumCompare(String year,Integer month,HttpServletRequest request,HttpServletResponse response){
		try{
			//String month="12";
			Calendar ca=Calendar.getInstance();
			if(month==null||month.intValue()<1||month.intValue()>12){
				month=ca.get(Calendar.MONTH)+1; //默认自然月当月
			}
			if(StringUtils.isEmpty(year)){
				year=String.valueOf(ca.get(Calendar.YEAR));
			}
			/*if(Integer.parseInt(year)==ca.get(Calendar.YEAR)){ //等于当前年份,月份就应该为当前月份，否则为12月份
				month=String.format("%02d",RptMonthUtil.getFullMonthNum(ca));
			}*/
			String monthstr=String.format("%02d",month);
			String preyear=String.valueOf(Integer.parseInt(year)-1);
			String startMonth=year+"-01",endMonth=year+"-"+monthstr;
			String preStartMonth=preyear+"-01",preEndMonth=preyear+"-"+monthstr;
			
			EnergyWaterCompareVO  yearSum=builderCompareVO(service,startMonth,endMonth);
		
			EnergyWaterCompareVO preYearSum=builderCompareVO(service,preStartMonth,preEndMonth);
			
			yearSum.setComparedValue(preYearSum);
			
			String rptTitle=year+"年";
			Map<String,Object> rootMap=new HashMap<String,Object>();
			rootMap.put("rptTitle", rptTitle);
			rootMap.put("yearSum", yearSum);
			rootMap.put("preYearSum", preYearSum);
			
			
			RptOuter.clearTemplateCache();
			RptOuter.write(request,response,rootMap,"yearSumCompare.ftl");
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	
	private EnergyWaterCompareVO  builderCompareVO(EnergySumRptService service,String startMonth,String endMonth){
		ERP erp=service.erpMonthSum(startMonth,endMonth);
		OtherConsume  kl=service.otherMonthSum(startMonth, endMonth);
		erp=ERPKlSumUtil.erpPlusKlMonth(kl, erp);
		
		Wpg wpg=service.wpgMonthSum(startMonth,endMonth);
		WpgCorrect wpgcrt=service.wpgCorrectMonthSum(startMonth,endMonth);
		wpg=ERPKlSumUtil.processWpgSum(wpg,wpgcrt);
		
		return new EnergyWaterCompareVO(erp,wpg);
		
	}
	
	
	
}
