package com.hb.proj.car.controller;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.hb.proj.car.service.EnergySumRptService;
import com.hb.proj.car.service.MaterialERPService;
import com.hb.proj.input.service.WorkloadService;
import com.hb.proj.model.EnergyWaterSumTotalVal;
import com.hb.proj.model.EnergyWaterSumVO;
import com.hb.proj.model.EnergyWaterSumVOUtil;
import com.hb.proj.model.Workload;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.MySessionUser;
import com.hb.proj.utils.RptOuter;
import com.hb.xframework.util.MapUtils;
import com.hb.xframework.util.SessionThreadLocal;
/**
 * 用能用水统计汇总
 */
@Controller
@RequestMapping("/**/rpt")
public class EnergyWaterSumController {
	
	@Autowired
	private EnergySumRptService service;
	
	@Autowired
	private WorkloadService wkservice;
	
	@Autowired
	private MaterialERPService erpService;
	
	
	
	/**
	 * 各月水电气=上报数据+当月校正数据=数据校正中的水电气统计中的总合计
	 * 各月汽油、柴油=部门上报的数据（t_other_consume）+当月数据校正+物资ERP填报当月数据=数据校正中表二中的当月数据
	 * workload,发电机 年累同比按同期月份计算（即是当年按实际月份累计同比）
	 * @param year
	 * @param month  累计同比月份
	 * @param request
	 * @param response
	 */
	@RequestMapping("/rptEnergyWaterSum")
	public void rptEnergyWaterSum(String year,Integer month,HttpServletRequest request,HttpServletResponse response){
		try{
			//int endMonth=12;
			Calendar ca=Calendar.getInstance();
			if(month==null||month.intValue()<1||month.intValue()>12){
				month=ca.get(Calendar.MONTH)+1; //默认自然月当月
			}
			if(StringUtils.isEmpty(year)){
				year=String.valueOf(ca.get(Calendar.YEAR));
			}
			/*else if(Integer.parseInt(year)==ca.get(Calendar.YEAR)){  //等于当前年份,月份就使用足月的月份
				endMonth=RptMonthUtil.getFullMonthNum(ca);
			}*/
			List<Map<String,Object>>  mths=new ArrayList<Map<String,Object>>(12);
			
			String[] mthNames={"一月","二月","三月","四月","五月","六月","七月","八月","九月","十月","十一月","十二月"};
			Map<String,Object> perMth=null;
			for(int i=1;i<=12;i++){
				perMth=MapUtils.build("month",mthNames[i-1],"monthNum",String.format("%02d",i));
				mths.add(perMth);
			}
			String preyear=String.valueOf(Integer.parseInt(year)-1);
			
			EnergyWaterSumTotalVal  totalVal=new EnergyWaterSumTotalVal(month),pretotalVal=new EnergyWaterSumTotalVal(month);
			
			Map<String,EnergyWaterSumVO> voIndex=EnergyWaterSumVOUtil.build(service.erpMonthGrp(year), service.otherMonthGrp(year), service.wpgSumRpt(year), service.wpgCorrectSumRpt(year), service.workloadSumRpt(year),service.carOilSumRpt(year),totalVal);
			Map<String,EnergyWaterSumVO> preVoIndex=EnergyWaterSumVOUtil.build(service.erpMonthGrp(preyear), service.otherMonthGrp(preyear), service.wpgSumRpt(preyear), service.wpgCorrectSumRpt(preyear), service.workloadSumRpt(preyear),service.carOilSumRpt(preyear),pretotalVal);
			EnergyWaterSumVOUtil.caculateCompare(voIndex, preVoIndex);
			preVoIndex.clear();
			preVoIndex=null;
			
			totalVal.caculateCompare(pretotalVal);
			pretotalVal=null;
			
			String rptTitle=year+"年";
			Map<String,Object> rootMap=new HashMap<String,Object>();
			rootMap.put("rptTitle", rptTitle);
			rootMap.put("year", year);
			rootMap.put("months", mths); 
			rootMap.put("ds", voIndex); 
			rootMap.put("sums",addSumFormula()); 
			rootMap.put("dssum", totalVal); 
			
			
			RptOuter.clearTemplateCache();
			RptOuter.write(request,response,rootMap,"energyWaterSum.ftl");
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	
	
	
	
	
	
	private Map<String,String> addSumFormula(){
		Map<String,Object> sorts=MapUtils.build("ngas",1,"power",2,"oil",3,"coil",4,"lgas",5,"coal",6,"water",7,"well",8,"industry",9,"added",10);
		int clm=0,clm2=0;
		String f1="",f2="";
		Map<String,String> formulas=new HashMap<String,String>();
		for(String s : sorts.keySet()){
			f1="";
			f2="";
			for(int i=1;i<=12;i++){
				clm=3+(i-1)*3+1;  //money
				clm2=3+(i-1)*3+2; //count
				f1+="+"+getChar(clm)+sorts.get(s);
				f2+="+"+getChar(clm2)+sorts.get(s);
			}
			formulas.put(s+"Money", f1.substring(1));
			formulas.put(s+"Count", f2.substring(1));
		}
		return formulas;
		
	}
	
	private String getChar(int val){
		if(val==26){
			return "Z";
		}
		int h=val/26;
		int l=val%26;
		return (h>0?String.valueOf((char)(h+64)):"")+(l>0?String.valueOf((char)(l+64)):"") ;
	}
	
	
	/**
	 * 通过报表上报工作量和工业产值
	 * @param workload
	 * @param response
	 */
	@RequestMapping("/saveWorkload")
	public void saveWorkload(String jsonDatas,String year,HttpServletResponse response){
		try{
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			if(su==null){  
				JsonOutUtils.returnError(response, "缺少登录信息,请登录后再试");
				return;
			}
			
			List<Workload> workload=JSON.parseObject(jsonDatas,new TypeReference<List<Workload>>() {});
			wkservice.saveWorkloadFromRpt(workload,year);
			JsonOutUtils.returnOk(response);
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
}
