package com.hb.proj.car.controller;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.car.service.EnergySumRptService;
import com.hb.proj.car.service.MaterialERPService;
import com.hb.proj.car.service.YearSumCompareService;
import com.hb.proj.model.CarConsumeMulRptVO;
import com.hb.proj.model.ERP;
import com.hb.proj.model.EnergyWaterCompareVO;
import com.hb.proj.model.EnergyWaterSumVO;
import com.hb.proj.model.EnergyWaterSumVOUtil;
import com.hb.proj.model.OtherConsume;
import com.hb.proj.model.Workload;
import com.hb.proj.model.Wpg;
import com.hb.proj.model.WpgCorrect;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.RptOuter;
import com.hb.xframework.util.DateUtil;
import com.hb.xframework.util.MapUtils;

@Controller
@RequestMapping("/**/home/rpt")
public class HomeRptController {

	@Autowired
	private EnergySumRptService  service;
	
	@Autowired
	private YearSumCompareService yearService;
	
	@Autowired
	private MaterialERPService crtService;
	
	/**
	 * 首页工作量统计表
	 * 2022.5.26 增加月份条件，统计截至到指定月份
	 * @param request
	 * @param response
	 */
	@RequestMapping("/workloadRpt")
	public void workloadRpt(String year,Integer month,HttpServletRequest request,HttpServletResponse response){
		Calendar ca=Calendar.getInstance();
		if(month==null||month.intValue()<1||month.intValue()>12){
			month=ca.get(Calendar.MONTH)+1; //默认自然月当月
		}
		if(StringUtils.isEmpty(year)){
			year=String.valueOf(ca.get(Calendar.YEAR));
		}
		
		String endMonth=String.format("%02d",month);
		
		String preyear=String.valueOf(Integer.parseInt(year)-1);
		List<Map<String,Object>>  mths=new ArrayList<Map<String,Object>>(12);
		String[] mthNames={"一月","二月","三月","四月","五月","六月","七月","八月","九月","十月","十一月","十二月"};
		Map<String,Object> perMth=null;
		for(int i=1;i<=12;i++){
			perMth=MapUtils.build("month",mthNames[i-1],"monthNum",String.format("%02d",i));
			mths.add(perMth);
		}
		
		Map<String,Object> rootMap=new HashMap<String,Object>();
		rootMap.put("months", mths); 
		rootMap.put("workload", buildIndex(service.workloadSumRpt(year+"-01",year+"-"+endMonth))); 
		rootMap.put("preworkload", buildIndex(service.workloadSumRpt(preyear+"-01",preyear+"-"+endMonth))); 
		RptOuter.clearTemplateCache();
		RptOuter.write(request,response,rootMap,"homeWorkload.ftl");
	}
	
	private Map<String,Workload> buildIndex(List<Workload> workloads){
		Map<String,Workload>  index=new HashMap<String,Workload>();
		for(Workload wd : workloads){
			index.put(DateUtil.format(wd.getRptMonth(), "MM"), wd);
		}
		return index;
	}
	
	/**
	 * 首页工作量图
	 * @param response
	 */
	@RequestMapping("/workloadRptChart")
	public void workloadRptChart(String year,HttpServletResponse response){
		if(StringUtils.isEmpty(year)){
			Calendar ca=Calendar.getInstance();
			year=String.valueOf(ca.get(Calendar.YEAR));
		}
		String preyear=String.valueOf(Integer.parseInt(year)-1);
		
		List<Integer> wc=getWellCounts(service.workloadSumRpt(year));
		List<Integer> prewc=getWellCounts(service.workloadSumRpt(preyear));
		
		Map<String,Object> rootMap=new HashMap<String,Object>();
		rootMap.put("wc", wc); 
		rootMap.put("prewc", prewc); 
		
		JsonOutUtils.returnOkWithData(response, rootMap);
	}
	
	private List<Integer>  getWellCounts(List<Workload> workloads){
		List<Integer> wellCounts=Arrays.asList(0,0,0,0,0,0,0,0,0,0,0,0);
		int idx=0;
		for(Workload wd : workloads){
			 idx=Integer.parseInt(DateUtil.format(wd.getRptMonth(), "MM"))-1;
			 if(wd.getWellCount()!=null){
				 wellCounts.set(idx, wd.getWellCount());
			 }
			
		}
		return wellCounts;
	}
	
	/**
	 * 用能用水耗量同比
	 * @param year
	 * @param month
	 * @param type
	 * @param request
	 * @param response
	 */
	@RequestMapping("/energyCompareRpt")
	public void energyCompareRpt(String year,Integer month,String type,HttpServletRequest request,HttpServletResponse response){
		try{
			//String month="12";
			Calendar ca=Calendar.getInstance();
			if(month==null||month.intValue()<1||month.intValue()>12){
				month=ca.get(Calendar.MONTH)+1; //默认自然月当月
			}
			if(StringUtils.isEmpty(year)){
				year=String.valueOf(ca.get(Calendar.YEAR));
			}
			/*if(Integer.parseInt(year)==ca.get(Calendar.YEAR)){ //等于当前年份,月份就应该为当前月份，否则为12月份
				month=String.format("%02d",RptMonthUtil.getFullMonthNum(ca));
				
			}*/
			String monthstr=String.format("%02d",month);
			String preyear=String.valueOf(Integer.parseInt(year)-1);
			String startMonth=year+"-01",endMonth=year+"-"+monthstr;
			String preStartMonth=preyear+"-01",preEndMonth=preyear+"-"+monthstr;
			
			
			
			Map<String,Object> rootMap=new HashMap<String,Object>();
			rootMap.put("yearSum", builderCompareVO(service,startMonth,endMonth));
			rootMap.put("preYearSum", builderCompareVO(service,preStartMonth,preEndMonth));
			if("grid".equals(type)){ //统计表格
				RptOuter.clearTemplateCache();
				RptOuter.write(request,response,rootMap,"homeEnergyCompare.ftl");
			}
			else if("chart".equals(type)){  //统计图
				JsonOutUtils.returnOkWithData(response, rootMap);
			}
			
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	
	/**
	 * 能耗费用占比分析  
	 * @param year
	 * @param month 累计同比月份
	 * @param type
	 * @param request
	 * @param response
	 */
	@RequestMapping("/costRatioRpt")
	public void costRatioRpt(String year,Integer month,String type,HttpServletRequest request,HttpServletResponse response){
		try{
			//String month="12";
			Calendar ca=Calendar.getInstance();
			if(month==null||month.intValue()<1||month.intValue()>12){
				month=ca.get(Calendar.MONTH)+1; //默认自然月当月
			}
			if(StringUtils.isEmpty(year)){
				year=String.valueOf(ca.get(Calendar.YEAR));
			}
			/*if(Integer.parseInt(year)==ca.get(Calendar.YEAR)){ //等于当前年份,使用足月的月份
				month=String.format("%02d",RptMonthUtil.getFullMonthNum(ca));
			}*/
			String monthstr=String.format("%02d",month);
			String preyear=String.valueOf(Integer.parseInt(year)-1);
			String startMonth=year+"-01",endMonth=year+"-"+monthstr;
			String preStartMonth=preyear+"-01",preEndMonth=preyear+"-"+monthstr;
			
			
			
			Map<String,Object> rootMap=new HashMap<String,Object>();
			rootMap.put("yearSum", builderCompareVO(service,startMonth,endMonth));
			rootMap.put("preYearSum", builderCompareVO(service,preStartMonth,preEndMonth));
			if("grid".equals(type)){ //统计表格
				RptOuter.clearTemplateCache();
				RptOuter.write(request,response,rootMap,"homeCostRatio.ftl");
			}
			else if("chart".equals(type)){  //统计图
				JsonOutUtils.returnOkWithData(response, rootMap);
			}
			
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	/**
	 * 能耗折算标准煤占比分析
	 * @param year
	 * @param request
	 * @param response
	 */
	@RequestMapping("/coalRatioRpt")
	public void coalRatioRpt(String year,String type,HttpServletRequest request,HttpServletResponse response){
		try{
			if(StringUtils.isEmpty(year)){
				Calendar ca=Calendar.getInstance();
				year=String.valueOf(ca.get(Calendar.YEAR));
			}
			String startMonth=year+"-01";
			String endMonth=year+"-12";
			
		    Map<String,Object> rootMap=new HashMap<String,Object>();
			rootMap.put("yearSum", builderCompareVO(service,startMonth,endMonth));
			if("grid".equals(type)){ //统计表格
				RptOuter.clearTemplateCache();
				RptOuter.write(request,response,rootMap,"homeCoalRatio.ftl");
			}
			else if("chart".equals(type)){  //统计图
				JsonOutUtils.returnOkWithData(response, rootMap);
			}
			
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	
	private EnergyWaterCompareVO  builderCompareVO(EnergySumRptService service,String startMonth,String endMonth){
		ERP erp=service.erpMonthSum(startMonth,endMonth);
		OtherConsume  kl=service.otherMonthSum(startMonth, endMonth);
		erp=ERPKlSumUtil.erpPlusKlMonth(kl, erp);
		
		Wpg wpg=service.wpgMonthSum(startMonth,endMonth);
		WpgCorrect wpgcrt=service.wpgCorrectMonthSum(startMonth,endMonth);
		wpg=ERPKlSumUtil.processWpgSum(wpg,wpgcrt);
		
		return new EnergyWaterCompareVO(erp,wpg);
		
	}
	
	
	/**
	 * 首页按月统计能源，水消耗量,2022.7.18  增加新指标计算，2022.7.20 不考虑期末余油的计算
	 * @param request
	 * @param response
	 */
	@RequestMapping("/energyMonthCost")
	public void energyMonthCost(String year,String type,HttpServletRequest request,HttpServletResponse response){
		if(StringUtils.isEmpty(year)){
			Calendar ca=Calendar.getInstance();
			year=String.valueOf(ca.get(Calendar.YEAR));
		}
		Map<String,EnergyWaterSumVO> voIndex=EnergyWaterSumVOUtil.build(service.erpMonthGrp(year), service.otherMonthGrp(year), service.wpgSumRpt(year), service.wpgCorrectSumRpt(year), service.workloadSumRpt(year),null,null);
		List<CarConsumeMulRptVO> indConsumes=service.carOilMulRptMonthGrp(year);
		//List<MonitorRemainOil> remainOils=service.carOilPeriodDiffMonthGrp(year);
		EnergyWaterSumVOUtil.attachForMulRpt(voIndex, indConsumes,null);
		
		if("chart".equals(type)){
			//Map<String,EnergyWaterSumVO> voIndex=EnergyWaterSumVOUtil.build(service.erpMonthGrp(year), service.otherMonthGrp(year), service.wpgSumRpt(year), service.wpgCorrectSumRpt(year), service.workloadSumRpt(year),null,null);
			JsonOutUtils.returnOkWithData(response, voIndex);
			return;
		}
		
		List<Map<String,Object>>  mths=new ArrayList<Map<String,Object>>(12);
		String[] mthNames={"一月","二月","三月","四月","五月","六月","七月","八月","九月","十月","十一月","十二月"};
		Map<String,Object> perMth=null;
		for(int i=1;i<=12;i++){
			perMth=MapUtils.build("month",mthNames[i-1],"monthNum",String.format("%02d",i));
			mths.add(perMth);
		}
		//Map<String,EnergyWaterSumVO> voIndex=EnergyWaterSumVOUtil.build(service.erpMonthGrp(year), service.otherMonthGrp(year), service.wpgSumRpt(year), service.wpgCorrectSumRpt(year), service.workloadSumRpt(year),null,null);
		
		
		Map<String,Object> rootMap=new HashMap<String,Object>();
		rootMap.put("months", mths); 
		rootMap.put("ds", voIndex); 
		
		
		RptOuter.clearTemplateCache();
		RptOuter.write(request,response,rootMap,"homeEnergyMonthCost.ftl");
	}
	
}
