package com.hb.proj.car.controller;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.hb.proj.car.quota.QuotaStandardSupport;
import com.hb.proj.car.service.MaterialERPService;
import com.hb.proj.car.service.YearSumCompareService;
import com.hb.proj.model.ERP;
import com.hb.proj.model.OtherConsume;
import com.hb.proj.model.WorkloadPO;
import com.hb.proj.model.WpgCorrect;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.MySessionUser;
import com.hb.proj.utils.RptOuter;
import com.hb.xframework.util.DateUtil;
import com.hb.xframework.util.SessionThreadLocal;

@Controller
@RequestMapping("/**/erp")
public class MaterialERPController {
	
	@Autowired
	private MaterialERPService service;
	
	@Autowired
	private YearSumCompareService compservice;

	@RequestMapping("/load")
	public void loadERP(String rptMonth,HttpServletRequest request,HttpServletResponse response){
		try{
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			if(su==null){  
				JsonOutUtils.returnError(response, "缺少登录信息");
				return;
			}
			if(StringUtils.isEmpty(rptMonth)){
				rptMonth=DateUtil.format(new Date(), "yyyy-MM");
			}
			String[] mths=rptMonth.split("-");
			String startMonth=mths[0]+"-01";
			//包含了当月和累计数据,不需要单独取累计数据
			ERP  erp=service.rptERPSum(rptMonth,rptMonth);
			
			Map<String,Object> rootMap=new HashMap<String,Object>();
			rootMap.put("rptMonth", rptMonth);
			rootMap.put("ratio",QuotaStandardSupport.getConfig());
			rootMap.put("erp", erp);
			rootMap.put("wpg", service.rptWpgSum(rptMonth,rptMonth));
			rootMap.put("wpgCorrect", service.rptWpgCorrectSum(rptMonth,rptMonth));
			
			rootMap.put("workload", service.loadWorkloadByMonth(rptMonth));
			
			//工业产值截止到上月累计
			String preMonth=startMonth;
			int mthNum=Integer.parseInt(mths[1]);
			if(mthNum>2){
				preMonth=mths[0]+"-"+String.format("%02d", mthNum-1);
			}
			
			rootMap.put("workloadSum", service.loadWorkloadMonthSum(startMonth,preMonth));
			
			
			
			OtherConsume kl=service.rptOtherConsume(rptMonth, rptMonth);
			ERP erpKl=ERPKlSumUtil.erpPlusKlMonth(kl,erp);
			rootMap.put("erpKl",erpKl);
			
			OtherConsume  klSum=service.rptOtherConsume(startMonth, rptMonth);
			ERP erpKlSum=ERPKlSumUtil.erpPlusKlSum(erp, klSum);
			rootMap.put("erpKlSum",erpKlSum);
			
			
			//9.24 第二个表的运输用油 = 柴油合计-当月其他用油
			//运输用柴油=当月柴油（ERP+昆仑卡）-当月其他用油（所有项目部当月合计）
			Double trasport=null;
			if(erpKl!=null&&erpKl.getCoilCount()!=null){
				double othcoil=0;
				if(kl!=null&&kl.getCoilCountKg()!=null){
					othcoil=kl.getCoilCountKg().doubleValue();
				}
				trasport=erpKl.getCoilCount().doubleValue()-othcoil;
				
			}
			rootMap.put("trasportOil", trasport);
			
			//累计运输用柴油
			trasport=null;
			if(erpKlSum!=null&&erpKlSum.getCoilCountSum()!=null){
				double othcoil=0;
				if(klSum!=null&&klSum.getCoilCountKg()!=null){
					othcoil=klSum.getCoilCountKg().doubleValue();
				}
				trasport=erpKlSum.getCoilCountSum().doubleValue()-othcoil;
				
			}
			rootMap.put("trasportSumOil", trasport);
			
			
			
			RptOuter.clearTemplateCache();
			RptOuter.write(request,response,rootMap,"materialERP.ftl");
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	
	
	
	
	
	
	
	
	
	
	/**
	 * erp+昆仑卡 数据校正 上报
	 * @param rptMonth
	 * @param erp
	 * @param wpg
	 * @param request
	 * @param response
	 */
	@RequestMapping("/save")
	public void save(String rptMonth,String erp,String wpg,String workload,HttpServletRequest request,HttpServletResponse response){
		if(StringUtils.isEmpty(rptMonth)){
			JsonOutUtils.returnError(response, "缺少月份");
			return;
		}
		MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
		//物资erp数据
		//Map<String,Object> erpObj=JsonParamHandler.getMapParam(erp);
		/*String oilRatio=(String)erpObj.remove("oilRatio");
		String oilRatio95=(String)erpObj.remove("oilRatio95");
		String coilRatio=(String)erpObj.remove("coilRatio");*/
		
		ERP  erpPO=JSON.parseObject(erp,new TypeReference<ERP>() {});
		erpPO.setRptMonth(DateUtil.parse(rptMonth+"-01"));
		erpPO.setModifier(su!=null?su.getUserName():"");
		
		service.saveERPData(erpPO);
		
		//水电气数据
		WpgCorrect wpgcrt=JSON.parseObject(wpg,new TypeReference<WpgCorrect>() {});
		wpgcrt.setRptMonth(DateUtil.parse(rptMonth+"-01"));
		wpgcrt.setModifier(su!=null?su.getUserName():"");
		service.saveWpgCorrect(wpgcrt);
		
		//产值
		WorkloadPO wkload=JSON.parseObject(workload,new TypeReference<WorkloadPO>() {});
		wkload.setRptMonth(DateUtil.parse(rptMonth+"-01"));
		wkload.setModifier(su!=null?su.getUserName():"");
		service.saveWorkload(wkload);
		
		
		//燃油折算系数(在此模块中也提供了燃油折算系数的设置功能) 2022.4.29 取消此处的折算系统设置
		/*if(DataUtils.isDouble(oilRatio)&&DataUtils.isDouble(oilRatio95)&&DataUtils.isDouble(coilRatio)){
			QuotaConfig config=QuotaStandardSupport.getConfig();
			config.setOilRatio(Double.parseDouble(oilRatio));
			config.setOilRatio95(Double.parseDouble(oilRatio95));
			config.setCoilRatio(Double.parseDouble(coilRatio));
			WebConfig  webConfig=ApplicationContextUtils.getBean("webConfig", WebConfig.class);
			webConfig.save(config);
		}*/
		
		//add 2020.12.17检查物资ERp汽、柴油吨数和ERP昆仑卡汇总表当月合计数据的差额,并提示到前端
		String msg="";
		OtherConsume oth=compservice.oilSumRpt(rptMonth,rptMonth);
		if(oth!=null){
			Double oilOthERP=oth.getOilErpCountKg(); //已转化为吨
			Double coilOthERP=oth.getCoilErpCountKg(); //已转化为吨
			Double diff=doubleDiff(erpPO.getOilCount(),oilOthERP);
			
			if(diff!=null){
				msg+=String.format("汽油吨数相差%.3f;", diff);
			}
			diff=doubleDiff(erpPO.getCoilCount(),coilOthERP);
			if(diff!=null){
				msg+=String.format("柴油吨数相差%.3f;", diff);
			}
		}
		if(StringUtils.isEmpty(msg)){
			JsonOutUtils.returnOk(response);
		}
		else{
			JsonOutUtils.returnOkWithData(response, "物资站ERP当月数据与ERP昆仑卡汇总表："+msg);
		}
		
		
	}
	
	private Double doubleDiff(Double d1,Double d2){
		Double diff=0.0;
		
		if(d1!=null&&d2!=null){
			diff=Math.abs(d1.doubleValue()-d2.doubleValue());
		}
		else if(d1==null){
			diff=d2;
		}
		else if(d2==null){
			diff=d1;
		}
		if(diff!=null&&!Double.isNaN(diff)&&diff.doubleValue()>0){
			return diff;
		}
		else{
			return null;
		}
		
	}
}
