package com.hb.proj.car.controller;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.car.service.OilBoxService;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.LayGridJsonUtils;
import com.hb.proj.utils.MySessionUser;
import com.hb.proj.utils.RequestUtils;
import com.hb.xframework.dao.util.PageModel;
import com.hb.xframework.util.MapUtils;
import com.hb.xframework.util.SessionThreadLocal;
import com.hb.xframework.util.SessionUser;

/**
 * 油箱标定管理
 * 标定基本算法：
 * 1. 底部，顶部不规则部分有一定高度，单独计算，其他部分按立方体计算（除去壁厚）
 * 2. 一般油不会用完才加油，也不会加满。顶部暂不考虑，只考虑底部
 * 3.底部高度为h，容积差额为四个圆角造成的。(其它不规格油箱暂不考虑)
 * 4.高度>h时，只需要减去四角的差额；高度<h时，先按立方体计算，减去以液位高为半径的圆角差额，再减一个以液位高为边长的正立方体
 */
@Controller
@RequestMapping("/**/oilbox")
public class OilBoxController {

	@Autowired
	private OilBoxService service;
	
	
	
	@RequestMapping("/query")
	public void query(HttpServletRequest request,Integer page,Integer limit,HttpServletResponse response){
		try{
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			if(su==null){  //测试暂时不用
				LayGridJsonUtils.error(response, "缺少登录信息");
				return;
		    }
			Map<String,Object> args=RequestUtils.getParams(request);
			PageModel<Map<String,Object>> pagedData=service.queryOilBox(args,page, limit);
			LayGridJsonUtils.wirte(response, pagedData.getData(),pagedData.getTotalRow(),"yyyy-MM-dd");
		}
		catch(Exception e){
			e.printStackTrace();
			LayGridJsonUtils.error(response, "服务出现错误");
		}
		
	}
	
	@RequestMapping("/get")
	public void get(String recordId,HttpServletResponse response) throws Exception{
		Map<String,Object> mapping=service.getOilBox(recordId);
		JsonOutUtils.returnOkWithData(response, mapping);
	}
	
	@RequestMapping("/add")
	public void add(HttpServletRequest request,HttpServletResponse response){
	   try{
		   Map<String,Object>  oilBox=RequestUtils.getParams(request);
		   MapUtils.blankValToNull(oilBox);
		  
		   
		   SessionUser su=SessionThreadLocal.getSessionUser();
		   oilBox.put("modifier",  su!=null?su.getUserName():"unknow");
		   service.addOilBox(oilBox);
		   
		   JsonOutUtils.returnOk(response);
		  }
	   catch(Exception e){
		   e.printStackTrace();
		   JsonOutUtils.returnError(response, "服务出现错误");
	   }
		
	}
	
	@RequestMapping("/update")
	public void update(HttpServletRequest request,HttpServletResponse response){
	   try{
		   Map<String,Object>  oilBox=RequestUtils.getParams(request);
		   MapUtils.blankValToNull(oilBox);
		  
		   SessionUser su=SessionThreadLocal.getSessionUser();
		   oilBox.put("modifier", su!=null?su.getUserName():"unknow");
		   service.updateOilBox(oilBox);
		   JsonOutUtils.returnOk(response);
	   }
	   catch(Exception e){
		   e.printStackTrace();
		   JsonOutUtils.returnError(response, "服务出现错误");
	   }
		
	}
	
	@RequestMapping("/delete")
	public void delete(String recordId,HttpServletResponse response){
		try{
			   service.deleteOilBox(recordId);
			   JsonOutUtils.returnOk(response);
		   }
		   catch(Exception e){
			   e.printStackTrace();
			   JsonOutUtils.returnError(response, "服务出现错误");
		   }
	}
	
	
}
