package com.hb.proj.car.controller;

import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.car.service.OilMeterService;
import com.hb.proj.utils.DataAuthUtils;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.LayGridJsonUtils;
import com.hb.proj.utils.MySessionUser;
import com.hb.proj.utils.OkhttpUtils;
import com.hb.proj.utils.RequestUtils;
import com.hb.xframework.dao.util.PageModel;
import com.hb.xframework.util.DateUtil;
import com.hb.xframework.util.JsonParamHandler;
import com.hb.xframework.util.MapUtils;
import com.hb.xframework.util.SessionThreadLocal;
import com.hb.xframework.util.SessionUser;
import com.hb.xframework.util.SysConfigUtil;

/**
 * 油量计管理
 */
@Controller
@RequestMapping("/**/meter")
public class OilMeterController {

	@Autowired
	private OilMeterService service;
	
	/**
	 * 供项目部通过车辆查询油量数据
	 * @param request
	 * @param page
	 * @param limit
	 * @param response
	 */
	@RequestMapping("/queryFromCar")
	public void queryFromCar(HttpServletRequest request,Integer page,Integer limit,HttpServletResponse response){
		try{
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			if(su==null){  //测试暂时不用
				LayGridJsonUtils.error(response, "缺少登录信息");
				return;
		    }
			Map<String,Object> args=RequestUtils.getParams(request);
			PageModel<Map<String,Object>> pagedData=service.queryOilMeterFromCar(args,DataAuthUtils.getAuthOrgAssistCode(su),page, limit);
			LayGridJsonUtils.wirte(response, pagedData.getData(),pagedData.getTotalRow(),"yyyy-MM-dd");
		}
		catch(Exception e){
			e.printStackTrace();
			LayGridJsonUtils.error(response, "服务出现错误");
		}
		
	}
	
	/**
	 * 油量计管理查询
	 * @param request
	 * @param page
	 * @param limit
	 * @param response
	 */
	@RequestMapping("/query")
	public void query(HttpServletRequest request,Integer page,Integer limit,HttpServletResponse response){
		try{
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			if(su==null){  //测试暂时不用
				LayGridJsonUtils.error(response, "缺少登录信息");
				return;
		    }
			Map<String,Object> args=RequestUtils.getParams(request);
			args.put("showAll", su.isSysAdmin()?"1":null); //系统管理员显示全部
			PageModel<Map<String,Object>> pagedData=service.queryOilMeter(args,DataAuthUtils.getAuthOrgAssistCode(su),page, limit);
			LayGridJsonUtils.wirte(response, pagedData.getData(),pagedData.getTotalRow(),"yyyy-MM-dd");
		}
		catch(Exception e){
			e.printStackTrace();
			LayGridJsonUtils.error(response, "服务出现错误");
		}
		
	}
	
	@RequestMapping("/get")
	public void get(String recordId,HttpServletResponse response) throws Exception{
		Map<String,Object> mapping=service.getOilMeter(recordId);
		if(mapping!=null&&mapping.get("installDate")!=null){
			mapping.put("installDate", DateUtil.format((Date)mapping.get("installDate"), "yyyy-MM-dd"));
		}
		JsonOutUtils.returnOkWithData(response, mapping);
	}
	
	
	
	@RequestMapping("/add")
	public void add(HttpServletRequest request,HttpServletResponse response){
	   try{
		   Map<String,Object>  oilMeter=RequestUtils.getParams(request);
		   MapUtils.blankValToNull(oilMeter);
		  
		   String meterNum=(String)oilMeter.get("meterNum");
		   if(StringUtils.isNotEmpty(meterNum)&&service.existsMeterNum(meterNum,null)){
			   JsonOutUtils.returnError(response, "该油量计识别码已在其它车辆使用，请更换");
			   return ;
		   }
		   SessionUser su=SessionThreadLocal.getSessionUser();
		   oilMeter.put("modifier",  su!=null?su.getUserName():"unknow");
		   service.addOilMeter(oilMeter);
		   
		   JsonOutUtils.returnOk(response);
		  }
	   catch(Exception e){
		   e.printStackTrace();
		   JsonOutUtils.returnError(response, "服务出现错误");
	   }
		
	}
	
	@RequestMapping("/update")
	public void update(HttpServletRequest request,HttpServletResponse response){
	   try{
		   Map<String,Object>  oilMeter=RequestUtils.getParams(request);
		   MapUtils.blankValToNull(oilMeter);
		  
		   
		   String meterNum=(String)oilMeter.get("meterNum");
		   if(StringUtils.isNotEmpty(meterNum)&&service.existsMeterNum(meterNum,(String)oilMeter.get("recordId"))){
			   JsonOutUtils.returnError(response, "该油量计识别码已在其它车辆使用，请更换");
			   return ;
		   }
		   SessionUser su=SessionThreadLocal.getSessionUser();
		   oilMeter.put("modifier", su!=null?su.getUserName():"unknow");
		   service.updateOilMeter(oilMeter);
		   JsonOutUtils.returnOk(response);
	   }
	   catch(Exception e){
		   e.printStackTrace();
		   JsonOutUtils.returnError(response, "服务出现错误");
	   }
		
	}
	
	@RequestMapping("/delete")
	public void delete(String recordId,HttpServletResponse response){
		try{
			   service.deleteOilMeter(recordId);
			   JsonOutUtils.returnOk(response);
		   }
		   catch(Exception e){
			   e.printStackTrace();
			   JsonOutUtils.returnError(response, "服务出现错误");
		   }
	}
	
	@RequestMapping("/loadCarBoxs")
	public void loadCarBoxs(String jsonConfig,HttpServletResponse response){
		Map<String,Object> args=JsonParamHandler.getMapParam(jsonConfig);
		List<Map<String,Object>> rst=service.loadCarBoxs((String)args.get("carId"));
		JsonOutUtils.returnOkWithData(response, MapUtils.build((String)args.get("selectId"),rst));
	}
	
	/**
	 * 从当前采集数据中同步油位计和车辆的关联
	 * @param response
	 */
	@RequestMapping("/syn")
	public void syn(HttpServletResponse response){
		try{
			   service.addMetersFromSampling();
			   JsonOutUtils.returnOk(response);
		   }
		   catch(Exception e){
			   e.printStackTrace();
			   JsonOutUtils.returnError(response, "服务出现错误");
		   }
	}
	
	/**
	 * 同步采集程序(重新加载油箱标定，以便新加设备生效)
	 * @param response
	 */
	@RequestMapping("/updateCollect")
	public void updateCollect(HttpServletResponse response){
		try{
			String resp=OkhttpUtils.get(SysConfigUtil.getConfig("collecter")+"/out/reloadCalibrater.do");
			JsonOutUtils.returnOkWithData(response, resp);
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "同步采集程序出错");
		}
	}

}
