package com.hb.proj.car.controller;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.car.service.CarRptUtils;
import com.hb.proj.car.service.CarSumRptService;
import com.hb.proj.model.Wpg;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.MapUtils;
import com.hb.proj.utils.RptMonthUtil;
import com.hb.proj.utils.RptOuter;
import com.hb.xframework.util.DateUtil;

@Controller
@RequestMapping("/**/rpt")
public class WpgSumRptController {

	@Autowired
	private CarSumRptService service;
	
	/**
	 * 水电气汇总  同比折线图
	 * @param rptMonth
	 * @param request
	 * @param response
	 */
	@RequestMapping("/loadWpgForChart")
	public void loadWpgForChart(String rptMonth,HttpServletRequest request,HttpServletResponse response){
		try{
			if(StringUtils.isEmpty(rptMonth)){
				Calendar ca=Calendar.getInstance();
				
				rptMonth=String.valueOf(ca.get(Calendar.YEAR))+"-"+String.format("%02d", RptMonthUtil.getFullMonthNum(ca));
				
			}
			String[] ymtr=rptMonth.split("-");
			String startMonth=ymtr[0]+"-01";
			String preyear=String.valueOf(Integer.parseInt(ymtr[0])-1);
			String preStartMonth=preyear+"-01";
			String preEndMonth=preyear+"-12";   //preyear+"-"+ymtr[1];
			
			List<Wpg>  yearDatas=service.loadWpgGroupOrgAndMth(startMonth, rptMonth);
			List<Wpg>  preyearDatas=service.loadWpgGroupOrgAndMth(preStartMonth, preEndMonth);
			List<Map<String,Object>> orgs=service.loadAllSubOrgs();
			JsonOutUtils.returnOkWithData(response, MapUtils.builder("year",yearDatas,"pre",preyearDatas,"org",orgs),"MM");
			
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	/**
	 * 水电气汇总表---报表
	 * @param rptMonth
	 * @param request
	 * @param response
	 */
	@RequestMapping("/loadWpgSum")
	public void loadWpgSum(String rptMonth,HttpServletRequest request,HttpServletResponse response){
		try{
			if(StringUtils.isEmpty(rptMonth)){
				rptMonth=DateUtil.format(new Date(), "yyyy-MM");
			}
			String startMonth=(rptMonth.split("-"))[0]+"-01";
			Map<String,Wpg> thisMonthIdx=builderIndex(service.loadWpg(rptMonth, rptMonth));
			Map<String,Wpg> sumRptIdx=builderIndex(service.loadWpg(startMonth, rptMonth));
			
			String rptMonthTitle=rptMonth.replace("-", "年")+"月";
			Map<String,Object> rootMap=new HashMap<String,Object>();
			rootMap.put("rptMonthTitle", rptMonthTitle);
			rootMap.put("monthDS", thisMonthIdx); //当月统计数据集
			rootMap.put("sumDS", sumRptIdx);  //累计统计数据集
			rootMap.put("orgDS", service.loadAllSubOrgs()); //所有项目部
			
			RptOuter.clearTemplateCache();
			RptOuter.write(request,response,rootMap,"wpgSum.ftl");
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	private Map<String,Wpg> builderIndex(List<Wpg> wpgs){
		Map<String,Wpg> index=new HashMap<String,Wpg>();
		Wpg total=new Wpg();
		for(Wpg w : wpgs){
			index.put(w.getOrgId(), w);
			addWpg(total,w);
		}
		
		index.put("合计", total);
		return index;
	}
	
	private void addWpg(Wpg t,Wpg d){
		t.setWaterCount(CarRptUtils.addDouble(t.getWaterCount(), d.getWaterCount()));
		t.setWaterCost(CarRptUtils.addDouble(t.getWaterCost(),d.getWaterCost()));
		t.setPowerCount(CarRptUtils.addDouble(t.getPowerCount(),d.getPowerCount()));
		t.setPowerCost(CarRptUtils.addDouble(t.getPowerCost(),d.getPowerCost()));
		t.setNgasCount(CarRptUtils.addDouble(t.getNgasCount(),d.getNgasCount()));
		t.setNgasCost(CarRptUtils.addDouble(t.getNgasCost(),d.getNgasCost()));
		t.setLgasCount(CarRptUtils.addDouble(t.getLgasCount(),d.getLgasCount()));
		t.setLgasCost(CarRptUtils.addDouble(t.getLgasCost(),d.getLgasCost()));
	}
	
	
}
