package com.hb.proj.car.quota;

import java.util.Map;

import org.apache.commons.lang.StringUtils;

import com.hb.proj.car.service.CarContext;
import com.hb.proj.model.Quota;
import com.hb.proj.utils.DataUtils;

//定额标准取于定额表，需要同步更新
public class TravelQuotaStandard extends AbstractQuotaStandard{

	/**
	 * recordId_areaCode
	 */
	@Override
	public String getQuotaKey(Quota quota) {
		return quota.getQuotaKey()+"_"+quota.getWorkArea();
	}

	public String getRealKey(CarContext context) {
		return context.getQuotaTravalWorkKey()+"_"+context.getWorkArea();
	}

	@Override
	public void fillCarQuota(CarContext context) {
		 String key=getRealKey(context);
		 Quota quota=getQuota(key);
		 if(quota!=null&&quota.getQuotaCount()!=null){
			 context.setTravalQuota(quota.getQuotaCount().doubleValue());
		 }
	}

	/**
	 * 百公里定额的确定特殊：定额+上浮量(月份上浮)+上浮量(车辆年限上浮)  2021.11.15
	 * @param context
	 * @param config
	 */
	public void fillCarQuotaSpec(CarContext context,QuotaConfig  config) {
		 String key=getRealKey(context);
		 Quota quota=getQuota(key);
		 if(quota!=null&&quota.getQuotaCount()!=null){
			 context.setTravalQuota(quota.getQuotaCount().doubleValue());
			 addRiseQuota(context,quota,config);
		 }
	}
	
	//增加上浮量（月份，车型相关）  定额标准中有设备名称（车型） 2021.11.11
	//增加上浮量2(使用年限相关) 2021.11.15
	private void addRiseQuota(CarContext context,Quota quota,QuotaConfig  config){
				//根据使用年份上浮的规则，年份以5的倍数划分
				int calYears=(int)(Math.floor(context.getWorkYear()/5)*5);
				Double risePercent=getRisePercent(calYears,quota.getRiseMap());
				if(risePercent!=null&&!Double.isNaN(risePercent)){
					context.setTravalQuota(DataUtils.round(context.getTravalQuota()*risePercent, 2));
					
				}
				/*if(quota.getRiseMap()!=null&&quota.getRiseMap().containsKey(calYears)){
					Double riseVal=quota.getRiseMap().get(calYears);
					context.setTravalQuota(context.getTravalQuota()+(riseVal!=null&&!Double.isNaN(riseVal)?riseVal.doubleValue():0));  //增加上浮量
				}*/
		
				//判断上报月份是否在指定的油耗上浮月份中(针对的是百公里定额)
				String riseMths=config.getQuotaRiseMonths();
				if(StringUtils.isEmpty(riseMths)||riseMths.trim().length()==0){  //没有设置上浮月份表示不上浮
					return;
				}
				
				riseMths=","+(riseMths.trim()).replaceAll("\\D+", ",")+",";
				riseMths=riseMths.replaceAll(",0", ",");  //月份统一为首位不带0
				String rptMonth=(context.getRptMonth()).replaceFirst("^0", "");
				boolean riseMthIf=riseMths.indexOf(","+rptMonth+",")>=0;
				
				if(!riseMthIf){ //不满足上浮月份条件
					return;
				}
				
				boolean riseDeviceIf=true;  //默认不限制设备类型
				String riseDevices=config.getQuotaRiseDeviceNames();
				if(StringUtils.isNotEmpty(riseDevices)&&riseDevices.trim().length()>0){
					riseDevices=","+(riseDevices.trim()).replaceAll("[,;，；、]+", ",")+",";
					String dn=quota.getDeviceName()!=null?quota.getDeviceName().trim():null;
					riseDeviceIf=dn==null?false:(riseDevices.indexOf(","+dn+",")>=0);
				}
				
				if(riseMthIf&&riseDeviceIf){   //2021.11.11 新百公里定额中的上浮规则中，需要为指定车型且在指定月份里
					context.setTravalQuota(context.getTravalQuota()+config.getQuotaRiseCount());  //增加上浮量
				}
				
	}
	
	/**
	 * 以5年为步长，复合递增
	 * @param calYears
	 * @param riseMap
	 * @return
	 */
	private Double getRisePercent(int calYears,Map<Integer, Double> riseMap){
		if(calYears>15){
			calYears=15;  //车龄大于15年的按15年算上浮量
		}
		if(riseMap==null||!riseMap.containsKey(calYears)){
			return null;
		}
		if(riseMap.get(5)==null){
			riseMap.put(5, 0.0);
		}
		if(riseMap.get(10)==null){
			riseMap.put(10, 0.0);
		}
		if(riseMap.get(15)==null){
			riseMap.put(15, 0.0);
		}
		
		if(calYears==5){
			return 1+riseMap.get(5)/100.0;
		}
		if(calYears==10){
			return (1+riseMap.get(5)/100.0)*(1+riseMap.get(10)/100.0);
		}
		if(calYears==15){
			return (1+riseMap.get(5)/100.0)*(1+riseMap.get(10)/100.0)*(1+riseMap.get(15)/100.0);
		}
		return null;
	}
}
