package com.hb.proj.car.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.proj.model.OilAddLog;
import com.hb.proj.model.OilChangeBO;
import com.hb.xframework.dao.core.SpringJdbcDAO;
import com.hb.xframework.dao.util.FieldAttrConverter;
import com.hb.xframework.dao.util.PageModel;

@Service
public class CarOilMonitorService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	/**
	 * 车辆当前油量数据查询
	 * @param args
	 * @param authAssistCodes
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	public PageModel<Map<String,Object>> queryCarOil(Map<String,Object> args,Set<String> authAssistCodes,int pageNo,int pageSize){
		List<Object> sqlParams=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder(100);
		sql.append("select * from (");
		sql.append("select g.org_name,oil.*");
		sql.append(" from t_oil_current oil ");
		sql.append(" left join t_car car on car.car_num like concat('%',left(dtu_num,instr(dtu_num,'_')-1),'%')");
		sql.append(" left join t_org g on car.belong_org=g.org_id");
		sql.append(" ) tab where 1=1 ");
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("dtuNum"))){
			sql.append(" and (dtu_num like ?)");
			sqlParams.add("%"+args.get("dtuNum")+"%");
		}
		if(args!=null&&"1".equals((String)args.get("sort"))){
			sql.append(" order by data_time desc ");
			
		}
		else{
			sql.append(" order by dtu_num ");
		}
		
		Object[] sqlArgs=sqlParams.size()>0?sqlParams.toArray():null;
		return dao.queryForPagedListMap(sql.toString(),pageNo, pageSize,sqlArgs);
	}
	
	/**
	 * 油量采集数据查询--分页表格
	 * @param mountId
	 * @param startDate
	 * @param endDate
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	public PageModel<Map<String,Object>>  queryOilCollect(String mountId,Date startDate,Date endDate,int pageNo,int pageSize){
		StringBuilder sql=new StringBuilder(100);
		sql.append("select t.* from t_oil_history_2 t");
		sql.append(" where (noise_if is null or noise_if<>1)  and  mount_id=? and data_time between ? and ?");
		sql.append(" order by data_time desc");
		return dao.queryForPagedListMap(sql.toString(),pageNo, pageSize,mountId,startDate,endDate);
	}
	
	/**
	 * 油量采集数据查询--曲线绘制
	 * @param mountId
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	public List<Map<String,Object>> queryOilCollectForCurve(String mountId,Date startDate,Date endDate){
		StringBuilder sql=new StringBuilder(100);
		sql.append("select t.*,oil_volume data from t_oil_history_2 t");
		sql.append(" where (noise_if is null or noise_if<>1)  and mount_id=? and data_time between ? and ? and oil_volume is not null and oil_volume>0");
		sql.append(" order by data_time asc");
		return dao.queryForListMap(sql.toString(),mountId,startDate,endDate);
	}
	
	
	/**
	 * 油量异常数据监控查询
	 * @param args
	 * @param authAssistCodes
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	public PageModel<Map<String,Object>> queryOilException(Map<String,Object> args,Set<String> authAssistCodes,int pageNo,int pageSize){
		List<Object> sqlParams=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder(100);
		sql.append("select * from (");
		sql.append("select g.assist_code,g.org_name belong_org_name,t.car_num,t.custom_num,t.device_model,t.device_name,t.discard_if,t.custom_tag,");
		sql.append(" oil_type,m.box_num,c.*");
		sql.append(" from t_oil_change c inner join t_car t on c.car_id=t.car_id");
		sql.append(" left join t_oil_meter m on c.mount_id=m.record_id");
		sql.append(" left join t_org g on t.belong_org=g.org_id ");
		sql.append(" ) tab where discard_if=0 ");
		
		sql.append(" and DATE_FORMAT(data_time,'%Y-%m-%d %T') between ? and ?");
		sqlParams.add(args.get("startDate"));
		sqlParams.add(args.get("endDate"));
		
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("carOrCustomNum"))){
			sql.append(" and (car_num like ? or custom_num like ?)");
			sqlParams.add("%"+args.get("carOrCustomNum")+"%");
			sqlParams.add("%"+args.get("carOrCustomNum")+"%");
		}
		
		if(authAssistCodes!=null){ //权限过滤  单位assistCode
			sql.append(getPartSQL(authAssistCodes,sqlParams));
		}
		//客户端指定排序规则
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("sortField"))){
			String sortField=FieldAttrConverter.getFieldName((String)args.get("sortField"));
			sql.append(" order by "+sortField+" "+(args.get("sortAction")==null?"asc":args.get("sortAction")));
		}
		else{
			sql.append(" order by data_time desc,custom_num,car_num");
		}
		Object[] sqlArgs=sqlParams.size()>0?sqlParams.toArray():null;
		return dao.queryForPagedListMap(sql.toString(),pageNo, pageSize,sqlArgs);
	}
	
	private String getPartSQL(Set<String> authAssistCodes,List<Object> sqlParams){
		StringBuilder sql=new StringBuilder();
		for(String c : authAssistCodes){
			sql.append("or assist_code like ?");
			sqlParams.add(c+"%");
		}
		return "and ("+sql.substring(2)+")";
	}
	
	public PageModel<Map<String,Object>>  queryOilMonitorRpt(Map<String,Object> args,Set<String> authAssistCodes,int pageNo,int pageSize){
		List<Object> sqlParams=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder(100);
		sql.append("select * from (");
		sql.append("select m.box_num,box_name,r.*,t.car_num,t.custom_num,t.device_model,t.device_name,t.oil_type,t.custom_tag,consume.rpt_volume,(IFNULL(consume.rpt_volume,0)-IFNULL(add_volume,0)) diff_volume,g.org_name belong_org_name,g.assist_code ");
		sql.append(" from t_oil_monitor_report r ");
		sql.append(" left join t_oil_meter m on r.mount_id=m.record_id");
		sql.append(" inner join t_car t on m.belong_car_id=t.car_id and t.discard_if=0");
		sql.append(" left join t_oil_box box on m.box_id=box.record_id");
		sql.append(" left join t_org g on t.belong_org=g.org_id ");
		sql.append(" left join (");
		sql.append(" select  car_id,IFNULL(oil_erp,0)+IFNULL(oil_kl,0)+IFNULL(coil_erp,0)+IFNULL(coil_kl,0) rpt_volume");
		sql.append(" from t_car_consume c where  DATE_FORMAT(rpt_date,'%Y-%m')=? ");
		sql.append(" ) consume on r.car_id=consume.car_id");
		sql.append(" ) rpt where rpt_month=? ");
		
		sqlParams.add(args.get("rptMonth"));
		sqlParams.add(args.get("rptMonth"));
		
		
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("carKey"))){
			sql.append(" and (car_num like ? or custom_num like ?)");
			sqlParams.add("%"+args.get("carKey")+"%");
			sqlParams.add("%"+args.get("carKey")+"%");
		}
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("belongOrg"))){  //入参belongOrg为assistCode
			sql.append(" and assist_code like ?");
			sqlParams.add(args.get("belongOrg")+"%");
		}
		if(authAssistCodes!=null){ //权限过滤  单位assistCode
			sql.append(getPartSQL(authAssistCodes,sqlParams));
		}
		//客户端指定排序规则
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("sortField"))){
			String sortField=FieldAttrConverter.getFieldName((String)args.get("sortField"));
			sql.append(" order by "+sortField+" "+(args.get("sortAction")==null?"asc":args.get("sortAction")));
		}
		else{
			sql.append(" order by custom_num asc,box_num asc");
		}				
		
		return dao.queryForPagedListMap(sql.toString(),pageNo, pageSize,sqlParams.toArray());
	}
	
	/**
	 * 查询指定车辆的油量监控统计，用于单车油耗统计报表（安全科用），用月初、月终余油计算耗油
	 * @param carIds
	 * @param startMonth
	 * @param endMonth
	 * @return
	 */
	public Map<String,Map<String,Object>> loadCarMonitorRpt(Set<String> carIds,String startMonth,String endMonth){
		String carIdstr="'"+StringUtils.join(carIds, "','")+"'";
		StringBuilder sql=new StringBuilder();
		List<Map<String,Object>>  rst=null;
		if(startMonth.equals(endMonth)){
			sql.append("select car_id,start_volume,end_volume ");
			sql.append("from t_oil_monitor_report r  where r.car_id in ("+carIdstr+") and rpt_month=? ");
			rst=dao.queryForListMap(sql.toString(),endMonth);
		}
		else{
			sql.append("select t1.car_id,start_volume,end_volume from ( ");
			sql.append("select car_id,start_volume from t_oil_monitor_report r ");
			sql.append("where r.car_id in ("+carIdstr+") and rpt_month=? ");
			sql.append(") t1 ");
			sql.append("inner join( ");
			sql.append("select car_id,end_volume from t_oil_monitor_report r ");
			sql.append("where r.car_id in ("+carIdstr+") and rpt_month=? ");
			sql.append(") t2  on t1.car_id=t2.car_id ");
			
			rst= dao.queryForListMap(sql.toString(), startMonth,endMonth);
		}
		
		if(rst==null||rst.size()==0){
			return null;
		}
		Map<String,Map<String,Object>>  mapping=new HashMap<String,Map<String,Object>>(rst.size());
		for(Map<String,Object> itm : rst){
			mapping.put((String)itm.get("carId"), itm);
		}
		
		return mapping;
	}
	
	/**
	 * 查找实际加油记录对应的测量加油记录
	 * @param log
	 * @return
	 */
	public List<OilChangeBO>  loadCarOilAdd(OilAddLog log){
		StringBuilder sql=new StringBuilder(100);
		sql.append(" select m.box_num,c.* from t_oil_change c ");
		sql.append(" left join t_oil_meter m on c.mount_id=m.record_id");
		sql.append(" where c.car_id=? and date_format(c.data_time,'%Y-%m-%d')=date_format(?,'%Y-%m-%d') and diff_volume>0");
		return dao.queryForListPojo(sql.toString(), OilChangeBO.class, log.getCarId(),log.getAddTime());
	}
	
	public OilChangeBO  getOilChange(String id){
		String sql="select * from t_oil_change where id=?";
		return dao.queryForObject(sql, OilChangeBO.class, id);
	}
}
