package com.hb.proj.car.service;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.proj.model.CarConsume;
import com.hb.proj.model.CarConsumeRptVO;
import com.hb.xframework.dao.core.SpringJdbcDAO;
import com.hb.xframework.dao.util.FieldAttrConverter;

@Service
public class CarRptService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	
	

	/**
	 * 查询车辆油耗明细记录(野外时长直接取用，录入时做月份限制即可）
	 * 可能查询还需要根据投产月份，统计月份分开查询，涉及到使用年数的计算（如百公里定额）,目前：sql语句中计算出已使用年数，并按年数分组
	 * 使用年限的计算，和人年龄一样，只要月份是超过投产月份的就认为是进入下一年的。
	 * 由于在上报数据是就会计算。所以此时不需要计算。直接分组求和即可（除了实际单耗要重新计算）
	 * @param startMonth
	 * @param endMonth
	 * @param orgAssistCode
	 * @param deviceKey
	 * @param sortField
	 * @param sortType
	 * @return
	 */
	public List<CarConsumeRptVO>  loadCarConsumeDtl(String startMonth,String endMonth,String orgAssistCode,String deviceKey,String sortField,String sortType){
		List<Object> params=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder();
		sql.append("select c.car_num,c.work_start,cg.org_name,oil.*, ");
		sql.append(" r.oil_type,r.device_name,r.device_model");
		sql.append(" from t_car c ");
		
		sql.append(" inner join (  ");
		
		sql.append("select car_id,group_concat(distinct d.code_name) work_area,max(custom_num) custom_num,sum(travel_mile) travel_mile,sum(work_hour) work_hour,sum(engine_work_hour) engine_work_hour,sum(out_hour) out_hour,  ");
		sql.append("sum(coil_zsh) coil_zsh,sum(coil_meter_kg) coil_meter_kg,sum(oil_meter_kg) oil_meter_kg,sum(oil_erp) oil_erp,sum(oil_kl) oil_kl,sum(coil_erp) coil_erp,sum(coil_kl) coil_kl, ");
		sql.append("sum(check_mile) check_mile,group_concat(distinct quota_per100km) union_quota_per100km,max(real_cost_per100km) real_cost_per100km,sum(oil_real_cost) oil_real_cost,sum(coil_real_cost) coil_real_cost, ");
		sql.append("sum(oil_total_quota) oil_total_quota,sum(coil_total_quota) coil_total_quota ,sum(work_cost) work_cost,sum(out_cost) out_cost,sum(engine_cost) engine_cost ");
		
		sql.append(" from t_car_consume s ");
		sql.append(" left join t_org g on s.rpt_org=g.org_id");
		sql.append(" left join t_sort_code d on s.work_area=d.code_id");
		sql.append(" where DATE_FORMAT(s.rpt_date,'%Y-%m') between ? and ?   and   g.assist_code like ? ");
		
		sql.append(" group by car_id");
		
		
		
		sql.append(" )  oil  on c.car_id=oil.car_id ");
		
		sql.append(" left join t_org  cg on c.belong_org=cg.org_id");
		
		sql.append(" left join t_quota_rule r on c.quota_rule_id=r.record_id");
		
		sql.append(" where 1=1 ");
		
		params.add(startMonth);
		params.add(endMonth);
		params.add(orgAssistCode+"%");
		
		if(StringUtils.isNotEmpty(deviceKey)){
			String[] dns=deviceKey.split("[,，;；\\-~\\|\\s]+");
			if(dns.length>1){ //两个参数
				sql.append(" and ((r.device_name like ?  and r.device_model like ?) or (r.device_name like ?  and r.device_model like ?))");
				params.add("%"+dns[0]+"%");
				params.add("%"+dns[1]+"%");
				params.add("%"+dns[1]+"%");
				params.add("%"+dns[0]+"%");
			}
			else{
				sql.append(" and (r.device_name like ? or r.device_model like ?)");
				params.add("%"+dns[0]+"%");
				params.add("%"+dns[0]+"%");
			}
		}
		
		if(StringUtils.isNotEmpty(sortField)){
			sql.append(" order by "+FieldAttrConverter.getFieldName(sortField)+" "+(StringUtils.isEmpty(sortType)?"":sortType));
		}
		else{
			sql.append(" order by cg.display_num,custom_num,car_num");
		}
		
		
		
		return dao.queryForListPojo(sql.toString(),CarConsumeRptVO.class, params.toArray());
	}
	
	/**
	 * 按车型统计油耗。直接按车型分组求和即可
	 * @param startMonth
	 * @param endMonth
	 * @param orgAssistCode
	 * @return
	 */
	public List<CarConsume>  loadCarTypeConsume(String startMonth,String endMonth,String orgAssistCode){
		List<Object> params=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder();
		
		sql.append("select device_name,sum(travel_mile) travel_mile,sum(work_hour) work_hour,sum(engine_work_hour) engine_work_hour,sum(out_hour) out_hour,  ");
		sql.append("sum(coil_zsh) coil_zsh,sum(coil_meter) coil_meter,sum(oil_meter) oil_meter,sum(oil_erp) oil_erp,sum(oil_kl) oil_kl,sum(coil_erp) coil_erp,sum(coil_kl) coil_kl, ");
		sql.append("sum(check_mile) check_mile,sum(oil_real_cost) oil_real_cost,sum(coil_real_cost) coil_real_cost,sum(oil_total_quota) oil_total_quota,sum(coil_total_quota) coil_total_quota ");
		
		sql.append(" from t_car c  left join t_car_consume s on c.car_id=s.car_id and DATE_FORMAT(s.rpt_date,'%Y-%m') between ? and ?");
		sql.append(" left join t_org g on s.rpt_org=g.org_id");
		sql.append(" where  g.assist_code like ?");
		sql.append(" group by device_name");
		
		sql.append(" order by device_name");
		
		params.add(startMonth);
		params.add(endMonth);
		params.add(orgAssistCode+"%");
		return dao.queryForListPojo(sql.toString(),CarConsume.class, params.toArray());
	}
	

	

	
	
	
	
	

}
