package com.hb.proj.car.service;

import java.text.DecimalFormat;
import java.util.List;

import com.hb.proj.car.quota.QuotaConfig;
import com.hb.proj.car.quota.QuotaStandardSupport;
import com.hb.proj.model.CarConsume;
import com.hb.xframework.util.DateUtil;

public class CarRptUtils {

	
	
	/**
	 * oilWeightRatio 带权重的汽油折算系数
	 * @param car
	 * @param oilWeightRatio
	 */
	public static void rptCalculate(CarConsume car,Double oilWeightRatio){
		if(car.getRealAge()==null){  //初始数据可能车还未输入投产日期，无法计算使用年数
			return;
		}
		/*if(car.getRealAge()>20){ //车辆使用年限超过20年的都按20年计算取定额数值
			car.setRealAge(20);
		}*/
		//CarContext  carContext=new CarContext(car.getDeviceName(),car.getDeviceModel(),car.getRealAge()); 
		//车辆和定额标准的关联采用新的方式2021-10
		CarContext  carContext=new CarContext(car.getQuotaRuleId(),car.getQuotaAlternatorId(),car.getWorkArea(),DateUtil.format(car.getRptDate(), "MM"));
		carContext.setDeviceName(car.getDeviceName()); //2021.11.11 新百公里定额中的上浮规则(固定月份上浮)中，需要为指定车型
		carContext.setWorkYear(car.getRealAge()); //2021.11.15 新百公里定额中的上浮规则(根据使用年限上浮)
		QuotaStandardSupport.fillCarQuota(carContext);
		calculate(car,carContext,oilWeightRatio);
	}
	
	
	/**
	 * 计算实际单耗.跨月统计时不能是实际单耗的相加，应重新按公式计算
	 * @param car
	 */
	public static void calculateRealCostPer100km(List<? extends CarConsume>  cars){
		if(cars==null||cars.size()==0){
			return;
		}
		Double realCost=null;
		for(CarConsume car :  cars){
			realCost="汽油".equals(car.getOilType())?car.getOilRealCost():car.getCoilRealCost(); //单位kg
			
			if(realCost!=null&&car.getCheckMile()!=null&&car.getCheckMile()>0){
				realCost=getLfromKg(realCost,car.getOilType());
				car.setRealCostPer100km(realCost*100/car.getCheckMile());
				if(Double.isNaN(car.getRealCostPer100km())){
					car.setRealCostPer100km(0.0);
				}
				car.setRealCostPer100km(formatVal(car.getRealCostPer100km()));
			}
		}
		
				
	}
	
	
	/**
	 * 计算定额总量(定额标准*里程（工作小时）)和考核里程
	 * 行驶里程/100*百公里定额+作业小时*作业定额+发动机工作小时*发动机定额+野外小时*野外定额
	 * 
	 * 考核里程是行驶里程+（作业小时耗油+野外小时耗油+发动机耗油）*100/定额单耗
	 * 
	 * 实际单耗：实际百公里消耗量=实际消耗量（kg）、折算系数-》转实际消耗（升）*100/考核里程
	 * 数据录入使用的单位升，报表中会转成kg
	 * 没有  汽柴油混用的车
	 * 
	 * 对于计算字段数据，每次都应计算一次。避免使用旧数据入库
	 */
	public static void calculate(CarConsume car,CarContext  carContext,Double oilWeightRatio){
		
		car.setQuotaPer100km(carContext.getTravalQuota());
		//car.setOilMeter(getKgFromL(car.getOilMeter(),"汽油"));  //从库中取出的是升，转为kg
		//car.setCoilMeter(getKgFromL(car.getCoilMeter(),"柴油"));  //从库中取出的是升，转为kg
		
		if(car.getOilType()==null){
			car.setOilType("汽油");
		}
		
		double totalQuota=0;
		
		car.setWorkCost(car.getWorkHour()!=null?(car.getWorkHour()*carContext.getWorkQuota()):null);
		car.setEngineCost(car.getEngineWorkHour()!=null?(car.getEngineWorkHour()*carContext.getEngineQuota()):null);
		car.setOutCost(car.getOutHour()!=null?(car.getOutHour()*carContext.getOuterQuota()):null);
		
		//发动机耗油kg 默认发动机为柴油
		if(car.getEngineCost()!=null){
			car.setEngineCostKg(getKgFromL(car.getEngineCost(),"柴油",null));
		}
		
		totalQuota+=car.getWorkCost()!=null?car.getWorkCost():0;
		totalQuota+=car.getEngineCost()!=null?car.getEngineCost():0;
		totalQuota+=car.getOutCost()!=null?car.getOutCost():0;
		
		double checkMile=totalQuota*100/carContext.getTravalQuota();
		if(Double.isNaN(checkMile)||Double.isInfinite(checkMile)){
			checkMile=0;
		}
		
		if(car.getTravelMile()!=null){
			totalQuota+=car.getTravelMile()*carContext.getTravalQuota()/100.0;
		}
		
		totalQuota=getKgFromL(totalQuota,car.getOilType(),oilWeightRatio) ; //升转为kg
		
		if("汽油".equals(car.getOilType())){
			car.setOilTotalQuota(formatVal(totalQuota));
		}
		else{
			car.setCoilTotalQuota(formatVal(totalQuota));
		}
		//考核里程
		car.setCheckMile(checkMile+(car.getTravelMile()!=null?car.getTravelMile():0));
		
		car.setCheckMile(formatVal(car.getCheckMile()));
		
		
		
		//实际单耗
		//实际耗量=erp(升-转公斤)+昆仑(公斤)
		Double realCost=null,realCostLiter=null; //单位公斤,升
		if("汽油".equals(car.getOilType())){
			car.setOilErpKg(getKgFromL(car.getOilErp(),car.getOilType(),oilWeightRatio)); //erp 升转为 kg
			car.setOilKlKg(getKgFromL(car.getOilKl(),car.getOilType(),oilWeightRatio)); //昆仑卡 升转为 kg 
			
			realCost=addDouble(car.getOilErpKg(),car.getOilKlKg());
			car.setOilRealCost(realCost);
			
			realCostLiter=addDouble(car.getOilErp(),car.getOilKl());
			
			car.setOilMeterKg(getKgFromL(car.getOilMeter(),car.getOilType(),oilWeightRatio));
			car.setOilCoal(toCoal(car.getOilRealCost(),car.getOilType()));  //转为标准吨煤
		}
		else{
			car.setCoilErpKg(getKgFromL(car.getCoilErp(),car.getOilType(),null)); //erp 升转为 kg
			car.setCoilKlKg(getKgFromL(car.getCoilKl(),car.getOilType(),null));  //昆仑卡 升转为 kg 
			
			
			realCost=addDouble(car.getCoilErpKg(),car.getCoilKlKg()); //中石化柴油不参与统计  +getVal(car.getCoilZsh()
			car.setCoilRealCost(realCost);
			
			realCostLiter=addDouble(car.getCoilErp(),car.getCoilKl());
			
			car.setCoilMeterKg(getKgFromL(car.getCoilMeter(),car.getOilType(),null));
			car.setCoilCoal(toCoal(car.getCoilRealCost(),car.getOilType()));  //转为标准吨煤
			
		}
		if(realCostLiter!=null&&car.getCheckMile()!=null&&car.getCheckMile()>0){ //公里数可能为0，影响后面处理
			car.setRealCostPer100km(realCostLiter*100/car.getCheckMile());
			if(Double.isNaN(car.getRealCostPer100km())){
				car.setRealCostPer100km(0.0);
			}
			car.setRealCostPer100km(formatVal(car.getRealCostPer100km()));
		}
		else{
			car.setRealCostPer100km(null);
		}
		
	}
	
	
	private static double formatVal(double data){
		if(Double.isNaN(data)){
			return data;
		}
		DecimalFormat df=new DecimalFormat("#0.00");
		return Double.parseDouble(df.format(data));
	}
	
	/*private static double getVal(Double data){
		return data==null?0:data.doubleValue();
	}*/
	
	/**
	 * 指定了折算系数的用指定的，否则用默认的
	 * @param data
	 * @param oilType
	 * @param oilWeightRatio
	 * @return
	 */
	public static double getKgFromL(Double data,String oilType,Double oilWeightRatio){
		if(data==null){
			return 0;
		}
		QuotaConfig qc=QuotaStandardSupport.getConfig();
		if("汽油".equals(oilType)){
			return oilWeightRatio!=null?(data*oilWeightRatio.doubleValue()):(data*qc.getOilRatio());   //默认92号
		}
		else{
			return data*qc.getCoilRatio();
		}
	}
	
	
	//获得带权重(92#:95#)的汽油折算系数---加权值
	public static double getOilWeightRatio(Double oil92,Double oil95){
		if(oil92==null){
			oil92=0.0;
		}
		if(oil95==null){
			oil95=0.0;
		}
		if(oil92==0&&oil95==0){
			return 1;
		}
		double w92=oil92/(oil92+oil95),w95=oil95/(oil92+oil95);  //计算权重
		QuotaConfig qc=QuotaStandardSupport.getConfig();
		return qc.getOilRatio()*w92+qc.getOilRatio95()*w95;
	}
	
	public static double getLfromKg(Double data,String oilType){
		if(data==null){
			return 0;
		}
		QuotaConfig qc=QuotaStandardSupport.getConfig();
		if("汽油".equals(oilType)){
			return data/qc.getOilRatio();
		}
		else{
			return data/qc.getCoilRatio();
		}
	}
	
	private static double toCoal(Double data,String oilType){
		if(data==null){
			return 0;
		}
		QuotaConfig config=QuotaStandardSupport.getConfig();
		if("汽油".equals(oilType)){
			return data*config.getOilCoalRatio()/1000.0;
		}
		else{
			return data*config.getCoilCoalRatio()/1000.0;
		}
	}
	
	
	public static void sumConsume(CarConsume main,CarConsume extra){
		//考核里程,行驶里程
		main.setCheckMile(addDouble(main.getCheckMile(),extra.getCheckMile()));
		main.setTravelMile(addDouble(main.getTravelMile(),extra.getTravelMile()));
		main.setWorkHour(addInteger(main.getWorkHour(),extra.getWorkHour()));
		main.setOutHour(addInteger(main.getOutHour(),extra.getOutHour()));
		main.setEngineWorkHour(addDouble(main.getEngineWorkHour(),extra.getEngineWorkHour()));
		main.setOilTotalQuota(addDouble(main.getOilTotalQuota(),extra.getOilTotalQuota()));
		main.setCoilTotalQuota(addDouble(main.getCoilTotalQuota(),extra.getCoilTotalQuota()));
		main.setOilRealCost(addDouble(main.getOilRealCost(),extra.getOilRealCost()));
		main.setCoilRealCost(addDouble(main.getCoilRealCost(),extra.getCoilRealCost()));
		main.setOilMeter(addDouble(main.getOilMeter(),extra.getOilMeter()));
		main.setCoilMeter(addDouble(main.getCoilMeter(),extra.getCoilMeter()));
		main.setOilMeterKg(addDouble(main.getOilMeterKg(),extra.getOilMeterKg()));
		main.setCoilMeterKg(addDouble(main.getCoilMeterKg(),extra.getCoilMeterKg()));
	}
	
	public static Double addDouble(Double d1,Double d2){
		if(d2==null||Double.isNaN(d2)){
			return d1!=null?d1.doubleValue():null;
		}
		if(d1==null||Double.isNaN(d1)){
			return d2!=null?d2.doubleValue():null;
		}
		else{
			return d1.doubleValue()+d2.doubleValue();
		}
	}
	
	public static Double reduceDouble(Double d1,Double d2){
		if(d2==null||Double.isNaN(d2)){
			return d1;
		}
		if(d1==null||Double.isNaN(d1)){
			return d2!=null?(-1*d2.doubleValue()):null;
		}
		else{
			return d1.doubleValue()-d2.doubleValue();
		}
	}
	
	public static Integer addInteger(Integer t1,Integer t2){
		if(t2==null){
			return t1;
		}
		if(t1==null){
			t1=t2;
		}
		else{
			t1=t1.intValue()+t2.intValue();
		}
		return t1;
	}
	
	
}
