package com.hb.proj.car.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.proj.model.Wpg;
import com.hb.xframework.dao.core.SpringJdbcDAO;

@Service
public class CarSumRptService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	/**
	 * 查询所有项目部
	 * @return
	 */
	public List<Map<String,Object>> loadAllSubOrgs(){
		String sql="select org_id,org_name from t_org g where length(assist_code)=7 order by display_num";
		return dao.queryForListMap(sql);
	}
	
	/**
	 * 查询柴油类型编码
	 * @return
	 */
	public List<Map<String,Object>>  loadCoilCarTypes(){
		String sql="SELECT code_id,code_name,spare1 FROM t_sort_code  d where d.father_code_id='402881e8734da7fa01734da7fa600000' order by display_num";
		return dao.queryForListMap(sql);
	}
	
	/**
	 * 统计柴油车数量（按柴油车类型和单位）需求根据车辆调动特殊处理(车辆流转记录)
	 * 一个季度内同一车在一单位内可能多次调入，车辆数只算一次(distinct car_id)
	 * @return
	 */
	public Map<String,Object>  loadCoilCarCountRpt(String startMonth,String endMonth){
		StringBuilder sql=new StringBuilder();
		
		sql.append("select  concat(coil_car_type,'_',belong_org) ckey,count(distinct car_id) car_count  from (  ");
		sql.append(" select cf.car_id,cf.belong_org,c.coil_car_type  from t_car_transfer cf  ");
		sql.append(" inner join t_car c on cf.car_id=c.car_id  ");
		sql.append(" where coil_car_type is not null and oil_type='柴油' and  c.discard_if=0 ");
		sql.append(" and DATE_FORMAT(start_date,'%Y-%m') <=?  and (end_date is null or  DATE_FORMAT(end_date,'%Y-%m')>=?) ");
		sql.append(" ) tab  ");
		sql.append(" group by belong_org,coil_car_type ");
		
		return dao.queryForMapping(sql.toString(), "ckey", "car_count",endMonth,startMonth);
	}
	
	/**
	 * 统计柴油车耗油量（按柴油车类型和单位）
	 * @param startMonth
	 * @param endMonth
	 * @return
	 */
	public Map<String,Object>  loadCoilCarRealCost(String startMonth,String endMonth){
		StringBuilder sql=new StringBuilder();
		sql.append("select concat(coil_car_type,'_',rpt_org) ckey,coil_real_cost from ( ");
		sql.append(" select coil_car_type,rpt_org,sum(s.coil_real_cost) coil_real_cost ");
		sql.append(" from t_car_consume s ");
		sql.append(" inner join t_car c on s.car_id=c.car_id ");
		sql.append(" where coil_car_type is not null and oil_type='柴油' and DATE_FORMAT(rpt_date,'%Y-%m') between ? and ? group by coil_car_type,rpt_org ");
		sql.append(" ) tab ");
		return dao.queryForMapping(sql.toString(), "ckey", "coil_real_cost",startMonth,endMonth);
	}
	
	
	/**
	 * 统计水电气消耗
	 * @param startMonth
	 * @param endMonth
	 * @return
	 */
	public List<Wpg> loadWpg(String startMonth,String endMonth){
		List<Object> params=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder();
		
		sql.append("SELECT w.org_id,sum(water_count)  water_count,sum(water_cost) water_cost, ");
		sql.append(" sum(power_count)  power_count,sum(power_cost) power_cost, ");
		sql.append(" sum(ngas_count)  ngas_count,sum(ngas_cost) ngas_cost, ");
		sql.append(" sum(lgas_count)  lgas_count,sum(lgas_cost) lgas_cost ");
		sql.append(" FROM t_wpg_account  w  where w.org_id is not null and  DATE_FORMAT(rpt_month,'%Y-%m') between ? and ? ");
		sql.append(" group by w.org_id ");
		
		
		params.add(startMonth);
		params.add(endMonth);
		
		return dao.queryForListPojo(sql.toString(),Wpg.class,params.toArray());
	}
	
	
	/**
	 * 统计水电气消耗 按单位、月份分组
	 * @param startMonth
	 * @param endMonth
	 * @return
	 */
	public List<Wpg> loadWpgGroupOrgAndMth(String startMonth,String endMonth){
		List<Object> params=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder();
		
		sql.append("SELECT max(rpt_month) rpt_month,w.org_id,sum(water_count)  water_count,sum(water_cost) water_cost, ");
		sql.append(" sum(power_count)  power_count,sum(power_cost) power_cost, ");
		sql.append(" sum(ngas_count)  ngas_count,sum(ngas_cost) ngas_cost, ");
		sql.append(" sum(lgas_count)  lgas_count,sum(lgas_cost) lgas_cost ");
		sql.append(" FROM t_wpg_account  w  where w.org_id is not null and  DATE_FORMAT(rpt_month,'%Y-%m') between ? and ? ");
		sql.append(" group by w.org_id, DATE_FORMAT(rpt_month,'%Y-%m') ");
		sql.append(" order by org_id,DATE_FORMAT(rpt_month,'%Y-%m')");
		
		params.add(startMonth);
		params.add(endMonth);
		
		return dao.queryForListPojo(sql.toString(),Wpg.class,params.toArray());
	}
	
	public boolean checkLackRptData(String rptMonth){
		StringBuilder sql=new StringBuilder(100);
		sql.append("select g.org_id,tab.coil_count from t_org g left join (");
		sql.append("select org_id, sum(ifnull(coil_erp_count,0)+ifnull(coil_kl_count,0)) coil_count ");
		sql.append(" from t_other_consume");
		sql.append(" where DATE_FORMAT(rpt_month,'%Y-%m')=?  group by org_id");
		sql.append(" ) tab on g.org_id=tab.org_id");
		sql.append(" where length(g.assist_code)>4  and ifnull(g.org_type,'')!='unsubmitcoil' "); //限定为项目部,排除不需要上报油耗项目部
		
		List<Map<String,Object>> rst=dao.queryForListMap(sql.toString(), rptMonth);
		if(rst==null||rst.size()==0){  
			return false;
		}
		Double total=0.0;
		Number val=null;
		boolean findUnRpt=false;
		for(Map<String,Object> itm : rst){
			val=(Number)itm.get("coilCount");
			if(val==null||val.doubleValue()==0){  //发现未上报的项目部
				findUnRpt=true;
			}
			else{
				total+=val.doubleValue();
			}
		}
		
		if(total==0){ //全都未上报，就不提醒了，只有部分项目部未上报的才提醒
			return false;
		}
		else{
			return findUnRpt;
		}
		
		
	}
}
