package com.hb.proj.car.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.proj.model.CarConsume;
import com.hb.proj.model.OtherConsume;
import com.hb.xframework.dao.core.SpringJdbcDAO;

@Service
public class ERPKLSumRptService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	/**
	 * 统计各月份的考核里程、行驶里程、汽油柴油定额。目前统计的是全分公司
	 * @param year
	 * @return
	 */
	public List<Map<String,Object>>  rptCarMileAndQuota(String year){
		StringBuilder sql=new StringBuilder();
		sql.append("select rpt_date, sum(check_mile) check_mile,sum(travel_mile) travel_mile,sum(oil_total_quota) oil_total_quota,sum(coil_total_quota) coil_total_quota ");
		sql.append("from( ");
		sql.append("select DATE_FORMAT(rpt_date,'%m') rpt_date,s.check_mile,s.travel_mile,s.oil_total_quota,s.coil_total_quota ");
		sql.append("from t_car_consume s");
		sql.append(" inner join t_car c on s.car_id=c.car_id");
		sql.append(" where DATE_FORMAT(rpt_date,'%Y')=?");
		sql.append(") tab ");
		sql.append("group by rpt_date ");
		
		return dao.queryForListMap(sql.toString(),year);
	}
	
	/**
	 * 统计各月份的考核里程、行驶里程。目前统计的是全分公司
	 * @param year
	 * @return
	 */
	public List<Map<String,Object>>  rptCheckAndTravelMile(String year){
		StringBuilder sql=new StringBuilder();
		sql.append("select rpt_date, sum(check_mile) check_mile,sum(travel_mile) travel_mile,sum(out_hour) out_hour,sum(work_hour) work_hour,sum(engine_work_hour) engine_work_hour ");
		sql.append("from( ");
		sql.append("select DATE_FORMAT(rpt_date,'%m') rpt_date,s.check_mile,s.travel_mile,s.out_hour,s.work_hour,s.engine_work_hour ");
		sql.append("from t_car_consume s");
		sql.append(" inner join t_car c on s.car_id=c.car_id");
		sql.append(" where DATE_FORMAT(rpt_date,'%Y')=?");
		sql.append(") tab ");
		sql.append("group by rpt_date ");
		
		return dao.queryForListMap(sql.toString(),year);
	}
	
	/**
	 * ERP昆仑卡汇总统计-考核里程、行驶里程(2021.11 更改为：考核里程、发电机耗油--->2022.1.25更改为：发电机小时、发电机耗油)
	 * @param startMonth
	 * @param endMonth
	 * @return
	 */
	public List<CarConsume> loadERPKLSumMile(String startMonth,String endMonth){
		List<Object> params=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder();
		sql.append("select * from (");
		sql.append("select rpt_org belong_org,sum(engine_work_hour) engine_work_hour,sum(travel_mile) travel_mile,sum(check_mile) check_mile,sum(engine_cost_kg)/1000 engine_cost_kg");
		
		sql.append(" from t_car_consume s   ");
		sql.append(" inner join t_car c on s.car_id=c.car_id");
		sql.append(" where DATE_FORMAT(s.rpt_date,'%Y-%m') between ? and ? ");
		sql.append(" group by rpt_org");
		sql.append(" ) tab ");
		params.add(startMonth);
		params.add(endMonth);
		
		return dao.queryForListPojo(sql.toString(),CarConsume.class,params.toArray());
	}
	
	
	/**
	 * 其它油耗
	 * @param startMonth
	 * @param endMonth
	 * @return
	 */
	public List<OtherConsume> loadOtherConsume(String startMonth,String endMonth){
		StringBuilder sql=new StringBuilder();
		sql.append("select org_id,sum(oil_count_kg)/1000 oil_count_kg,sum(coil_count_kg)/1000 coil_count_kg,");
		sql.append(" sum(oil_erp_count_kg)/1000 oil_erp_count_kg,sum(coil_erp_count_kg)/1000 coil_erp_count_kg,");
		sql.append(" sum(oil_kl_count_kg)/1000 oil_kl_count_kg,sum(coil_kl_count_kg)/1000 coil_kl_count_kg,");
		sql.append(" sum(oil_kl_money) oil_kl_money,sum(coil_kl_money) coil_kl_money");
		sql.append(" from t_other_consume s   ");
		sql.append(" where DATE_FORMAT(s.rpt_month,'%Y-%m') between ? and ? ");
		sql.append(" group by org_id");
		
		return dao.queryForListPojo(sql.toString(),OtherConsume.class,startMonth,endMonth);
	}
	
	
}
