package com.hb.proj.car.service;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.proj.model.CarConsume;
import com.hb.proj.model.OtherConsume;
import com.hb.proj.model.Wpg;
import com.hb.xframework.dao.core.SpringJdbcDAO;

@Service
public class EnergyCostAccountService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	/**
	 * 油能源统计-根据要求从t_other_consume取（各项目部录入月累计消耗数据），不再通过车辆累加
	 * @param rptYear
	 * @return
	 */
	public List<OtherConsume> oilSumRpt(String rptYear){
		List<Object> params=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder();
		sql.append("select  max(s.rpt_month) rpt_month,");
		sql.append(" sum(oil_count_kg) oil_count_kg,sum(coil_count_kg) coil_count_kg,");
		sql.append(" sum(oil_erp_count_kg) oil_erp_count_kg,sum(coil_erp_count_kg) coil_erp_count_kg,");
		sql.append(" sum(oil_kl_count_kg) oil_kl_count_kg,sum(coil_kl_count_kg) coil_kl_count_kg,");
		sql.append(" sum(oil_coal) oil_coal,sum(coil_coal) coil_coal");
		sql.append(" from t_other_consume s   ");
		sql.append(" where DATE_FORMAT(s.rpt_month,'%Y') = ? ");
		sql.append(" group by DATE_FORMAT(s.rpt_month,'%Y-%m')");
		params.add(rptYear);
		return dao.queryForListPojo(sql.toString(),OtherConsume.class,params.toArray());
	}
	
	/**
	 * 统计分公司油耗(按月分组)
	 * @param rptYear
	 * @return
	 */
	public List<CarConsume> rptCompOilCost(String rptYear){
		List<Object> params=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder();
		sql.append("select max(s.rpt_date) rpt_date,sum(travel_mile) travel_mile,sum(check_mile) check_mile,");
		sql.append("sum(oil_total_quota) oil_total_quota,sum(coil_total_quota) coil_total_quota,");
		sql.append("sum(oil_real_cost) oil_real_cost,sum(coil_real_cost) coil_real_cost,sum(oil_coal+coil_coal) oil_coal ");
		sql.append(" from t_car_consume s   ");
		sql.append(" inner join t_car c on s.car_id=c.car_id");
		sql.append(" where DATE_FORMAT(s.rpt_date,'%Y') = ? ");
		sql.append(" group by DATE_FORMAT(s.rpt_date,'%Y-%m')");
		
		params.add(rptYear);
		
		
		return dao.queryForListPojo(sql.toString(),CarConsume.class,params.toArray());
	}
	
	/**
	 * 统计分公司水电气(按月分组)
	 * @param rptYear
	 * @return
	 */
	public List<Wpg> rptCompWpg(String rptYear){
		List<Object> params=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder();
		
		sql.append("SELECT max(rpt_month) rpt_month,sum(water_count)/10000  water_count,sum(water_count_quota)/10000 water_count_quota, ");
		sql.append(" sum(power_count)/10000  power_count,sum(power_count_quota)/10000  power_count_quota, ");
		sql.append(" sum(ngas_count)/10000  ngas_count,sum(ngas_count_quota)/10000 ngas_count_quota, ");
		sql.append(" sum(lgas_count)  lgas_count,sum(lgas_count_quota) lgas_count_quota,sum(power_coal) power_coal, sum(lgas_coal) lgas_coal,sum(ngas_coal) ngas_coal");
		sql.append(" FROM t_wpg_account  w  where  DATE_FORMAT(rpt_month,'%Y') = ? ");
		sql.append(" group by DATE_FORMAT(rpt_month,'%Y-%m') ");
		
		
		params.add(rptYear);
		
		
		return dao.queryForListPojo(sql.toString(),Wpg.class,params.toArray());
	}
}
