package com.hb.proj.car.service;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.proj.model.CarConsume;
import com.hb.proj.model.ERP;
import com.hb.proj.model.ERPCalculater;
import com.hb.proj.model.OtherConsume;
import com.hb.proj.model.Workload;
import com.hb.proj.model.WorkloadPO;
import com.hb.proj.model.Wpg;
import com.hb.proj.model.WpgCorrect;
import com.hb.proj.model.WpgCorrectCalculater;
import com.hb.xframework.dao.core.SpringJdbcDAO;
import com.hb.xframework.dao.util.UUIDHexGenerator;
import com.hb.xframework.util.DateUtil;

@Service
public class MaterialERPService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	/**
	 * 保存建筑面积到workload表月份固定为1月份
	 * 2022.5.26 按指定的月份存储，不再固定为1月份
	 * @param rptMonth
	 * @param builderArea
	 * @param us
	 * @return
	 */
	public boolean saveBuilderArea(String rptMonth,Double builderArea,String us){
		String sql="select record_id from t_workload_value where DATE_FORMAT(rpt_month,'%Y-%m')=?";
		Map<String,Object> record=dao.queryForSingleMap(sql,rptMonth);
		if(record!=null&&record.size()>0){ //存在记录，则直接修改
			sql="update t_workload_value set builder_area=? where DATE_FORMAT(rpt_month,'%Y-%m')=?";
			dao.getJdbcTemplate().update(sql, builderArea,rptMonth);
		}
		else{ //无则插入
			sql="insert into t_workload_value(record_id,builder_area,rpt_month,modifier,modify_time) values(?,?,?,?,?)";
			UUIDHexGenerator  uuid=UUIDHexGenerator.getInstance();
			dao.getJdbcTemplate().update(sql, uuid.generate(),builderArea,rptMonth+"-01",us,new Date());
		}
		return true;
	}
	
	public boolean saveERPData(ERP erp) {
		String sql="delete from t_material_erp where DATE_FORMAT(rpt_month,'%Y-%m')=?";
		dao.getJdbcTemplate().update(sql,DateUtil.format(erp.getRptMonth(), "yyyy-MM"));
		ERPCalculater.calculate(erp);
		UUIDHexGenerator uuid=UUIDHexGenerator.getInstance();
		erp.setRecordId(uuid.generate());
		erp.setModifyTime(new Date());
		dao.insertPojo(erp,"t_material_erp");
		return true;
	}
	
	public boolean saveWpgCorrect(WpgCorrect wpgcrt) {
		String sql="delete from t_wpg_correct where DATE_FORMAT(rpt_month,'%Y-%m')=?";
		dao.getJdbcTemplate().update(sql,DateUtil.format(wpgcrt.getRptMonth(), "yyyy-MM"));
		
	    WpgCorrectCalculater.calculate(wpgcrt);  //换算处理
		UUIDHexGenerator uuid=UUIDHexGenerator.getInstance();
		wpgcrt.setRecordId(uuid.generate());
		wpgcrt.setModifyTime( new Date());
		dao.insertPojo(wpgcrt,"t_wpg_correct");
		return true;
	}
	
	
	/**
	 * 录入工作量时避免对建筑面积的影响
	 * @param wkload
	 * @return
	 */
	public boolean saveWorkload(WorkloadPO wkload){
		String rptMonth=DateUtil.format(wkload.getRptMonth(), "yyyy-MM");
		/*if(rptMonth.endsWith("-01")){ //如果是1月份，1月份存储有建筑面积删除前先取出
			String sql="select builder_area from t_workload_value where DATE_FORMAT(rpt_month,'%Y-%m')=?";
			Map<String,Object> oldData=dao.queryForSingleMap(sql, rptMonth);
			if(oldData.get("builderArea")!=null){   //存储有建筑面积的，再存回去，避免丢失
				wkload.setBuilderArea(((Number)oldData.get("builderArea")).doubleValue()); 
			}
			else{
				wkload.setBuilderArea(null);  //建筑面积不在此处修改，所以必须要明确值不能用对象默认（默认是未知）
			}
			
		}
		String sql="delete from t_workload_value where DATE_FORMAT(rpt_month,'%Y-%m')=?";
		dao.getJdbcTemplate().update(sql,rptMonth);
		UUIDHexGenerator uuid=UUIDHexGenerator.getInstance();
		wkload.setRecordId(uuid.generate());
		wkload.setModifyTime( new Date());
		dao.insertPojo(wkload,"t_workload_value");*/
		
		String sql="select record_id from t_workload_value where DATE_FORMAT(rpt_month,'%Y-%m')=?";
		Map<String,Object> record=dao.queryForSingleMap(sql,rptMonth);
		if(record!=null&&record.size()>0){ //存在记录，则直接修改
			sql="update t_workload_value set well_count=?,industry_value=?,added_value=?,modifier=?,modify_time=? where DATE_FORMAT(rpt_month,'%Y-%m')=?";
			dao.getJdbcTemplate().update(sql, wkload.getWellCount(),wkload.getIndustryValue(),wkload.getAddedValue(),wkload.getModifier(),new Date(),rptMonth);
		}
		else{ //无则插入
			sql="insert into t_workload_value(record_id,well_count,industry_value,added_value,rpt_month,modifier,modify_time) values(?,?,?,?,?,?,?)";
			UUIDHexGenerator  uuid=UUIDHexGenerator.getInstance();
			dao.getJdbcTemplate().update(sql, uuid.generate(),wkload.getWellCount(),wkload.getIndustryValue(),wkload.getAddedValue(),wkload.getRptMonth(),wkload.getModifier(),new Date());
		}
		
		return true;
	}
	
	public List<ERP> rptMonthGrpERP(String year){
		StringBuilder sql=new StringBuilder();
		sql.append("select max(rpt_month) rpt_month,sum(oil_count) oil_count,sum(oil_money) oil_money,sum(coil_count) coil_count,sum(coil_money) coil_money,");
		sql.append("sum(oil_count_correct) oil_count_correct,sum(oil_money_correct) oil_money_correct,");
		sql.append("sum(coil_count_correct) coil_count_correct,sum(coil_money_correct) coil_money_correct,");
		sql.append("sum(oil_count_sum) oil_count_sum,sum(coil_count_sum) coil_count_sum,sum(oil_money_sum) oil_money_sum,sum(coil_money_sum) coil_money_sum ");
		sql.append(" from t_material_erp");
		sql.append(" where DATE_FORMAT(rpt_month,'%Y') = ? ");
		sql.append(" group by DATE_FORMAT(rpt_month,'%Y-%m')");
		return dao.queryForListPojo(sql.toString(), ERP.class,year);
	}
	
	
	public ERP  rptERPSum(String startMonth,String rptMonth){
		StringBuilder sql=new StringBuilder();
		sql.append("select sum(oil_count) oil_count,sum(oil_money) oil_money,sum(coil_count) coil_count,sum(coil_money) coil_money,");
		sql.append("sum(oil_count_correct) oil_count_correct,sum(oil_money_correct) oil_money_correct,");
		sql.append("sum(coil_count_correct) coil_count_correct,sum(coil_money_correct) coil_money_correct,");
		sql.append("sum(oil_coal) oil_coal,sum(coil_coal) coil_coal,sum(oil_correct_coal) oil_correct_coal,sum(coil_correct_coal) coil_correct_coal,");
		sql.append("sum(oil_sum_coal) oil_sum_coal,sum(coil_sum_coal) coil_sum_coal,");
		sql.append("sum(oil_count_sum) oil_count_sum,sum(coil_count_sum) coil_count_sum,sum(oil_money_sum) oil_money_sum,sum(coil_money_sum) coil_money_sum ");
		sql.append(" from t_material_erp");
		sql.append(" where DATE_FORMAT(rpt_month,'%Y-%m') between ? and ?");
		return dao.queryForObject(sql.toString(), ERP.class, startMonth,rptMonth);
	}
	
	public Wpg rptWpgSum(String startMonth,String rptMonth){
		StringBuilder sql=new StringBuilder();
		
		sql.append("select sum(water_count)/10000  water_count,sum(water_cost)/10000 water_cost, ");
		sql.append(" sum(power_count)/10000  power_count,sum(power_cost)/10000  power_cost, ");
		sql.append(" sum(ngas_count)/10000  ngas_count,sum(ngas_cost)/10000 ngas_cost, ");
		sql.append(" sum(lgas_count)/1000  lgas_count,sum(lgas_cost)/10000 lgas_cost,");
		sql.append(" sum(lgas_coal) lgas_coal,sum(ngas_coal) ngas_coal,sum(power_coal) power_coal");
		sql.append(" from t_wpg_account  w  where org_id is not null and   DATE_FORMAT(rpt_month,'%Y-%m') between ? and ? ");
		return dao.queryForObject(sql.toString(),Wpg.class,startMonth,rptMonth);
	}
	
	public WpgCorrect rptWpgCorrectSum(String startMonth,String rptMonth){
		StringBuilder sql=new StringBuilder();
		sql.append("select sum(water_count_correct) water_count_correct,sum(water_cost_correct) water_cost_correct,sum(power_count_correct) power_count_correct,sum(power_cost_correct) power_cost_correct,");
		sql.append("sum(ngas_count_correct) ngas_count_correct,sum(ngas_cost_correct) ngas_cost_correct,sum(lgas_count_correct) lgas_count_correct,sum(lgas_cost_correct) lgas_cost_correct,");
		sql.append("sum(power_correct_coal) power_correct_coal,sum(ngas_correct_coal) ngas_correct_coal,sum(lgas_correct_coal) lgas_correct_coal ");
		sql.append(" from t_wpg_correct where DATE_FORMAT(rpt_month,'%Y-%m') between ? and ?");
		return dao.queryForObject(sql.toString(),WpgCorrect.class,startMonth,rptMonth);
	}
	
	public CarConsume rptKLSum(String startMonth,String endMonth){
		StringBuilder sql=new StringBuilder();
		sql.append("select ");
		sql.append("sum(oil_kl_kg)/1000 oil_kl_kg,sum(oil_kl_money)/10000 oil_kl_money,sum(coil_kl_kg)/1000 coil_kl_kg, ");
		sql.append("sum(coil_kl_money)/10000 coil_kl_money,sum(coil_erp_kg)/1000 coil_erp_kg ");
		sql.append(" from t_car_consume s   ");
		sql.append(" where DATE_FORMAT(s.rpt_date,'%Y-%m') between ? and ? ");
	    return dao.queryForObject(sql.toString(),CarConsume.class,startMonth,endMonth);
	}
	
	public OtherConsume  rptOtherConsume(String startMonth,String endMonth){
		StringBuilder sql=new StringBuilder();
		sql.append("select  ");
		sql.append(" sum(oil_count_kg)/1000 oil_count_kg,sum(coil_count_kg)/1000 coil_count_kg,");
		sql.append(" sum(oil_erp_count_kg)/1000 oil_erp_count_kg,sum(coil_erp_count_kg)/1000 coil_erp_count_kg,");
		sql.append(" sum(oil_kl_count_kg)/1000 oil_kl_count_kg,sum(coil_kl_count_kg)/1000 coil_kl_count_kg,");
		sql.append(" sum(oil_kl_coal) oil_kl_coal,sum(coil_kl_coal) coil_kl_coal,");
		sql.append(" sum(oil_kl_money)/10000 oil_kl_money,sum(coil_kl_money)/10000 coil_kl_money");
		sql.append(" from t_other_consume s   ");
		sql.append(" where DATE_FORMAT(s.rpt_month,'%Y-%m') between ?  and ?");
		
	    return dao.queryForObject(sql.toString(),OtherConsume.class,startMonth,endMonth);
	}
	
	public Workload loadWorkloadByMonth(String month){
		String sql="select * from t_workload_value where DATE_FORMAT(rpt_month,'%Y-%m')=?";
		return dao.queryForObject(sql,Workload.class,month);
	}
	
	public Workload loadWorkloadMonthSum(String startMonth,String endMonth){
		String sql="select sum(well_count) well_count,sum(industry_value) industry_value, sum(added_value) added_value from t_workload_value where DATE_FORMAT(rpt_month,'%Y-%m') between ?  and ?";
		return dao.queryForObject(sql,Workload.class,startMonth,endMonth);
	}
}
