package com.hb.proj.car.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.xframework.dao.core.SpringJdbcDAO;
import com.hb.xframework.dao.util.FieldAttrConverter;
import com.hb.xframework.dao.util.PageModel;
import com.hb.xframework.dao.util.UUIDHexGenerator;


@Service
public class OilMeterService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	/**
	 * 项目部用户通过车信息查询油量计设备(下一步通过设备查询油量数据)
	 * @param args
	 * @param authAssistCodes
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	public PageModel<Map<String,Object>> queryOilMeterFromCar(Map<String,Object> args,Set<String> authAssistCodes,int pageNo,int pageSize){
		List<Object> sqlParams=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder(100);
		sql.append("select * from (");
		sql.append("select g.assist_code,g.org_name belong_org_name,t.*,");
		sql.append("(select count(1) from t_oil_meter m where m.belong_car_id=t.car_id) meter_count,");
		sql.append(" DATE_FORMAT(work_start,'%Y-%m') wk_start");
		sql.append(" from t_car t  ");
		sql.append(" left join t_org g on t.belong_org=g.org_id ");
		sql.append(" ) tab where discard_if=0 ");
		
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("deviceKey"))){
			String[] dns=((String)args.get("deviceKey")).split("[,，;；\\-~\\|\\s]+");
			if(dns.length>1){ //两个参数
				sql.append(" and ((device_name like ?  and device_model like ?) or (device_name like ?  and device_model like ?))");
				sqlParams.add("%"+dns[0]+"%");
				sqlParams.add("%"+dns[1]+"%");
				sqlParams.add("%"+dns[1]+"%");
				sqlParams.add("%"+dns[0]+"%");
			}
			else{
				sql.append(" and (device_name like ? or device_model like ?)");
				sqlParams.add("%"+dns[0]+"%");
				sqlParams.add("%"+dns[0]+"%");
			}
		}
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("carOrCustomNum"))){
			sql.append(" and (car_num like ? or custom_num like ?)");
			sqlParams.add("%"+args.get("carOrCustomNum")+"%");
			sqlParams.add("%"+args.get("carOrCustomNum")+"%");
		}
		
		if(authAssistCodes!=null){ //权限过滤  单位assistCode
			sql.append(getPartSQL(authAssistCodes,sqlParams));
		}
		//客户端指定排序规则
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("sortField"))){
			String sortField=FieldAttrConverter.getFieldName((String)args.get("sortField"));
			sql.append(" order by "+sortField+" "+(args.get("sortAction")==null?"asc":args.get("sortAction")));
		}
		else{
			sql.append(" order by custom_num,car_num");
		}
		Object[] sqlArgs=sqlParams.size()>0?sqlParams.toArray():null;
		return dao.queryForPagedListMap(sql.toString(),pageNo, pageSize,sqlArgs);
	}
	
	/**
	 * 油量计设备管理查询
	 * @param args
	 * @param authAssistCodes
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	public PageModel<Map<String,Object>> queryOilMeter(Map<String,Object> args,Set<String> authAssistCodes,int pageNo,int pageSize){
		boolean showAll=args!=null&&"1".equals(args.get("showAll")) ;//是否明确显示全部含报废车辆
		List<Object> sqlParams=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder(100);
		sql.append("select * from (");
		sql.append("select g.assist_code,g.org_name belong_org_name,t.car_num,t.custom_num,t.device_model,t.device_name,t.discard_if,b.box_name,");
		sql.append(" m.*");
		sql.append(" from t_oil_meter m  inner join t_car t on m.belong_car_id=t.car_id");
	    sql.append(" left join t_oil_box b on m.box_id=b.record_id");
		sql.append(" left join t_org g on t.belong_org=g.org_id ");
		sql.append(" ) tab where 1=1 ");
		if(!showAll){ //并非显示全部,则去掉报废车
			sql.append(" and discard_if=0  ");
		}
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("deviceKey"))){
			String[] dns=((String)args.get("deviceKey")).split("[,，;；\\-~\\|\\s]+");
			if(dns.length>1){ //两个参数
				sql.append(" and ((device_name like ?  and device_model like ?) or (device_name like ?  and device_model like ?))");
				sqlParams.add("%"+dns[0]+"%");
				sqlParams.add("%"+dns[1]+"%");
				sqlParams.add("%"+dns[1]+"%");
				sqlParams.add("%"+dns[0]+"%");
			}
			else{
				sql.append(" and (device_name like ? or device_model like ?)");
				sqlParams.add("%"+dns[0]+"%");
				sqlParams.add("%"+dns[0]+"%");
			}
		}
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("carOrCustomNum"))){
			sql.append(" and (car_num like ? or custom_num like ?)");
			sqlParams.add("%"+args.get("carOrCustomNum")+"%");
			sqlParams.add("%"+args.get("carOrCustomNum")+"%");
		}
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("belongOrg"))){  //入参belongOrg为assistCode
			sql.append(" and assist_code like ?");
			sqlParams.add(args.get("belongOrg")+"%");
		}
		if(authAssistCodes!=null){ //权限过滤  单位assistCode
			sql.append(getPartSQL(authAssistCodes,sqlParams));
		}
		//客户端指定排序规则
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("sortField"))){
			String sortField=FieldAttrConverter.getFieldName((String)args.get("sortField"));
			sql.append(" order by "+sortField+" "+(args.get("sortAction")==null?"asc":args.get("sortAction")));
		}
		else{
			sql.append(" order by custom_num,car_num,box_num");
		}
		Object[] sqlArgs=sqlParams.size()>0?sqlParams.toArray():null;
		return dao.queryForPagedListMap(sql.toString(),pageNo, pageSize,sqlArgs);
	}
	
	private String getPartSQL(Set<String> authAssistCodes,List<Object> sqlParams){
		StringBuilder sql=new StringBuilder();
		for(String c : authAssistCodes){
			sql.append("or assist_code like ?");
			sqlParams.add(c+"%");
		}
		return "and ("+sql.substring(2)+")";
	}
	
	public Map<String, Object> getOilMeter(String recordId) {
		StringBuilder sql=new StringBuilder();
		sql.append("select m.*,t.car_num,b.box_name,b.record_id oil_box_id  from t_oil_meter m ");
		sql.append(" left join t_car t on m.belong_car_id=t.car_id");
		sql.append(" left join t_oil_box b on m.box_id=b.record_id");
		sql.append(" where m.record_id=?");
		return dao.queryForSingleMap(sql.toString(), recordId);
	}
	
	
	
	
	public boolean addOilMeter(Map<String, Object> oilMeter) {
		if(oilMeter==null){
			return false;
		}

		
		UUIDHexGenerator uuid=UUIDHexGenerator.getInstance();
		oilMeter.put("recordId", uuid.generate());
		oilMeter.put("modifyTime", new Date());
		dao.insert(oilMeter,"t_oil_meter");
		
		
		return true;
	}
	
	/**
	 * 建立车、油箱、油量计关联
	 * @param carId
	 * @param boxSerial
	 * @param oilBoxId
	 * @param oilMeterId
	 */
	/*private void createCarBoxMeterMapping(String carId,String boxSerial,String oilBoxId,String oilMeterId){
		String sql="update t_car set oil_box_id=?,oil_meter_id=? where car_id=?";
		if("2".equals(boxSerial)){
			sql="update t_car set oil_box_id2=?,oil_meter_id2=? where car_id=?";
		}
		dao.getJdbcTemplate().update(sql, oilBoxId,oilMeterId,carId);
	}*/
	
	public boolean updateOilMeter(Map<String, Object> oilMeter) {
	
		oilMeter.put("modifyTime", new Date());
		dao.update(oilMeter,"t_oil_meter","record_id");
		
		
		return true;
	}

	public boolean deleteOilMeter(String recordId) {
		String sql="delete from t_oil_meter  where record_id=?";
		dao.getJdbcTemplate().update(sql,recordId);  //删除油量计后，也清除和车辆的关联关系
		
		return true;
	}
	
	//检测油位计设备识别码是否在用
	public boolean existsMeterNum(String meterNum,String recordId){
		Map<String,Object> rst=null;
		if(StringUtils.isNotEmpty(recordId)){
			rst=dao.queryForSingleMap("select meter_num from t_oil_meter m where meter_num=? and record_id!=?", meterNum,recordId);
		}
		else{
			rst=dao.queryForSingleMap("select meter_num from t_oil_meter m where meter_num=?", meterNum);
		}
		return rst!=null&&rst.size()>0;
		
	}
	
	/**
	 * 查询指定车辆的油箱配置
	 * @param carId
	 * @return
	 */
	public List<Map<String,Object>> loadCarBoxs(String carId){
		String sql="select record_id val,box_num txt from t_oil_meter where belong_car_id=?";
		return dao.queryForListMap(sql, carId);
	}
	
	//从采集数据中同步油量计设备,临时用(适用一车一油位计且按规律编号，多油位计不适用)
	public Integer addMetersFromSampling(){
		String sql="select dtu_num,(case when left(dtu_num,1)='L' then CONCAT('辽',dtu_num) else dtu_num end) car_num from t_oil_current where  not exists(select meter_num from t_oil_meter where meter_num=dtu_num)";
		Map<String,Object> rst=dao.queryForMapping(sql, "dtu_num", "car_num");
		if(rst==null||rst.size()==0){
			return 0;
		}
		
		sql="select car_id,replace(car_num,'-','') car_num from t_car where replace(car_num,'-','') in ('"+StringUtils.join(rst.values(),"','")+"')";
		Map<String,Object> carMap=dao.queryForMapping(sql,"car_num","car_id");
		
		List<Object[]> params=new ArrayList<Object[]>(rst.size());
		List<Object> values=null;
		UUIDHexGenerator  uuid=UUIDHexGenerator.getInstance();
		for(String dtuNum : rst.keySet()){
			if(carMap.get(rst.get(dtuNum))==null){
				continue;
			}
			values=new ArrayList<Object>(6);
			values.add(uuid.generate());
			//values.add(carMap.get(rst.get(dtuNum)));
			values.add(dtuNum);
			values.add(dtuNum.startsWith("辽")?"3318":"DTU");
			values.add(new Date());
			values.add("sys");
			params.add(values.toArray());
		}
		
		if(params.size()==0){
			return 0;
		}
		
		sql="insert into t_oil_meter(record_id,meter_num,transfer_type,modify_time,modifier) values(?,?,?,?,?)";
		
		dao.getJdbcTemplate().batchUpdate(sql, params);
		
		//增加油量计与车辆的关联
		List<Object[]> params2=new ArrayList<Object[]>(rst.size());
		Object[] values2=null;
		for(Object[] args : params){
			values2=new Object[2];
			values2[0]=args[0];  //args[0]:t_oil_meter recordId;
			values2[1]=carMap.get(rst.get(args[1])); //args[1]:t_oil_meter meter_num
			params2.add(values2);
		}
		
		sql="update t_car set oil_meter_id=? where car_id=?";
		dao.getJdbcTemplate().batchUpdate(sql, params2);
		
		return params.size();
	}
	
	
	/**
	 * 更新油量设备配置表中体积换算系数
	 * @param carId
	 * @param ratio
	 * @param ratio2
	 */
	public void updateCarVolumeRatio(String carId,Double ratio,Double ratio2){
		if(ratio==null||ratio.doubleValue()<=0){ //油箱1系数都无效就不再处理
			return;
		}
		String sql="select volume_ratio from t_oil_meter where belong_car_id=? and box_num='1'";
		Map<String,Object> rst=dao.queryForSingleMap(sql, carId);
		if(rst!=null&&rst.get("volumeRatio")!=null){ //已经标定过的再标定就进行累计标定
			ratio=((Number)rst.get("volumeRatio")).doubleValue()*ratio.doubleValue();
		}
		sql="update t_oil_meter set volume_ratio=?,ratio_modify_time=? where belong_car_id=? and box_num='1'";
		dao.getJdbcTemplate().update(sql,ratio,new Date(),carId);
		
		
		if(ratio2==null||ratio2.doubleValue()<=0){ 
			return;
		}
		sql="select volume_ratio from t_oil_meter where belong_car_id=? and box_num='2'";
		rst=dao.queryForSingleMap(sql, carId);
		if(rst!=null&&rst.get("volumeRatio")!=null){ //已经标定过的再标定就进行累计标定
			ratio2=((Number)rst.get("volumeRatio")).doubleValue()*ratio2.doubleValue();
		}
		sql="update t_oil_meter set volume_ratio=?,ratio_modify_time=? where belong_car_id=? and box_num='2'";
		dao.getJdbcTemplate().update(sql,ratio2,new Date(),carId);
	}
}
