package com.hb.proj.car.service;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.proj.model.CarConsume;
import com.hb.proj.model.OtherConsume;
import com.hb.proj.model.Wpg;
import com.hb.xframework.dao.core.SpringJdbcDAO;

/**
 * 用能用水累计同比-表6
 * @author hb
 *
 */

@Service
public class YearSumCompareService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	/**
	 * 油能源统计-根据要求从t_other_consume取（各项目部录入月累计消耗数据），不再通过车辆累加2020.9.15
	 * @param rptYear
	 * @return
	 */
	public OtherConsume oilSumRpt(String startMonth,String endMonth){
		StringBuilder sql=new StringBuilder();
		sql.append("select  ");
		sql.append(" sum(oil_count_kg)/1000 oil_count_kg,sum(coil_count_kg)/1000 coil_count_kg,");
		sql.append(" sum(oil_erp_count_kg)/1000 oil_erp_count_kg,sum(coil_erp_count_kg)/1000 coil_erp_count_kg,");
		sql.append(" sum(oil_kl_count_kg)/1000 oil_kl_count_kg,sum(coil_kl_count_kg)/1000 coil_kl_count_kg,");
		sql.append(" sum(oil_kl_coal) oil_kl_coal,sum(coil_kl_coal) coil_kl_coal,");
		sql.append(" sum(oil_kl_money)/10000 oil_kl_money,sum(coil_kl_money)/10000 coil_kl_money");
		sql.append(" from t_other_consume s   ");
		sql.append(" where DATE_FORMAT(s.rpt_month,'%Y-%m') between ?  and ?");
		return dao.queryForObject(sql.toString(),OtherConsume.class,startMonth,endMonth);
	}
	
	/**
	 * 油能源统计 全年12个月
	 * @param rptYear
	 * @return
	 */
	public CarConsume oilYearSumRpt(String rptYear){
		return oilYearSumRpt(rptYear,null);
	}
	
	
	/**
	 * 油能源统计
	 * @param rptYear
	 * @param maxMM 最大月份
	 * @return
	 */
	public CarConsume oilYearSumRpt(String rptYear,Integer maxMM){
		List<Object> params=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder();
		sql.append("select  sum(oil_real_cost)/1000 oil_real_cost,sum(coil_real_cost)/1000 coil_real_cost,sum(oil_coal)oil_coal, sum(coil_coal) coil_coal, ");
		sql.append(" sum(oil_kl_money)/10000 oil_kl_money,sum(coil_kl_money)/10000 coil_kl_money");
		sql.append(" from t_car_consume s   ");
		sql.append(" where DATE_FORMAT(s.rpt_date,'%Y') = ? ");
		params.add(rptYear);
		if(maxMM!=null){
			sql.append(" and DATE_FORMAT(s.rpt_date,'%m') <= ? ");
			params.add(String.format("%02d",maxMM));
		}
		
		return dao.queryForObject(sql.toString(),CarConsume.class,params.toArray());
	}
	
	/**
	 * 水电气能耗  全年12月
	 * @param rptYear
	 * @return
	 */
	public Wpg wpgYearSumRpt(String rptYear){
		return wpgYearSumRpt(rptYear,null);
	}
	
	/**
	 * 水电气能耗
	 * @param rptYear
	 * @return
	 */
	public Wpg wpgYearSumRpt(String startMonth,String endMonth){
		StringBuilder sql=new StringBuilder();
		
		sql.append("SELECT sum(water_count)/10000  water_count,sum(water_cost)/10000 water_cost, ");
		sql.append(" sum(power_count)/10000  power_count,sum(power_cost)/10000 power_cost, ");
		sql.append(" sum(ngas_count)/10000  ngas_count,sum(ngas_cost)/10000 ngas_cost, ");
		sql.append(" sum(lgas_count)/1000  lgas_count,sum(lgas_cost)/10000  lgas_cost,sum(power_coal) power_coal, sum(lgas_coal) lgas_coal,sum(ngas_coal) ngas_coal");
		sql.append(" FROM t_wpg_account  w  where  DATE_FORMAT(rpt_month,'%Y-%m') between ?  and ? ");
		
		return dao.queryForObject(sql.toString(),Wpg.class,startMonth,endMonth);
	}
	
	
}
