package com.hb.proj.excel;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;

import com.hb.xframework.sqlmapper.parse.OgnlExecuter;

public class ExcelSupport {

	/**
	 * 获取单元格的字符形式值
	 * @param cell
	 * @return
	 */
	public static String getCellStringValue(Cell cell){
		if(cell==null){
			return "";
		}
		String cellValue="";
		
		CellType ct=cell.getCellTypeEnum();
		if(ct==CellType.BLANK||ct==CellType.ERROR){
			cellValue="";
		}
		else if(ct==CellType.NUMERIC||ct==CellType.FORMULA){
			if (HSSFDateUtil.isCellDateFormatted(cell)) {
				Date date = cell.getDateCellValue();
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
				cellValue = sdf.format(date);
			} else {
				cellValue=String.valueOf(cell.getNumericCellValue());
			}
		}
		else if(ct==CellType.STRING){
			cellValue = cell.getRichStringCellValue().toString().trim();
		}
		return cellValue;
	}
	
	
	public static CellStyle createCellStyle(Workbook workbook,short color,short size){
		// 设置字体
		CellStyle cellStyle = workbook.createCellStyle();
		Font redFont = workbook.createFont();
		//颜色
		redFont.setColor(color);
		//设置字体大小
		redFont.setFontHeightInPoints(size);
		//字体
		//redFont.setFontName("宋体");
		cellStyle.setFont(redFont);
		
		cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
		
		cellStyle.setBorderBottom(BorderStyle.THIN);
		 
		return cellStyle;
	}
	
	/**
	 * 设置单元格的值
	 * @param cell
	 * @param val
	 */
	public static void setValue(Cell cell,Object val){
		if(val==null){
			return;
		}
		if(val instanceof Number){
			cell.setCellValue(((Number)val).doubleValue());
		}
		else if(val instanceof Date){
			cell.setCellValue((Date)val);
		}
		else if(val instanceof String && ((String)val).startsWith("=")){
			cell.setCellFormula(((String)val).substring(1));
		}
		else{
			cell.setCellValue(val==null?null:String.valueOf(val).replaceAll("<br>|<BR>", "\n"));
		}
	}
	
	/**
	 * 插入新行
	 * @param sheet
	 * @param startRow 需要在其后插入行的索引
	 * @param rows  需要插入的行数
	 */
	public static void insertRow(Sheet sheet, int startRow, int rows) {
	    int afterRow=startRow;
	    startRow++;
	    if(startRow<=sheet.getLastRowNum()){
	    	sheet.shiftRows(startRow, sheet.getLastRowNum(), rows, true, false);
	    }
	    if(rows<0){
	    	return;
	    }
        Row targetRow = null;//参照目标
        targetRow = sheet.getRow(afterRow);

        for (int i = 0; i < rows; i++) {
            Row sourceRow = null;//原始位置
          
            Cell sourceCell = null;
            Cell targetCell = null;
            sourceRow = sheet.createRow(startRow);
            sourceRow.setHeight(targetRow.getHeight());

            for (int m = targetRow.getFirstCellNum(); m < targetRow.getPhysicalNumberOfCells(); m++) {
                sourceCell = sourceRow.createCell(m);
                targetCell = targetRow.getCell(m);
                sourceCell.setCellStyle(targetCell.getCellStyle());
                sourceCell.setCellType(targetCell.getCellTypeEnum());
            }
            startRow++;
        }

    }
	
	public static void insertRow(Sheet sheet, int startRow, int rows,InsertRowCallback  rowCallback) {
	    int afterRow=startRow;
	    startRow++;
	    if(startRow<sheet.getLastRowNum()){
	    	sheet.shiftRows(startRow, sheet.getLastRowNum(), rows, true, false);
	    }
        Row targetRow = null;//参照目标
        targetRow = sheet.getRow(afterRow);

        for (int i = 0; i < rows; i++) {
            Row sourceRow = null;//原始位置
          
            Cell sourceCell = null;
            Cell targetCell = null;
            sourceRow = sheet.createRow(startRow);
            sourceRow.setHeight(targetRow.getHeight());

            for (int m = targetRow.getFirstCellNum(); m < targetRow.getPhysicalNumberOfCells(); m++) {
                sourceCell = sourceRow.createCell(m);
                targetCell = targetRow.getCell(m);
                sourceCell.setCellStyle(targetCell.getCellStyle());
                sourceCell.setCellType(targetCell.getCellTypeEnum());
            }
            rowCallback.process(sheet,sourceRow);
            startRow++;
        }

    }
	
	public static void insertColumn(Sheet sheet,int startColumn,int columns){
		int afterCell=startColumn;
		Row row=null;
		Cell targetCell,newCell=null;
		int columnWith=sheet.getColumnWidth(startColumn);
		for (int j = 0, rlen = sheet.getPhysicalNumberOfRows(); j < rlen; j++) {
			row = sheet.getRow(j);
			targetCell = row.getCell(startColumn);
			if(targetCell==null){
				break;
			}
			for (int i = afterCell+1,len=columns+afterCell; i <= len ; i++) {
				sheet.setColumnWidth(i, columnWith);
				newCell = row.createCell(i);
				newCell.setCellStyle(targetCell.getCellStyle());
				newCell.setCellType(targetCell.getCellTypeEnum());
				
			}
		}

	}
	
	/**
	 * 判断是否为空行
	 * @param sheet
	 * @param rowIndex
	 * @return
	 */
	public static boolean isNullRow(Sheet sheet, int rowIndex) {
		Row row = sheet.getRow(rowIndex);
		if (row == null) {
			return true;
		}
		String val = null;
		for (int c = 0, clen = row.getPhysicalNumberOfCells(); c < clen; c++) {
			val = getCellStringValue(row.getCell(c));
			if (StringUtils.isNotEmpty(val)) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * 执行单值替换表达式
	 * @param express
	 * @param context
	 * @return
	 */
	public static Object executeContextExp(String express,Map<String,Object> context){
		boolean isStr=exists(express,"\\$=\\s*\\S+\\s*\\{|\\}\\s*\\S+");
		express=express.replaceFirst("\\$=", "");
		Pattern pattern=Pattern.compile("\\{([^\\{]+)\\}");  
		Matcher matcher=pattern.matcher(express);
		String findItm=null;
		Object rst=null;
		while(matcher.find()){
			findItm=matcher.group(1);
			rst=OgnlExecuter.getValue(findItm, context);
			if(!isStr){
				return rst;
			}
			express=express.replace("{"+findItm+"}", rst!=null?String.valueOf(rst):"");
		}
		return express;
	}
	
	/**
	 * 获得行中的所有表达式
	 * @param row
	 * @return
	 */
	public static Map<String,List<String>> getFields(Row row){
		int clen=row.getPhysicalNumberOfCells();
		List<String>  fields=new ArrayList<String>(clen);
		List<String>  groupFields=new ArrayList<String>();
		Cell cell=null;
		String cellVal=null;
		String field=null;
		for(int c=0;c<clen;c++){
			cell=row.getCell(c);
			cellVal=ExcelSupport.getCellStringValue(cell);
			if(StringUtils.isEmpty(cellVal)||cellVal.indexOf("&=")<0){  //不需要表达式的
				fields.add(null);
				continue;
			}
			if(cellVal.contains("group")){
				groupFields.add(String.valueOf(c));
			}
			field=extractString(cellVal, "^&=(?:group\\()?(?:\\w+\\.)?([\\w-]+)", 1, 0);
			fields.add(field);
			cell.setCellValue(""); //获取表达式后清空,避免没数据填充时,表达式还在
		}
		Map<String,List<String>>  rst=new HashMap<String,List<String>>();
		rst.put("fields", fields);
		rst.put("groupFields", groupFields);
		return rst;
	}
	
	public static List<String> getFieldExpress(Row row){
		int clen=row.getPhysicalNumberOfCells();
		List<String>  fields=new ArrayList<String>(clen);
		Cell cell=null;
		String cellVal=null;
		String field=null;
		for(int c=0;c<clen;c++){
			cell=row.getCell(c);
			cellVal=ExcelSupport.getCellStringValue(cell);
			
			field=extractString(cellVal, "^&=(?:group\\()?(?:\\w+\\.)?([\\w-]+)", 1, 0);
			if(field==null){
				field=cellVal;
			}
			fields.add(field);
		}
		return fields;
	}
	
	/**
	 * 分组合并单元格
	 * @param startIdx
	 * @param endIdx
	 * @param sheet
	 * @param groupIdxs
	 */
	public static void processGroups(int startIdx,int endIdx,Sheet sheet,List<String> groupIdxs){
		List<int[]>  preGroupAreas=null;
		List<int[]>  tempGroupAreas=null;
		for(String gidx : groupIdxs){
			tempGroupAreas=null;
			if(preGroupAreas==null){
				tempGroupAreas=processGroup(startIdx,endIdx,sheet,Integer.parseInt(gidx));
			}
			else{
				for(int[]  idxs : preGroupAreas){
					if(idxs[0]<idxs[1]){
						if(tempGroupAreas==null){
							tempGroupAreas=new ArrayList<int[]>();
						}
						tempGroupAreas.addAll(processGroup(idxs[0],idxs[1],sheet,Integer.parseInt(gidx)));
					}
				}
			}
			preGroupAreas=tempGroupAreas;
			
		}
	}
	
	public static List<int[]> processGroup(int startIdx,int endIdx,Sheet sheet,int groupIdx){
		Cell cell=null;
		String cellVal=null,preCellVal=null;
		int preStart=startIdx,i=startIdx;
		List<int[]>  groupAreas=new ArrayList<int[]>(); //存在分组各区间
		for(;i<=endIdx;i++){
			cell=sheet.getRow(i).getCell(groupIdx);
			cellVal=ExcelSupport.getCellStringValue(cell);
			if(preCellVal!=null&&!cellVal.equals(preCellVal)){  //不同的数据出现，上面的就要合并
				sheet.addMergedRegion(new CellRangeAddress(preStart,i-1,groupIdx,groupIdx));
				groupAreas.add(new int[]{preStart,i-1});
				preStart=i;
			}
			preCellVal=cellVal;
			
		}
		if(preStart<i-1){
			sheet.addMergedRegion(new CellRangeAddress(preStart,i-1,groupIdx,groupIdx));
			groupAreas.add(new int[]{preStart,i-1});
		}
		return groupAreas;
	}
	
	/**
	 * 获得指定捕获组(从0开始计数)
	 * 
	 * @param src
	 * @param exp
	 * @return
	 */
	public static String extractString(String src, String exp, int groupIndex,int fromIndex) {
		if (src == null || exp == null) {
			return null;
		}
		Pattern p = Pattern.compile(exp);
		Matcher m = p.matcher(src);
		return m.find(fromIndex) ? m.group(groupIndex) : null;
	}
	
	
	public static boolean  exists(String src,String regExp){
		if(src==null||src.length()==0){
			return false;
		}
		Pattern pattern=Pattern.compile(regExp);  
		Matcher matcher=pattern.matcher(src);
		return matcher.find();
	}
	
	/**
	 * 从单元个表达式中获取数据集名称
	 * @param cell
	 * @return
	 */
	public static String getDatasetName(Cell cell){
		String cellVal=getCellStringValue(cell);
		if(cellVal.indexOf("&=")<0){
			return null;
		}
		String dsName=extractString(cellVal, "^&=(?:group\\()?(\\w+)\\.(\\w+)", 1, 0);
		return dsName!=null?dsName:"ds";
	}
	
	
	
}
