package com.hb.proj.excel;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;

public class TableExporter {

	public static HSSFWorkbook buildExcel(Table tab){
		HSSFWorkbook workbook = new HSSFWorkbook();
        HSSFSheet sheet = workbook.createSheet(tab.getTitle());
        sheet.setDefaultColumnWidth(10);
        sheet.setDefaultRowHeight((short)450);
        //创建基本单元格
        buildBase(tab,workbook,sheet);
        
        fillCell(tab,sheet);
        
        //插入标题
        int lockRowExtr=0;
        if(StringUtils.isNotEmpty(tab.getSecTitle())){
        	insertSecTitleRow(tab,workbook,sheet);
        	lockRowExtr+=1;
        }
        insertTitleRow(tab,workbook,sheet);
        lockRowExtr+=1;
        
        if(tab.getFreezeClm()!=null&&tab.getFreezeRow()!=null){
        	 sheet.createFreezePane(tab.getFreezeClm(), tab.getFreezeRow()+lockRowExtr);
        }
       
        
        return workbook;
	}
	
	//插入标题栏
	private static void insertTitleRow(Table tab,HSSFWorkbook workbook,HSSFSheet sheet){
		sheet.shiftRows(0, sheet.getLastRowNum(),1,true,false);
        HSSFRow row=sheet.createRow(0);
        row.setHeight((short)550);
        HSSFCell cell=row.createCell(0);
        CellRangeAddress region = new CellRangeAddress(cell.getRowIndex(), cell.getRowIndex(), cell.getColumnIndex(), cell.getColumnIndex()+tab.getColumnCount()-1);
		sheet.addMergedRegion(region);
		
		
		 Font font=workbook.createFont();
		 font.setFontHeightInPoints((short) 20);
	     font.setFontName("Courier New ");
	     font.setBold(true);
	     CellStyle style=workbook.createCellStyle();
	     style.setVerticalAlignment(VerticalAlignment.CENTER);
	     style.setAlignment(HorizontalAlignment.CENTER);
	     style.setFont(font);
	     cell.setCellStyle(style);
	     cell.setCellValue(tab.getTitle());
	}
	
	//插入二级标题栏
	private static void insertSecTitleRow(Table tab,HSSFWorkbook workbook,HSSFSheet sheet){
		sheet.shiftRows(0, sheet.getLastRowNum(),1,true,false);
        HSSFRow row=sheet.createRow(0);
        row.setHeight((short)550);
        HSSFCell cell=row.createCell(0);
        CellRangeAddress region = new CellRangeAddress(cell.getRowIndex(), cell.getRowIndex(), cell.getColumnIndex(), cell.getColumnIndex()+tab.getColumnCount()-1);
		sheet.addMergedRegion(region);
		
		
		 Font font=workbook.createFont();
		 font.setFontHeightInPoints((short) 14);
	     font.setFontName("Courier New ");
	     //font.setBold(true);
	     CellStyle style=workbook.createCellStyle();
	     style.setVerticalAlignment(VerticalAlignment.CENTER);
	     style.setAlignment(HorizontalAlignment.LEFT);
	     style.setFont(font);
	     cell.setCellStyle(style);
	     cell.setCellValue(tab.getSecTitle());
	}	
	
	//在excel中单元格合并后，索引还是从原有格算起
	private static void fillCell(Table tab,HSSFSheet sheet){
		TableTd[]  tds=tab.getTds();
		int maxClm=tab.getColumnCount();
		int clmIdx=0,rowIdx=0;
		HSSFCell cell=null;
		CellRangeAddress region=null;
		Map<String,String> cellUseTag=new HashMap<String,String>(); //已经被使用的单元格
		String val=null;
		for(int i=0,len=tds.length;i<len;){
			if(clmIdx>=maxClm){ //换行
				rowIdx+=1;
				clmIdx=0;
			}
			if(cellUseTag.containsKey(clmIdx+"_"+rowIdx)){ //已经使用过（被合并单元格占用）
				clmIdx++;
				continue;
			}
			cell=sheet.getRow(rowIdx).getCell(clmIdx);
			
			val=tds[i].getTxt();
			if(val.indexOf("<br>")>0){
				cell.setCellValue(new HSSFRichTextString(val.replace("<br>", "\r\n")));
				sheet.getRow(rowIdx).setHeight((short)700);
			}
			else{
				cell.setCellValue(tds[i].getTxt());
			}
			
			if(tds[i].getColspan()>1||tds[i].getRowspan()>1){  //合并单元格
				region = new CellRangeAddress(cell.getRowIndex(), cell.getRowIndex()+tds[i].getRowspan()-1, cell.getColumnIndex(), cell.getColumnIndex()+tds[i].getColspan()-1);
				sheet.addMergedRegion(region);
				clmIdx+=tds[i].getColspan();  //下一列
				addUseTag(cellUseTag,region.getFirstRow(),region.getLastRow(),region.getFirstColumn(),region.getLastColumn());
			}
			else{
				addUseTag(cellUseTag,rowIdx,clmIdx);
				clmIdx+=1; //下一列
			}
			i++; //只有实际使用了Td才向后迭代
		}
	}
	
	
	private static void addUseTag(Map<String,String> cellUseTag,int rowIdx,int clmIdx){
		addUseTag(cellUseTag,rowIdx,rowIdx,clmIdx,clmIdx);
	}
	
	private static void addUseTag(Map<String,String> cellUseTag,int startRow,int endRow,int startClm,int endClm){
		int clmIdx=startClm;
		while(startRow<=endRow){
			clmIdx=startClm;
			while(clmIdx<=endClm){
				cellUseTag.put(clmIdx+"_"+startRow, "1");
				clmIdx++;
			}
			startRow++;
		}
	}
	
	
	private static void buildBase(Table tab,HSSFWorkbook workbook, HSSFSheet sheet){
		HSSFCellStyle cellStyle=workbook.createCellStyle();
        cellStyle.setWrapText(true);
        cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        cellStyle.setAlignment(HorizontalAlignment.CENTER);
        cellStyle.setBorderTop(BorderStyle.THIN);
        cellStyle.setBorderRight(BorderStyle.THIN);
        cellStyle.setBorderBottom(BorderStyle.THIN);
        cellStyle.setBorderLeft(BorderStyle.THIN);
        cellStyle.setTopBorderColor(IndexedColors.BLACK.index);
        cellStyle.setRightBorderColor(IndexedColors.BLACK.index);
        cellStyle.setBottomBorderColor(IndexedColors.BLACK.index);
        cellStyle.setLeftBorderColor(IndexedColors.BLACK.index);
        HSSFRow row=null;
        HSSFCell cell=null;
        for(int r=0,len=tab.getRowCount();r<len;r++){
        	row = sheet.createRow(r);
        	for(int c=0,clen=tab.getColumnCount();c<clen;c++){
        		cell=row.createCell(c);
        		cell.setCellStyle(cellStyle);
        	}
        }
        //设置各列宽度
        int[] clmw=tab.getClmWidth();
        for(int c=0,clen=tab.getColumnCount();c<clen;c++){
        	sheet.setColumnWidth(c, (int)(clmw[c]*35.7));  //35.7  px转poi单元格宽度
        }
	}
	
	
}
