package com.hb.proj.input.controller;

import java.io.File;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.hb.proj.car.service.CarRptUtils;
import com.hb.proj.car.service.CarService;
import com.hb.proj.excel.ExcelUtils;
import com.hb.proj.excel.imp.ExcelDataExtractor;
import com.hb.proj.excel.imp.ExcelResult;
import com.hb.proj.input.service.CarConsumeService;
import com.hb.proj.model.CarConsume;
import com.hb.proj.model.OtherConsumeBO;
import com.hb.proj.sysdata.service.OrgService;
import com.hb.proj.sysdata.service.SortCodeService;
import com.hb.proj.utils.DataAuthUtils;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.LayGridJsonUtils;
import com.hb.proj.utils.MySessionUser;
import com.hb.xframework.dao.util.PageModel;
import com.hb.xframework.util.DateUtil;
import com.hb.xframework.util.MapUtils;
import com.hb.xframework.util.SessionThreadLocal;


@Controller
@RequestMapping("/**/consume")
public class CarConsumeController {
	
	@Autowired
	private CarConsumeService service;
	
	@Autowired
	private CarService  carService;
	
	@Autowired
	private OrgService  orgService;
	
	@Autowired
	private SortCodeService  codeService;

	/**
	 * 加载上报数据，没有则提供空白行(本部门的车辆基本信息)供填写(页面上报列表应以已上报数据为主为结果集（同一车不同地区是多条记录）)
	 * @param request
	 * @param response
	 */
	@RequestMapping("/load")
	public void load(String carOrCustomNum,String rptMonth,Integer page,Integer limit,HttpServletResponse response){
		try{
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			if(su==null){  //测试暂时不用
				LayGridJsonUtils.error(response, "缺少登录信息");
				return;
			}
			Set<String>  authAssistCode=DataAuthUtils.getAuthOrgAssistCode(su);
			if(authAssistCode==null||authAssistCode.size()==0){
				LayGridJsonUtils.error(response, "缺少上报单位信息");
				return;
			}
			if(StringUtils.isEmpty(rptMonth)){
				Calendar ca=Calendar.getInstance();
				rptMonth=ca.get(Calendar.YEAR)+"-"+String.format("%02d", ca.get(Calendar.MONTH)); //默认上报上个月
			}
			PageModel<Map<String,Object>> pagedData=carService.loadCarForInput(carOrCustomNum,rptMonth,authAssistCode,page, limit);
			if(pagedData==null||pagedData.getData()==null||pagedData.getData().size()==0){
				LayGridJsonUtils.wirte(response, null,0);
				return;
			}
			
			Map<String,Map<String,Object>> carMap=new LinkedHashMap<String,Map<String,Object>>(pagedData.getData().size());
			for(Map<String,Object> car : pagedData.getData()){
				car.put("rptDate", rptMonth+"-01");
				car.put("realAge", getWorkAge((String)car.get("wkStart"),rptMonth));
				carMap.put((String)car.get("carId"), car);
			}
			List<Map<String,Object>> rundatas=service.loadConsume(rptMonth,su.getOrgId(), carMap.keySet());
			if(rundatas==null||rundatas.size()==0){
				LayGridJsonUtils.wirte(response, pagedData.getData(),pagedData.getTotalRow());
				return;
			}
			Map<String,Object>  car=null;
			for(Map<String,Object> run : rundatas){  //车辆数据+运行数据合并
				car=carMap.get((String)run.get("carId"));
				if(car!=null){
					run.putAll(car) ;    //car.putAll(run);以已上报记录为主为结果集，车辆信息中的customNum会覆盖已上报记录中的cuostomNum
					//carMap.remove((String)run.get("carId")); //此时还不能删除，还有同车不同地区的记录需要用
					car.put("hadUs", "1");  //标记为已使用
				}
			}
			//还有上报的车辆也要加入结果集中
			for(String key : carMap.keySet()){
				car=carMap.get(key);
				if(!car.containsKey("hadUs")){
					rundatas.add(car);
				}
			}
			
			LayGridJsonUtils.wirte(response, rundatas,rundatas.size(),"yyyy-MM-dd");
		}
		catch(Exception e){
			e.printStackTrace();
			LayGridJsonUtils.error(response, "服务出现错误");
		}
		
	}
	
	
	
	/**
	 * 保存车辆油耗上报数据
	 * @param jsonDatas
	 * @param  oil92 92汽油总量 来自 other上报
	 * @param  oil95 95汽油总量 来自 other上报
	 * @param response
	 */
	@RequestMapping("/save")
	public void save(String jsonDatas,Double oil92,Double oil95,HttpServletResponse response){
		try{
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			if(su==null||StringUtils.isEmpty(su.getOrgId())){  
				JsonOutUtils.returnError(response, "缺少登录信息或上报单位,请用非管理账号登录后再试");
				return;
			}
			
			List<CarConsume> consumes=JSON.parseObject(jsonDatas,new TypeReference<List<CarConsume>>() {});
			if(consumes==null||consumes.size()==0){
				JsonOutUtils.returnOk(response);
				return;
			}
			
			//检测当前是否在上报期限内
			String limitRst=consumes.get(0).getRptDate()==null?null:InputDateLimit.check(DateUtil.format(consumes.get(0).getRptDate(), "yyyy-MM"));
			if(limitRst!=null){
				JsonOutUtils.returnError(response,limitRst);
				return;
			}
			
			service.saveConsume(consumes,CarRptUtils.getOilWeightRatio(oil92, oil95));
			JsonOutUtils.returnOk(response);
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
		
	}
	
	/**
	 * 根据投产月份和统计月份计算实际使用年数
	 * @param startMonth
	 * @param rptMonth
	 * @return
	 */
	private Integer getWorkAge(String startMonth,String rptMonth){
		if(StringUtils.isEmpty(startMonth)||StringUtils.isEmpty(rptMonth)){
			return null;
		}
		String[] ym=startMonth.split("-"),ym2=rptMonth.split("-");
		int age=Integer.parseInt(ym2[0])-Integer.parseInt(ym[0]);
		if(Integer.parseInt(ym2[1])>=Integer.parseInt(ym[1])){  //足月
			return age+1;
		}
		else{
			return age;
		}
	}
	
	
	/**
	 * 加载各项目部每月其它用油
	 * @param rptMonth
	 * @param response
	 */
	@RequestMapping("/other/load")
	public void loadOtherConsume(String rptMonth,HttpServletResponse response){
		MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
		if(su==null||StringUtils.isEmpty(su.getOrgId())){  //测试暂时不用
			JsonOutUtils.returnError(response, "缺少登录信息或上报单位");
			return;
		}
		OtherConsumeBO otherConsume=service.getOtherConsume(rptMonth, su.getOrgId());
		if(otherConsume==null){
			JsonOutUtils.returnOk(response);
		}
		JsonOutUtils.returnOkWithData(response, otherConsume);
	}
	
	/**
	 * 保存各项目部每月其它用油
	 * @param rptMonth
	 * @param oil
	 * @param coil
	 * @param response
	 */
	
	@RequestMapping("/other/save")
	public void saveOtherConsume(String jsonData,HttpServletResponse response){
		MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
		if(su==null){  //测试暂时不用
			JsonOutUtils.returnError(response, "缺少登录信息");
			return;
		}
		
		OtherConsumeBO  othOil=JSON.parseObject(jsonData,new TypeReference<OtherConsumeBO>() {});
		
		//检测当前是否在上报期限内
		String limitRst=InputDateLimit.check(DateUtil.format(othOil.getRptMonth(), "yyyy-MM"));
		if(limitRst!=null){
			JsonOutUtils.returnError(response,limitRst);
			return;
		}
		
		othOil.setModifier(su.getUserName());
		othOil.setOrgId(su.getOrgId());
		service.saveOtherConsume(othOil,true);
		JsonOutUtils.returnOk(response);
	}
	
	
	/**
	 * 导入车辆油耗、其它油耗数据
	 * @param request
	 * @param response
	 */
	@RequestMapping("/importCarConsume")
	public void importCarConsume(HttpServletRequest request,HttpServletResponse response){
		try{
			ExcelResult importRst=ExcelDataExtractor.extractExcelData(request);
			if(importRst==null||importRst.getException()!=null){
				JsonOutUtils.returnError(response,(importRst==null?"导入失败":importRst.getException()));
				return;
			}
			else if(importRst.getListDatas()==null||importRst.getListDatas().size()==0){
				JsonOutUtils.returnError(response,"未提取到符合格式的数据");
				return;
			}
			Map<String,Object> otherImp=importRst.getOtherData();
			if(otherImp==null||otherImp.get("rptOrg")==null||otherImp.get("rptMonth")==null){
				JsonOutUtils.returnError(response,"缺少上报单位或上报月份");
				return;
			}
			
			//检测当前是否在上报期限内
			String limitRst=InputDateLimit.check(DateUtil.format((Date)otherImp.get("rptMonth"), "yyyy-MM"));
			if(limitRst!=null){
				JsonOutUtils.returnError(response,limitRst);
				return;
			}
			
			
			Map<String,Object> orgs=orgService.loadOrgMapping();
			if(!orgs.containsKey(otherImp.get("rptOrg"))){
				JsonOutUtils.returnError(response,"上报单位并非系统内的单位");
				return;
			}
			
			otherImp.put("orgId", orgs.get(otherImp.get("rptOrg"))); //上报单位转为orgId
			
			String jsonData=JSON.toJSONString(otherImp);
			OtherConsumeBO othConsume=JSON.parseObject(jsonData,new TypeReference<OtherConsumeBO>() {});
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			othConsume.setModifier(su!=null?su.getUserName():null);
			
			String[] msgs=processImportData(importRst,othConsume.getRptMonth()); //carNums+ignorCarNums
			
			boolean sameCover="1".equals(importRst.getReqParams().get("sameCover")); //相同车辆、月份是否覆盖库中的
			
			String  jsonConsume=JSON.toJSONString(importRst.getListDatas());
			List<CarConsume> consumes=JSON.parseObject(jsonConsume,new TypeReference<List<CarConsume>>() {});
			
			String equalMsg=equalCheck(consumes,othConsume);
			String sameIgnor=service.addConsumesFromExcel(consumes,othConsume,sameCover);
			
			/**
			 * sameIgnor  禁止覆盖时被忽略的车牌
			 * checkIgnor 业务校验不通过的车牌
			 * validateError 基本数据校验错误信息
			 */
			Map<String,Object>  rtn=MapUtils.build("equalMsg",equalMsg,"sameIgnor",sameIgnor,"checkIgnor",msgs[1],"validateError",importRst.getError().toString(),"successCount",consumes.size());
			JsonOutUtils.returnOkWithData(response,rtn);
			
			
			
			
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "导入出现错误");
		}
	}
	
	/**
	 * 检测项目部上报的总计数据是否和单车的合计数据一致
	 * @param consumes
	 * @param othConsume
	 * @return
	 */
	private String equalCheck(List<CarConsume> consumes,OtherConsumeBO othConsume){
		Double oilErp=null,oilKl=null,coilErp=null,coilKl=null;
		for(CarConsume car : consumes){
			oilErp=CarRptUtils.addDouble(oilErp,car.getOilErp());
			oilKl=CarRptUtils.addDouble(oilKl,car.getOilKl());
			coilErp=CarRptUtils.addDouble(coilErp,car.getCoilErp());
			coilKl=CarRptUtils.addDouble(coilKl,car.getCoilKl());
		}
		Double oilErpTotal=CarRptUtils.addDouble(othConsume.getOilErp92(),othConsume.getOilErp95());
		Double oilKlTotal=CarRptUtils.addDouble(othConsume.getOilKl92(),othConsume.getOilKl95());
		
		//单车合计还要加上其它用油才能与总计比较一致性,比较时只能比较汽油、柴油总量（因为其它用油不区分ERP、昆仑卡）
		Double oilTotal=CarRptUtils.addDouble(oilErpTotal,oilKlTotal);
		Double coilTotal=CarRptUtils.addDouble(othConsume.getCoilErpCount(),othConsume.getCoilKlCount());
		Double oilSum=CarRptUtils.addDouble(oilErp,oilKl);
		oilSum=CarRptUtils.addDouble(oilSum,othConsume.getOilCount());
		
		Double coilSum=CarRptUtils.addDouble(coilErp,coilKl);
		coilSum=CarRptUtils.addDouble(coilSum,othConsume.getCoilCount());
		
		StringBuilder msg=new StringBuilder();
		if(!equalDouble(oilSum,oilTotal)){
			msg.append(String.format("汽油耗量总计【ERP+昆仑卡=%s】和单车合计【单车ERP+单车昆仑卡+其它=%s】不一致;",getStr(oilTotal),getStr(oilSum)));
		}
		if(!equalDouble(coilSum,coilTotal)){
			msg.append(String.format("柴油耗量总计【ERP+昆仑卡=%s】和单车合计【单车ERP+单车昆仑卡+其它=%s】不一致;",getStr(coilTotal),getStr(coilSum)));
		}
		return msg.length()>0?msg.toString():null;
	}
	
	private boolean equalDouble(Double d1,Double d2){
		if(d1==null&&d2==null){
			return true;
		}
		if(d1!=null&&d2!=null&&d1.doubleValue()==d2.doubleValue()){
			return true;
		}
		return false;
	}
	
	private String getStr(Object obj){
		return obj==null?"空":obj.toString();
	}
	
	/**
	 * 对上传的原始数据进行处理，业务约束校验、数据转换（名称转编码）
	 * 目前校验车牌是否与系统内的对应，对应则转为carId
	 */
	@SuppressWarnings("unchecked")
	private String[] processImportData(ExcelResult importRst,Date rptMonth){
		Map<String,Object>  cars=carService.loadCarObjMapping();
		Map<String,Object>  workAreas=carService.loadWorkAreaMapping();
		StringBuilder ignorMsg=new StringBuilder();
		Set<String> impcarNums=new HashSet<String>();
		Iterator<Map<String,Object>> iterator=importRst.getListDatas().iterator();
		Map<String,Object> itm=null;
		Map<String,Object> car=null;
		String carNum=null;
		while(iterator.hasNext()){
			itm=iterator.next();
			itm.put("rptDate", DateUtil.format(rptMonth, "yyyy-MM-dd"));
			carNum=(String)itm.get("carNum");
			carNum=carNum.replaceAll("\\s+", "");
			if(StringUtils.isEmpty(carNum)){
				iterator.remove();
				ignorMsg.append(itm.get("customNum")+";");
				continue;
			}
			itm.put("carNum", carNum);
			if(!cars.containsKey(carNum)){   //||!workAreas.containsKey((String)itm.get("workArea"))
				iterator.remove();
				ignorMsg.append(itm.get("carNum")+";");
				continue;
			}
			else{
				car=(Map<String,Object>)cars.get(carNum);
				impcarNums.add(carNum);
				itm.put("workArea", workAreas.get((String)itm.get("workArea")));
				itm.put("quotaAlternatorId", car.get("quotaAlternatorId"));
				itm.put("quotaRuleId", car.get("quotaRuleId"));
				itm.put("carId", car.get("carId"));
				itm.put("oilType", car.get("oilType"));
				itm.put("deviceName", car.get("deviceName"));
				itm.put("deviceModel", car.get("deviceModel"));
				itm.put("realAge", getWorkAge(DateUtil.format((Date)car.get("workStart"), "yyyy-MM"),DateUtil.format(rptMonth, "yyyy-MM"))); //realAge 必须有，否则不会对基础数据进行自动计算
			}
			
			
			
		}
		
		return new String[]{StringUtils.join(impcarNums,","),(ignorMsg.length()>0?ignorMsg.toString():null)};
	}
	
	/**
	 * 下载导入模板(以车辆基本信息为主)
	 * @param rptMonth
	 * @param response
	 */
	@RequestMapping("/downloadTemp")
	public void downloadTemp(String rptMonth,HttpServletRequest request,HttpServletResponse response){
		try{
			if(StringUtils.isEmpty(rptMonth)){
				JsonOutUtils.writeToResponse(response, "<script>alert('缺少上报月份');window.close();</script>","text/html;charset=UTF-8");
				return;
			}
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			if(su==null){  //测试暂时不用
				JsonOutUtils.writeToResponse(response, "<script>alert('缺少登录信息');window.close();</script>","text/html;charset=UTF-8");
				return;
			}
			Set<String>  authAssistCode=DataAuthUtils.getAuthOrgAssistCode(su);
			if(authAssistCode==null||authAssistCode.size()==0){
				JsonOutUtils.writeToResponse(response, "<script>alert('缺少上报单位信息');window.close();</script>","text/html;charset=UTF-8");
				return;
			}
			PageModel<Map<String,Object>> pagedData=carService.loadCarForInput(null,rptMonth,authAssistCode,1, 1000);
			if(pagedData==null||pagedData.getData()==null||pagedData.getData().size()==0){
				JsonOutUtils.writeToResponse(response, "<script>alert('没有车辆需要上报数据');window.close();</script>","text/html;charset=UTF-8");
				return;
			}
			
			Map<String,Object> workAreaMap=carService.loadWorkAreaMapping();
			
			
			String rptOrg=su.getOrgName()!=null?su.getOrgName():"";
			String templateName=File.separator+"excel"+File.separator+"车辆油耗上报导入模板.xls";
			Map<String,Object> context=new HashMap<String,Object>();
			context.put("ds", pagedData.getData());
			context.put("rptOrg", rptOrg);
			context.put("rptMonth", DateUtil.parse(rptMonth+"-01"));
			
			if(workAreaMap!=null&&workAreaMap.size()>0){
				String[] wa=new String[workAreaMap.size()];
				workAreaMap.keySet().toArray(wa);
				context.put("workAreas",wa);
			}
			
			Workbook wb=ConsumeInputTempBound.bound(context,ExcelUtils.reader(templateName,request.getSession().getServletContext()));
			ExcelUtils.exportExcel(response, "车辆油耗上报模板("+rptOrg+rptMonth+")", wb);
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.writeToResponse(response, "<script>alert('下载导入模板出现错误');window.close();</script>","text/html;charset=UTF-8");
		}
	}
	
}
