package com.hb.proj.input.controller;

import java.util.Calendar;
import java.util.Date;

import org.apache.commons.lang.StringUtils;

import com.hb.proj.car.quota.QuotaConfig;
import com.hb.proj.car.quota.QuotaStandardSupport;
import com.hb.proj.utils.RptMonthUtil;
import com.hb.xframework.util.DateUtil;


/**
 * @author xe
 *
 */
public class InputDateLimit {
	
	/**
	 * 根据统计月份获得月份的开始日期和结束日期(上月16号00:00:00到本月15号23:59:59)
	 * 
	 * @param yyyy-MM
	 * @return start-end 不含end在内
	 */
	public static Date[] getStartEndInRptMonth(String yyyMM){
		String splitDate=String.format("%s-%02d", yyyMM,RptMonthUtil.getMonthSplitDay());
		Date end=DateUtil.parse(splitDate);
		Calendar ca=Calendar.getInstance();
		ca.setTime(end);
		ca.add(Calendar.MONTH, -1);
		return new Date[]{ca.getTime(),end};
	}
	
	/**
	 * 上报数据前进行上报期限的判断
	 * 一般为本月16日零点-下月1日零点前
	 * @param rptMonth
	 * @return
	 */
	public static String check(String rptMonth){
		if(StringUtils.isEmpty(rptMonth)){
			return "未指定上报月份";
		}
		QuotaConfig quotaConfig=QuotaStandardSupport.getConfig();
		if(quotaConfig.getRptStart()==null&&quotaConfig.getRptEnd()==null){ //未设定期限
			return null;
		}
		
		int rptMthNum=Integer.parseInt((rptMonth.split("-"))[1]);
		Calendar ca=Calendar.getInstance();
		int nowMth=ca.get(Calendar.MONTH)+1;
		int day=ca.get(Calendar.DAY_OF_MONTH);
		int diffMth=nowMth-rptMthNum;
		if(diffMth>1){ //延期两个月及以上
			return "已过上报最后期限【"+(rptMthNum+1)+"月"+quotaConfig.getRptEnd()+"日前】";
		}
		if(diffMth==1&&day>quotaConfig.getRptEnd().intValue()){ //次月超过规定日期。属于过期上报
			return "已过上报最后期限【"+(rptMthNum+1)+"月"+quotaConfig.getRptEnd()+"日前】";
		}
		else if(diffMth<0){ //当前月份小于上报数据月份。属于超前上报
			return "早于上报开始时间【"+rptMthNum+"月"+quotaConfig.getRptStart()+"日】";
		}
		else if(diffMth==0&&day<quotaConfig.getRptStart()){ //早于规定的起始日期
			return "早于上报开始时间【"+rptMthNum+"月"+quotaConfig.getRptStart()+"日】";
		}
		return null;
	}
	
	public static void main(String[] args){
		Date[] rst=InputDateLimit.getStartEndInRptMonth("2021-10");
		System.out.println(rst[0]);
		System.out.println(rst[1]);
	}

}
