package com.hb.proj.input.controller;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.hb.proj.input.service.WorkloadService;
import com.hb.proj.model.Workload;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.LayGridJsonUtils;
import com.hb.proj.utils.MySessionUser;
import com.hb.xframework.util.DateUtil;
import com.hb.xframework.util.MapUtils;
import com.hb.xframework.util.SessionThreadLocal;


@Controller
@RequestMapping("/**/workload")
public class WorkloadController {
	
	@Autowired
	private WorkloadService service;

	/**
	 * 加载上报数据，没有则提供空白行供填写
	 * @param request
	 * @param response
	 */
	@RequestMapping("/load")
	public void loadWorkload(HttpServletRequest request,String rptYear,HttpServletResponse response){
		try{
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			if(su==null){  
				LayGridJsonUtils.error(response, "缺少登录信息");
				return;
			}
			if(StringUtils.isEmpty(rptYear)){
				Calendar ca=Calendar.getInstance();
				rptYear=String.valueOf(ca.get(Calendar.YEAR));
			}
			List<Map<String,Object>>  workload=service.queryWorkload(rptYear);
			workload=wrapRecords(rptYear,workload);
			LayGridJsonUtils.wirte(response, workload,workload.size(),"yyyy-MM-dd");
		}
		catch(Exception e){
			e.printStackTrace();
			LayGridJsonUtils.error(response, "服务出现错误");
		}
	}
	
	private List<Map<String,Object>> wrapRecords(String rptYear,List<Map<String,Object>>  wpg){
		Map<String,Map<String,Object>> wpgIdx=null;
		if(wpg!=null&&wpg.size()>0){
			wpgIdx=new HashMap<String,Map<String,Object>>();
			for(Map<String,Object> itm : wpg){
				itm.put("rptMonth",DateUtil.format((Date)itm.get("rptMonth"), "yyyy-MM-dd"));
				wpgIdx.put((String)itm.get("rptMonth"), itm);  
			}
		}
		String key=null;
		List<Map<String,Object>>  rst=new ArrayList<Map<String,Object>>(12);
		Map<String,Object> newItm=null;
		for(int i=1;i<=12;i++){
			key=rptYear+"-"+String.format("%02d",i)+"-01";
			newItm=MapUtils.build("rptMonth",key);
			if(wpgIdx!=null&&wpgIdx.get(key)!=null){
				newItm.putAll(wpgIdx.get(key));
			}
			rst.add(newItm);
		}
		
		return rst;
	}
	
	
	@RequestMapping("/save")
	public void saveWorkload(String jsonDatas,HttpServletResponse response){
		try{
			MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
			if(su==null){  
				JsonOutUtils.returnError(response, "缺少登录信息,请登录后再试");
				return;
			}
			
			List<Workload> workload=JSON.parseObject(jsonDatas,new TypeReference<List<Workload>>() {});
			service.saveWorkload(workload);
			JsonOutUtils.returnOk(response);
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
		
	}
	
	
}
