package com.hb.proj.input.service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.proj.car.service.CarRptUtils;
import com.hb.proj.model.CarConsume;
import com.hb.proj.model.OtherConsumeBO;
import com.hb.proj.model.OtherOilCalculater;
import com.hb.xframework.dao.core.SpringJdbcDAO;
import com.hb.xframework.dao.util.UUIDHexGenerator;
import com.hb.xframework.util.DateUtil;
import com.hb.xframework.util.MapUtils;
import com.hb.xframework.util.SessionThreadLocal;
import com.hb.xframework.util.SessionUser;

@Service
public class CarConsumeService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	
	/**
	 * 根据上报月份和车辆信息查询上报的运行数据
	 * @param rptMonth
	 * @param rptOrg
	 * @param carIds
	 * @return
	 */
	public List<Map<String,Object>>  loadConsume(String rptMonth,String rptOrg,Set<String> carIds){
		if(carIds==null||carIds.size()==0){
			return null;
		}
		String ids="'"+StringUtils.join(carIds,"','")+"'";
		String sql="select r.* from t_car_consume r where car_id in ("+ids+") and rpt_org=? and DATE_FORMAT(rpt_date,'%Y-%m')=? order by custom_num";
		return dao.queryForListMap(sql, rptOrg,rptMonth);
	}
	
	
	/**
	 * 保存车辆上报数据
	 * @param datas
	 * @param weightRatio 带权重的汽油折算系数
	 * @throws Exception
	 */
	public void saveConsume(List<CarConsume> datas,Double oilWeightRatio) throws Exception{
		Set<String> existIds=new HashSet<String>();  //过滤出存在库里的运行记录，先进行删除再统一入库
		UUIDHexGenerator  uuid=UUIDHexGenerator.getInstance();
		final SessionUser su=SessionThreadLocal.getSessionUser();
		for(CarConsume car : datas){
			if(car.getRecordId()!=null){
				existIds.add(car.getRecordId());
			}
			car.setRecordId(uuid.generate()) ; //作为新记录，重新设置主键
			
			//入库前进行基本统计数据的计算
			CarRptUtils.rptCalculate(car,oilWeightRatio);
		}
		deleteConsume(existIds); //删除旧记录
		StringBuilder pnsb=new StringBuilder();
		pnsb.append("recordId,carId,travelMile,workHour,outHour,engineWorkHour,rptDate,modifyTime,modifier,rptOrg");
		pnsb.append(",oilErp,oilKl,oilMeter,oilKlMoney,coilErp,coilKl,coilMeter,coilKlMoney,coilZsh");
		pnsb.append(",checkMile,realCostPer100km,quotaPer100km,oilRealCost,coilRealCost,oilTotalQuota,coilTotalQuota,oilCoal,coilCoal,oilErpKg,oilKlKg,coilErpKg,coilKlKg,oilMeterKg,coilMeterKg");
		pnsb.append(",workCost,outCost,engineCost,engineCostKg,customNum,workArea");
		String[] pnary=pnsb.toString().split(",");
		Map<String,Object> extraData=MapUtils.build("rptOrg",su.getOrgId(),"modifier",(su!=null?su.getUserName():"unknow"),"modifyTime",(new Date()));
		dao.batchBeanInsert(datas, "t_car_consume", Arrays.asList(pnary),extraData);
	}
	
	
	
	private void deleteConsume(Set<String> ids){
		if(ids==null||ids.size()==0){
			return;
		}
		String str="'"+StringUtils.join(ids,"','")+"'";
		String sql="delete from t_car_consume where record_id in("+str+")";
		dao.getJdbcTemplate().update(sql);
	}
	

	public OtherConsumeBO  getOtherConsume(String rptMonth,String orgId){
		StringBuilder sql=new StringBuilder(100);
		sql.append("select * from t_other_consume where org_id=? and DATE_FORMAT(rpt_month,'%Y-%m')=?");
		return dao.queryForObject(sql.toString(),OtherConsumeBO.class, orgId,rptMonth);
	}
	
	public Map<String,Object>  loadOtherConsume(String startMonth,String endMonth,String orgAssistCode){
		StringBuilder sql=new StringBuilder(100);
		sql.append("select sum(oil_count_kg) oil_count_kg,sum(coil_count_kg) coil_count_kg from t_other_consume c");
		sql.append(" inner join t_org g on c.org_id=g.org_id");
		sql.append(" where g.assist_code like ? and DATE_FORMAT(rpt_month,'%Y-%m') between ? and ?");
		return dao.queryForSingleMap(sql.toString(), orgAssistCode+"%",startMonth,endMonth);
	}
	
	public void saveOtherConsume(OtherConsumeBO consume,boolean sameCover){
		if(sameCover){
			String sql="delete from t_other_consume where org_id=? and DATE_FORMAT(rpt_month,'%Y-%m')=?";
			dao.getJdbcTemplate().update(sql,consume.getOrgId(),DateUtil.format(consume.getRptMonth(), "yyyy-MM"));
		}
		
		OtherOilCalculater.calculate(consume);
		
		UUIDHexGenerator  uuid=UUIDHexGenerator.getInstance();
		consume.setRecordId(uuid.generate());
		consume.setModifyTime(new Date());
		dao.insertPojo(consume, "t_other_consume");
		
	}
	
	/**
	 * 导入油耗数据(包括车辆油耗和其它油耗)
	 * @param carCosumes
	 * @param othConsume
	 * @param sameCover
	 * @return
	 * @throws Exception
	 */
	public String addConsumesFromExcel(List<CarConsume> carCosumes,OtherConsumeBO othConsume,boolean sameCover) throws Exception{
		 Double w92=CarRptUtils.addDouble(othConsume.getOilKl92(), othConsume.getOilErp92());
		 Double w95=CarRptUtils.addDouble(othConsume.getOilKl95(), othConsume.getOilErp95());
		 Double oilWeightRatio=CarRptUtils.getOilWeightRatio(w92, w95);
		 String rst=addCarConsumesFromExcel(carCosumes,othConsume.getRptMonth(),othConsume.getOrgId(),sameCover,oilWeightRatio);
		 addOtherConsumeFromExcel(othConsume,sameCover);
		 return rst;
	}
	
	/**
	 * 导入其它油耗数据
	 * @param othConsume
	 * @param sameCover
	 * @return
	 */
	public boolean addOtherConsumeFromExcel(OtherConsumeBO othConsume,boolean sameCover){
		if(sameCover){ //允许覆盖，则先删除
			saveOtherConsume(othConsume,sameCover);
		}
		else{
			String sql="select record_id from t_other_consume where org_id=? and DATE_FORMAT(rpt_month,'%Y-%m')=?";
			Map<String,Object> rst=dao.queryForSingleMap(sql, othConsume.getOrgId(),DateUtil.format(othConsume.getRptMonth(), "yyyy-MM"));
			if(rst==null||rst.size()==0){ //不存在则入库
				saveOtherConsume(othConsume,sameCover);
			}
			
		}
		return true;
	}
	
	/**
	 * 导入车辆油耗数据
	 * @param importCosumes
	 * @param rptMonth
	 * @param orgId
	 * @param sameCover
	 * @param weightRatio 92# 95#带权重的汽油折算系数
	 * @return
	 */
	public String addCarConsumesFromExcel(List<CarConsume> importCosumes,Date rptMonth,String orgId,boolean sameCover,Double oilWeightRatio) throws Exception{
		String yyyyMM=DateUtil.format(rptMonth, "yyyy-MM");
		if(sameCover){ //允许覆盖(同车同部门同月份同地区)，则先删除  2021.11.5全覆盖以Excel里的为准，即同部门同月份全删除
			delSameConsumeForImport(yyyyMM,orgId);
			saveConsume(importCosumes,oilWeightRatio);
			return null;
		}
		else{
			
			String sameIgnor=excludeSameConsumeForImport(importCosumes,yyyyMM,orgId);
			
			if(importCosumes.size()>0){
				saveConsume(importCosumes,oilWeightRatio);
			}
			
			return sameIgnor;
		}
		
	}
	
	/**
	 * 导入前允许覆盖时，先删除旧的（同车同部门同月份同地区 才能唯一确定一条记录）
	 * 2021.11.5全覆盖以Excel里的为准，即同部门同月份全删除
	 * @param importCosumes 已取消按记录覆盖
	 * @param rptMonth  yyyy-MM
	 * @param rptOrg
	 */
	private void delSameConsumeForImport(String rptMonth,String rptOrg){
		/*if(importCosumes==null||importCosumes.size()==0){
			return;
		}
		List<Object[]> batchParams=new ArrayList<Object[]>(importCosumes.size());
		for(CarConsume  carcon : importCosumes){
			batchParams.add(new Object[]{carcon.getWorkArea(),rptOrg,rptMonth,carcon.getCarId()});
		}
		String sql="delete from t_car_consume where work_area=? and rpt_org=? and DATE_FORMAT(rpt_date,'%Y-%m')=? and  car_id =?  ";
		dao.getJdbcTemplate().batchUpdate(sql, batchParams);*/
		String sql="delete from t_car_consume where  rpt_org=? and DATE_FORMAT(rpt_date,'%Y-%m')=? ";
		dao.getJdbcTemplate().update(sql, rptOrg,rptMonth);
	}
	
	/**
	 * 导入前不允许覆盖时，先排除库中有的记录
	 * @param importCosumes
	 * @param rptMonth  yyyy-MM
	 * @param rptOrg
	 * @return 被忽略的车辆
	 */
	private String excludeSameConsumeForImport(List<CarConsume> importCosumes,String rptMonth,String rptOrg){
		if(importCosumes==null||importCosumes.size()==0){
			return null;
		}
		List<String> carIds=new ArrayList<String>(importCosumes.size());
		for(CarConsume  carcon : importCosumes){
			carIds.add(carcon.getCarId());
		}
		String cardstr="'"+StringUtils.join(carIds,"','")+"'";
		String sql="select concat(car_id,'_',work_area) id,1 val from t_car_consume where  rpt_org=? and DATE_FORMAT(rpt_date,'%Y-%m')=? and  car_id in ("+cardstr+")";
		Map<String,Object>  dbCons=dao.queryForMapping(sql,"id","val",rptOrg,rptMonth);
		if(dbCons==null||dbCons.size()==0){
			return null;
		}
		StringBuilder sameIgnor=new StringBuilder();
		Iterator<CarConsume> iterator=importCosumes.iterator();
		CarConsume carCon=null;
		while(iterator.hasNext()){
			carCon=iterator.next();
			if(dbCons.containsKey(carCon.getCarId()+"_"+carCon.getWorkArea())){
				sameIgnor.append(carCon.getCarNum()+";");
				iterator.remove();
			}
		}
		
		return sameIgnor.length()>0?sameIgnor.toString():null;
	}
	
}
