package com.hb.proj.input.service;

import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.proj.model.Workload;
import com.hb.xframework.dao.core.SpringJdbcDAO;
import com.hb.xframework.dao.util.UUIDHexGenerator;
import com.hb.xframework.util.MapUtils;
import com.hb.xframework.util.SessionThreadLocal;
import com.hb.xframework.util.SessionUser;

@Service
public class WorkloadService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	public List<Map<String,Object>>  queryWorkload(String rptYear){
		StringBuilder sql=new StringBuilder();
		sql.append("select w.* from t_workload_value w ");
		sql.append(" where DATE_FORMAT(rpt_month,'%Y')=? ");
		return dao.queryForListMap(sql.toString(), rptYear);
	}
	
	public void saveWorkload(List<Workload> datas) throws Exception{
		Set<String> dbIds=new HashSet<String>();  //过滤出存在库里的记录，先进行删除再统一入库
		UUIDHexGenerator  uuid=UUIDHexGenerator.getInstance();
		final SessionUser su=SessionThreadLocal.getSessionUser();
		for(Workload wd : datas){
			if(wd.getRecordId()!=null){
				dbIds.add(wd.getRecordId());
			}
			wd.setRecordId(uuid.generate()) ;  //作为新记录，重新设置主键
			wd.setRptOrg(su.getOrgId());
			
		}
		deleteWorkload(dbIds);
		
		StringBuilder pnsb=new StringBuilder();
		pnsb.append("recordId,rptMonth,rptOrg,wellCount,industryValue,addedValue,modifyTime,modifier");
		String[] pnary=pnsb.toString().split(",");
		Map<String,Object> extraData=MapUtils.build("modifier",(su!=null?su.getUserName():"unknow"),"modifyTime",(new Date()));
		dao.batchBeanInsert(datas, "t_workload_value", Arrays.asList(pnary),extraData);
	}
	
	
	/**
	 * 通过报表直接上报
	 * @param datas
	 * @throws Exception
	 */
	public void saveWorkloadFromRpt(List<Workload> datas,String year) throws Exception{
		//1月份存储有建筑面积删除前先取出
		String sql="select builder_area from t_workload_value where DATE_FORMAT(rpt_month,'%Y-%m')=?";
		Map<String,Object> oldData=dao.queryForSingleMap(sql, year+"-01");
		//先删除再保存
		sql="delete from t_workload_value where DATE_FORMAT(rpt_month,'%Y')=?";
		dao.getJdbcTemplate().update(sql,year);
		UUIDHexGenerator  uuid=UUIDHexGenerator.getInstance();
		final SessionUser su=SessionThreadLocal.getSessionUser();
		for(Workload wd : datas){
			wd.setRecordId(uuid.generate()) ;  //作为新记录，重新设置主键
		}
		StringBuilder pnsb=new StringBuilder();
		pnsb.append("recordId,rptMonth,rptOrg,wellCount,industryValue,addedValue,modifyTime,modifier");
		String[] pnary=pnsb.toString().split(",");
		Map<String,Object> extraData=MapUtils.build("modifier",(su!=null?su.getUserName():"unknow"),"modifyTime",(new Date()));
		dao.batchBeanInsert(datas, "t_workload_value", Arrays.asList(pnary),extraData);
		
		if(oldData!=null&&oldData.get("builderArea")!=null){ //存储有建筑面积的，再存回去，避免丢失
			sql="update t_workload_value set builder_area=? where DATE_FORMAT(rpt_month,'%Y-%m')=?";
			dao.getJdbcTemplate().update(sql, oldData.get("builderArea"),year+"-01");
		}
	}
	
	
	private void deleteWorkload(Set<String> ids){
		if(ids==null||ids.size()==0){
			return;
		}
		String str="'"+StringUtils.join(ids,"','")+"'";
		String sql="delete from t_workload_value where record_id in("+str+")";
		dao.getJdbcTemplate().update(sql);
	}
}
