package com.hb.proj.input.service;

import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.proj.model.Wpg;
import com.hb.proj.model.WpgCalculater;
import com.hb.xframework.dao.core.SpringJdbcDAO;
import com.hb.xframework.dao.util.UUIDHexGenerator;
import com.hb.xframework.util.DateUtil;
import com.hb.xframework.util.MapUtils;
import com.hb.xframework.util.SessionThreadLocal;
import com.hb.xframework.util.SessionUser;

@Service
public class WpgService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	public List<Map<String,Object>>  queryWpg(String rptYear,String orgId){
		StringBuilder sql=new StringBuilder();
		sql.append("select w.* from t_wpg_account w ");
		sql.append(" where DATE_FORMAT(rpt_month,'%Y')=? and org_id=? ");
		return dao.queryForListMap(sql.toString(), rptYear,orgId);
	}
	
	public void saveWpg(List<Wpg> datas) throws Exception{
		Set<String> dbIds=new HashSet<String>();  //过滤出存在库里的记录，先进行删除再统一入库
		UUIDHexGenerator  uuid=UUIDHexGenerator.getInstance();
		final SessionUser su=SessionThreadLocal.getSessionUser();
		for(Wpg wpg : datas){
			if(wpg.getRecordId()!=null){
				dbIds.add(wpg.getRecordId());
			}
			wpg.setRecordId(uuid.generate()) ;  //作为新记录，重新设置主键
			wpg.setOrgId(su.getOrgId());
			//入库前进行基本统计数据的计算,并清除旧的计算值
			WpgCalculater.calculate(wpg);
		}
		deleteWpgData(dbIds);
		
		StringBuilder pnsb=new StringBuilder();
		pnsb.append("recordId,rptMonth,orgId,waterCount,waterCountQuota,waterCost,powerCount,powerCountQuota,powerCost,modifyTime,modifier");
		pnsb.append(",ngasCount,ngasCountQuota,ngasCost,lgasCount,lgasCountQuota,lgasCost,powerCoal,lgasCoal,ngasCoal");
		String[] pnary=pnsb.toString().split(",");
		Map<String,Object> extraData=MapUtils.build("modifier",(su!=null?su.getUserName():"unknow"),"modifyTime",(new Date()));
		dao.batchBeanInsert(datas, "t_wpg_account", Arrays.asList(pnary),extraData);
	}
	
	public void  saveWpg2(List<Map<String,Object>> datas){
		Set<String> dbIds=new HashSet<String>();  //过滤出存在库里的记录，先进行删除再统一入库
		UUIDHexGenerator  uuid=UUIDHexGenerator.getInstance();
		SessionUser su=SessionThreadLocal.getSessionUser();
		for(Map<String,Object> d : datas){
			if(d.get("recordId")!=null){
				dbIds.add((String)d.get("recordId"));
			}
			MapUtils.blankValToNull(d);
			d.put("rptMonth", DateUtil.parse(d.get("rptMonth")+"-01")) ;  //月份字符串转成日期对象,同时转换字段
			d.put("orgId", su.getOrgId());
			d.put("modifyTime",new Date());
			d.put("createTime",new Date());
			d.put("recordId", uuid.generate()); //作为新记录，重新设置主键
			d.put("modifier", su!=null?su.getUserName():"unknow");
			d.put("creator", d.get("modifier"));
			
		}
		
		deleteWpgData(dbIds);
		List<String> pns=Arrays.asList("recordId","rptMonth","orgId","waterCount","waterCost","powerCount","powerCost","ngasCount","ngasCost","lgasCount","lgasCost","createTime","modifyTime","creator","modifier","waterCountQuota","powerCountQuota","ngasCountQuota","lgasCountQuota");
		dao.batchInsert(datas,"t_wpg_account", pns);
	}
	
	
	private void deleteWpgData(Set<String> ids){
		if(ids==null||ids.size()==0){
			return;
		}
		String str="'"+StringUtils.join(ids,"','")+"'";
		String sql="delete from t_wpg_account where record_id in("+str+")";
		dao.getJdbcTemplate().update(sql);
	}
}
