package com.hb.proj.model;

import com.hb.proj.car.service.CarRptUtils;
import com.hb.proj.utils.DataUtils;

/**
 * 用能用水累计同比有两个报表。2:能、水和产值等数据的比例的同比；
 * @author hb
 *
 */
public class EnergyWaterRatioCompareVO {

	private ERP  erpkl;
	
	private Wpg   wpg;
	
	private Workload workload;
	
	private CarConsumeMulRptVO carRptConsume;
	
	private Double oilTotal;  //汽油总量 吨
	
	private Double coilTotal;  //柴油总量 吨
	
	private Double oilCoilTotal;
	
	private Double coals; //折算吨煤
	
	private Double checkMile; //考核里程 累计值
	
	private Double coalKgIndustryValue; // 千克标煤/万元产值
	
	private Double coalKgAddedValue;  //千克标煤/万元增加值
	
	private Double waterIndustryValue ; //立方米 /万元产值
	
	private Double coalKgWellCount ;  //千克标煤/井次
	
	private Double powerBuilderArea; // 千瓦时/平方米 建筑面积
	
	private Double oilCost100km ; //百公里耗油=(汽油+柴油)/考核公里  车辆综合油耗
	
	
	
	public EnergyWaterRatioCompareVO (ERP erpkl, Wpg  wpg, Workload workload,CarConsumeMulRptVO carConsume){
		this.erpkl=erpkl;
		this.wpg=wpg;
		this.workload=workload;
		this.carRptConsume=carConsume;
		this.checkMile=carConsume!=null&&carConsume.getCheckMile()!=null?carConsume.getCheckMile():0;
		this.oilTotal=erpkl.getOilCount();  //借用字段存有erp+昆仑卡值
		this.coilTotal=erpkl.getCoilCount();
		
		init();
	}
	
	
	/**
	 * 计算综合吨标煤,报表中使用的是千克需要转换,计算指标值
	 */
	public void init(){
		Double d=null;
		if(this.erpkl!=null){
			d=CarRptUtils.addDouble(this.erpkl.getOilCoal(),this.erpkl.getCoilCoal());  //如果是和数据校正中的当月对应，就用oilCoal,用累计就用oilSumCoal
		}
		if(this.wpg!=null){
			d=CarRptUtils.addDouble(d,this.wpg.getNgasCoal());
			d=CarRptUtils.addDouble(d,this.wpg.getLgasCoal());
			d=CarRptUtils.addDouble(d,this.wpg.getPowerCoal());
		}
		
		this.coals=DataUtils.round(d, 3);
		this.coals=this.coals!=null?this.coals*1000:null;  //报表中使用的是千克所以*1000
		this.oilCoilTotal=CarRptUtils.addDouble(this.oilTotal,this.coilTotal);
		if(this.oilCoilTotal!=null){
			this.oilCoilTotal=this.oilCoilTotal*1000; //报表中使用的是千克所以*1000
		}
		if(this.getWpg()!=null&&this.getWpg().getWaterCount()!=null){
			this.getWpg().setWaterCount(this.getWpg().getWaterCount()*10000); //报表中使用的是立方米所以*10000
		}
		if(this.getWpg()!=null&&this.getWpg().getPowerCount()!=null){
			this.getWpg().setPowerCount(this.getWpg().getPowerCount()*10000); //报表中使用的是千瓦时所以*10000
		}
		
		if(this.checkMile!=null){
			this.checkMile=this.checkMile/100;
		}
		//指标值计算
		
		Double wellCount=getDouble(this.workload.getWellCount());
		
		this.coalKgIndustryValue=this.workload!=null?divideDouble(this.coals,this.workload.getIndustryValue()):null;
		this.coalKgAddedValue=this.workload!=null?divideDouble(this.coals,this.workload.getAddedValue()):null;
		
		if(this.getWpg()!=null&&this.getWpg().getWaterCount()!=null&&this.workload!=null){
			this.waterIndustryValue=divideDouble(this.getWpg().getWaterCount(),this.workload.getIndustryValue());
		}
		this.coalKgWellCount=this.workload!=null?divideDouble(this.coals,wellCount):null;
		if(this.getWpg()!=null&&this.getWpg().getPowerCount()!=null&&this.getWorkload()!=null){
			this.powerBuilderArea=divideDouble(this.getWpg().getPowerCount(),this.getWorkload().getBuilderArea());
		}
		
		if(this.oilCoilTotal!=null&&this.checkMile!=null){
			this.oilCost100km=divideDouble(this.oilCoilTotal,this.checkMile);
		}
		
		//2022.7.16 增加额外指标值： 柴油车行驶油耗(升/百公里)、柴油车井次油耗(升/井次)、汽油车行驶油耗(升/百公里)、汽油车井次油耗(升/井次)、发电机井次油耗(升/井次)
		this.carRptConsume.calculate(this.workload.getWellCount());
		
	}
	
	private Double getDouble(Integer val){
		return val==null?null:(val.intValue()+0.0);
	}
	
	
	private Double divideDouble(Double d1,Double d2){
		if(d1==null||d2==null||d2.doubleValue()==0){
			return null;
		}
		return d1.doubleValue()/d2.doubleValue();
	}


	public ERP getErpkl() {
		return erpkl;
	}


	public void setErpkl(ERP erpkl) {
		this.erpkl = erpkl;
	}


	public Wpg getWpg() {
		return wpg;
	}


	public void setWpg(Wpg wpg) {
		this.wpg = wpg;
	}


	public Workload getWorkload() {
		return workload;
	}


	public void setWorkload(Workload workload) {
		this.workload = workload;
	}


	public Double getOilTotal() {
		return oilTotal;
	}


	public void setOilTotal(Double oilTotal) {
		this.oilTotal = oilTotal;
	}


	public Double getCoilTotal() {
		return coilTotal;
	}


	public void setCoilTotal(Double coilTotal) {
		this.coilTotal = coilTotal;
	}


	public Double getCoals() {
		return coals;
	}


	public void setCoals(Double coals) {
		this.coals = coals;
	}


	public Double getCoalKgIndustryValue() {
		return coalKgIndustryValue;
	}


	public void setCoalKgIndustryValue(Double coalKgIndustryValue) {
		this.coalKgIndustryValue = coalKgIndustryValue;
	}


	public Double getCoalKgAddedValue() {
		return coalKgAddedValue;
	}


	public void setCoalKgAddedValue(Double coalKgAddedValue) {
		this.coalKgAddedValue = coalKgAddedValue;
	}


	public Double getWaterIndustryValue() {
		return waterIndustryValue;
	}


	public void setWaterIndustryValue(Double waterIndustryValue) {
		this.waterIndustryValue = waterIndustryValue;
	}


	public Double getCoalKgWellCount() {
		return coalKgWellCount;
	}


	public void setCoalKgWellCount(Double coalKgWellCount) {
		this.coalKgWellCount = coalKgWellCount;
	}


	public Double getPowerBuilderArea() {
		return powerBuilderArea;
	}


	public void setPowerBuilderArea(Double powerBuilderArea) {
		this.powerBuilderArea = powerBuilderArea;
	}


	public Double getCheckMile() {
		return checkMile;
	}


	public void setCheckMile(Double checkMile) {
		this.checkMile = checkMile;
	}


	public Double getOilCoilTotal() {
		return oilCoilTotal;
	}


	public void setOilCoilTotal(Double oilCoilTotal) {
		this.oilCoilTotal = oilCoilTotal;
	}


	


	public CarConsumeMulRptVO getCarRptConsume() {
		return carRptConsume;
	}


	public void setCarRptConsume(CarConsumeMulRptVO carRptConsume) {
		this.carRptConsume = carRptConsume;
	}


	public Double getOilCost100km() {
		return oilCost100km;
	}


	public void setOilCost100km(Double oilCost100km) {
		this.oilCost100km = oilCost100km;
	}
}
