package com.hb.proj.model;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.hb.proj.car.controller.ERPKlSumUtil;
import com.hb.proj.car.service.CarRptUtils;
import com.hb.xframework.util.DateUtil;

public class EnergyWaterSumVOUtil {
	

	/**
	 * 计算同比值
	 * @param voIndex
	 * @param preVoIndex
	 */
	public static void caculateCompare(Map<String,EnergyWaterSumVO>  voIndex,Map<String,EnergyWaterSumVO> preVoIndex){
		String month=null;
		EnergyWaterSumVO v1=null;
		for(int i=1;i<=12;i++){
			month=String.format("%02d",i);
			v1=voIndex.get(month);
			if(preVoIndex.get(month)!=null){
				if(v1==null){
					v1=new EnergyWaterSumVO(null,null,null,null);
					voIndex.put(month, v1);
				}
				v1.setComparedValue(preVoIndex.get(month));
			}
		}
	}
	
	/**
	 * 油量各月数据和数据校正中表二中的当月数据对应(单位：万元、吨、万立方米)
	 * @return
	 */
	public static Map<String,ERP>  builderERPKlMonthGroup(List<ERP> erps,List<OtherConsume> kls){
		Map<String,OtherConsume> klIdx=new HashMap<String,OtherConsume>();
		Map<String,ERP> erpIdx=new HashMap<String,ERP>();
		String month=null;
		
		for(OtherConsume consume : kls){
			month=DateUtil.format(consume.getRptMonth(), "MM");
			klIdx.put(month, consume);
		}
		for(ERP erp : erps){
			month=DateUtil.format(erp.getRptMonth(), "MM");
			erpIdx.put(month, erp);
		}
		for(int m=1;m<=12;m++){
			month=String.format("%02d", m);
			erpIdx.put(month, ERPKlSumUtil.erpPlusKlMonth(klIdx.get(month), erpIdx.get(month))); //借用，原始ERP替换为ERP+kl
		}
		klIdx.clear();
		klIdx=null;
		return erpIdx;
	}
	
	/**
	 * 水电气各月数据和数据校正中表三中的当月和计数据对应
	 * @param wpgs
	 * @param wpgCorrects
	 * @return
	 */
	public static Map<String,Wpg> buildWpgWithCorrect(List<Wpg> wpgs,List<WpgCorrect> wpgCorrects){
		Map<String,Wpg> wpgIdx=new HashMap<String,Wpg>();
		Map<String,WpgCorrect> correctIdx=new HashMap<String,WpgCorrect>();
		String month=null;
		
		for(Wpg wpg : wpgs){
			month=DateUtil.format(wpg.getRptMonth(), "MM");
			wpgIdx.put(month, wpg);
		}
		for(WpgCorrect crt : wpgCorrects){
			month=DateUtil.format(crt.getRptMonth(), "MM");
			correctIdx.put(month, crt);
		}
		Wpg wpg=null;
		for(int m=1;m<=12;m++){
			month=String.format("%02d", m);
			wpg=wpgIdx.get(month); 
			if(wpg==null){//可能为空
				wpg=new Wpg();
				wpgIdx.put(month, wpg);
			}
			ERPKlSumUtil.processWpgSum(wpg, correctIdx.get(month));
		}
		correctIdx.clear();
		correctIdx=null;
		return wpgIdx;
	}
	
	
	
	
	/**
	 * 综合分析报表中的用能用水分月统计  增加了新的统计内容2022.7.16,先调用build
	 * {@link EnergyWaterSumVOUtil.build}
	 * @param sumMap
	 * @param consumes
	 * @param remainOils  通过监控程序获得的期初期末剩余油量
	 */
	
	public static void attachForMulRpt(Map<String, EnergyWaterSumVO> sumMap,List<CarConsumeMulRptVO> consumes,List<MonitorRemainOil> remainOils){
		if(consumes==null||consumes.size()==0){
			return ;
		}
		Map<String,MonitorRemainOil>  adjustMap=null;
		if(remainOils!=null&&remainOils.size()>0){
			adjustMap=new HashMap<String,MonitorRemainOil>();
			for(MonitorRemainOil itm : remainOils){
				adjustMap.put(itm.getRptMonth().substring(5, 7), itm);
			}
		}
		String month=null;
		EnergyWaterSumVO  tmpSum=null;
		Workload wkload=null;
		int wellCount=0;
		for(CarConsumeMulRptVO consume : consumes){
			month=consume.getRptMonth().substring(5,7);
			tmpSum=sumMap.get(month);
			wkload=tmpSum.getWorkload();
			wellCount=wkload!=null&&wkload.getWellCount()!=null?wkload.getWellCount().intValue():0;
			if(adjustMap!=null&&adjustMap.get(month)!=null){
				consume.adjust(adjustMap.get(month));
			}
			consume.calculate(wellCount);
			
			tmpSum.setIndConsume(consume);
		}
	}
	
	/**
	 * 用能用水统计汇总(分月显示12个月)、综合分析报表中的用能用水分月统计
	 * @param erps
	 * @param kls
	 * @param wpgs
	 * @param wpgCorrects
	 * @param workloads
	 * @return
	 */
	public static Map<String, EnergyWaterSumVO> build(List<ERP> erps,
			List<OtherConsume> kls, List<Wpg> wpgs,
			List<WpgCorrect> wpgCorrects, List<Workload> workloads,
			List<CarConsume> carConsumes, EnergyWaterSumTotalVal totalVal) {
		Map<String,EnergyWaterSumVO>  voIdx=new HashMap<String,EnergyWaterSumVO>();
		String month=null;
		EnergyWaterSumVO vo=null;
		
		Map<String,ERP>   monthErps=builderERPKlMonthGroup(erps,kls); //汽油、柴油
		Map<String,Wpg>   monthWpgs=buildWpgWithCorrect(wpgs,wpgCorrects) ; //水电气
		Map<String,Workload> monthWorkloads=new HashMap<String,Workload>(workloads.size()); //工作量
		
		double total1=0,total2=0;
		int total3=0,endMonth=totalVal!=null?totalVal.getEndMonth():0;
		
		for(Workload wk : workloads){
			month=DateUtil.format(wk.getRptMonth(), "MM");
			monthWorkloads.put(month, wk);
			
			if(endMonth>=Integer.parseInt(month)){  //只累计截至月份以内的
				total3+=wk.getWellCount()!=null?wk.getWellCount().intValue():0;
				total1+=wk.getIndustryValue()!=null?wk.getIndustryValue().doubleValue():0;
				total2+=wk.getAddedValue()!=null?wk.getAddedValue().doubleValue():0;
			}
			
		}
		
		if(totalVal!=null){
			totalVal.setWellCount(total3);
			totalVal.setIndustryValue(total1);
			totalVal.setAddedValue(total2);
		}
		
		
		Map<String,CarConsume>  monthCarConsume=null;
		if(carConsumes!=null){
			total1=0;
			monthCarConsume=new HashMap<String,CarConsume>(carConsumes.size()); //车辆明细消耗统计
			for(CarConsume carCon : carConsumes){
				month=DateUtil.format(carCon.getRptDate(), "MM");
				monthCarConsume.put(month, carCon);
				if(endMonth>=Integer.parseInt(month)){  //只累计截至月份以内的
					total1+=carCon.getEngineCostKg()!=null?carCon.getEngineCostKg().doubleValue():0;
				}
				
			}
			
			if(totalVal!=null){
				totalVal.setEngineCost(total1);
			}
			
		}
		
		
		for(int m=1;m<=12;m++){
			month=String.format("%02d", m);
			vo=new EnergyWaterSumVO(monthErps.get(month),monthWpgs.get(month),monthWorkloads.get(month),monthCarConsume!=null?monthCarConsume.get(month):null);
			vo.initCoal();
			voIdx.put(month, vo);
		}
		
		return voIdx;
	}
	
	
	/**
	 * 用于能源台账报表：EnergyCostAccountController
	 * 来源数据单位：吨，万元，万立方米，万千瓦时。部分单位需要转换
	 * 汽油、柴油千克、液化气千克
	 * 汽油、柴油折算吨标煤显示合计值，合计后存储在oilCoal
	 * @param erps
	 * @param kls
	 * @param wpgs
	 * @param wpgCorrects
	 * @return
	 */
	public static Map<String,EnergyWaterSumVO> buildForCostAccount(List<ERP> erps,List<OtherConsume> kls,List<Wpg> wpgs,List<WpgCorrect> wpgCorrects){
		Map<String,EnergyWaterSumVO>  voIdx=new HashMap<String,EnergyWaterSumVO>();
		String month=null;
		EnergyWaterSumVO vo=null;
		
		Map<String,ERP>   monthErps=builderERPKlMonthGroup(erps,kls); //汽油、柴油
		Map<String,Wpg>   monthWpgs=buildWpgWithCorrect(wpgs,wpgCorrects) ; //水电气
		
		for(int m=1;m<=12;m++){
			month=String.format("%02d", m);
			vo=new EnergyWaterSumVO(monthErps.get(month),monthWpgs.get(month),null,null);
			vo.initCoal();
			if(vo.getErpKl()!=null){
				vo.getErpKl().setOilCoal(CarRptUtils.addDouble(vo.getErpKl().getOilCoal(),vo.getErpKl().getCoilCoal()));
			}
			vo.setOilTotal(vo.getOilTotal()!=null?(vo.getOilTotal()*1000):null);
			vo.setCoilTotal(vo.getCoilTotal()!=null?(vo.getCoilTotal()*1000):null);
			if(vo.getWpg()!=null&&vo.getWpg().getLgasCount()!=null){
				vo.getWpg().setLgasCount(vo.getWpg().getLgasCount()*1000);
			}
			
			voIdx.put(month, vo);
		}
		
		return voIdx;
	}
}
