package com.hb.proj.model;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.hb.proj.car.quota.QuotaConfig;
import com.hb.proj.car.quota.QuotaStandardSupport;

/**
 * 其它用油换算工具
 * @author hb
 *
 */
public class OtherOilCalculater {

	private static Logger logger=LoggerFactory.getLogger(OtherOilCalculater.class);
	
	//获得带权重(92#:95#)的汽油折算系数---加权值
	public static double getOilWeightRatio(Double oil92,Double oil95){
		if(oil92==null){
			oil92=0.0;
		}
		if(oil95==null){
			oil95=0.0;
		}
		if(oil92==0&&oil95==0){
			return 1;
		}
		double w92=oil92/(oil92+oil95),w95=oil95/(oil92+oil95);  //计算权重
		QuotaConfig qc=QuotaStandardSupport.getConfig();
		return qc.getOilRatio()*w92+qc.getOilRatio95()*w95;
	}		
		
	/**
	 * 油耗 升-转kg
	 * @param othOil
	 */
	public static void calculate(OtherConsumeBO othOil){
		QuotaConfig qc=QuotaStandardSupport.getConfig();
		if(qc==null){
			logger.warn("缺少折算系数，取消对其它油耗的换算");
			return;
		}
		 Double w92=addDouble(othOil.getOilKl92(), othOil.getOilErp92());
		 Double w95=addDouble(othOil.getOilKl95(), othOil.getOilErp95());
		 Double oilWeightRatio=getOilWeightRatio(w92, w95);
		 
		//先清空计算字段数据
		clearOther(othOil);
		//其它用油
		if(othOil.getOilCount()!=null){
			othOil.setOilCountKg(othOil.getOilCount()*oilWeightRatio);
		}
		if(othOil.getCoilCount()!=null){
			othOil.setCoilCountKg(othOil.getCoilCount()*qc.getCoilRatio());
		}
		//汽油92#，95# 升进行合并，重量分别折算后再合并
		
		othOil.setOilKlCount(addDouble(othOil.getOilKl92(),othOil.getOilKl95()));
		othOil.setOilErpCount(addDouble(othOil.getOilErp92(),othOil.getOilErp95()));
		double oilWeight=0;
		if(othOil.getOilKl92()!=null){
			oilWeight+=othOil.getOilKl92()*qc.getOilRatio();
		}
		if(othOil.getOilKl95()!=null){
			oilWeight+=othOil.getOilKl95()*qc.getOilRatio95();
		}
		othOil.setOilKlCountKg(oilWeight);
		//oilWeight要重置否则产生累计效果
		oilWeight=0;
		if(othOil.getOilErp92()!=null){
			oilWeight+=othOil.getOilErp92()*qc.getOilRatio();
		}
		if(othOil.getOilErp95()!=null){
			oilWeight+=othOil.getOilErp95()*qc.getOilRatio95();
		}
		othOil.setOilErpCountKg(oilWeight);
		
		
		
		//项目部合计用油
		
		
		if(othOil.getCoilErpCount()!=null){
			othOil.setCoilErpCountKg(othOil.getCoilErpCount()*qc.getCoilRatio());
		}
		if(othOil.getCoilKlCount()!=null){ 
			othOil.setCoilKlCountKg(othOil.getCoilKlCount()*qc.getCoilRatio());
		}
		
		//计算汽油、柴油合计折算煤(昆仑卡+ERP)
		Double oilKg=addDouble(othOil.getOilKlCountKg(),othOil.getOilErpCountKg());
		if(oilKg!=null){ //kg转吨再折算
			othOil.setOilCoal(oilKg*qc.getOilCoalRatio()/1000.0);
		}
		
		oilKg=addDouble(othOil.getCoilKlCountKg(),othOil.getCoilErpCountKg());
		
		if(oilKg!=null){
			othOil.setCoilCoal(oilKg*qc.getCoilCoalRatio()/1000.0);
		}
		
		//单独计算昆仑卡油耗折算煤
		if(othOil.getOilKlCountKg()!=null){  //kg转吨再折算
			othOil.setOilKlCoal(othOil.getOilKlCountKg()*qc.getOilCoalRatio()/1000.0);
		}
		if(othOil.getCoilKlCountKg()!=null){
			othOil.setCoilKlCoal(othOil.getCoilKlCountKg()*qc.getCoilCoalRatio()/1000.0);
		}
		//单独计算其它油耗折算煤
		if(othOil.getOilCountKg()!=null){  //kg转吨再折算
			othOil.setOilOthCoal(othOil.getOilCountKg()*qc.getOilCoalRatio()/1000.0);
		}
		if(othOil.getCoilCountKg()!=null){
			othOil.setCoilOthCoal(othOil.getCoilCountKg()*qc.getCoilCoalRatio()/1000.0);
		}
	}
	
	/**
	 * 上报数据有些本身来源于库中，特别是需要计算的数据，再次入库时应该重新计算，就需要把这些计算数据先置空
	 * @param othOil
	 */
	public static void clearOther(OtherConsumeBO othOil){
		//其它用油由升转公斤
		othOil.setOilCountKg(null);
		othOil.setCoilCountKg(null);
		othOil.setOilKlCountKg(null);
		othOil.setOilErpCountKg(null);
		othOil.setCoilErpCountKg(null);
		othOil.setCoilKlCountKg(null);
		othOil.setOilCoal(null);
		othOil.setCoilCoal(null);
		othOil.setOilKlCoal(null);
		othOil.setCoilKlCoal(null);
		othOil.setOilOthCoal(null);
		othOil.setCoilOthCoal(null);
	}
	
	
	public static Double addDouble(Double d1,Double d2){
		if(d2==null||Double.isNaN(d2)){
			return d1!=null?d1.doubleValue():null;
		}
		if(d1==null||Double.isNaN(d1)){
			return d2!=null?d2.doubleValue():null;
		}
		else{
			return d1.doubleValue()+d2.doubleValue();
		}
	}
}
