package com.hb.proj.sysdata.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.car.quota.QuotaType;
import com.hb.proj.sysdata.service.QuotaStandardService;
import com.hb.proj.sysdata.service.SortCodeService;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.xframework.util.JsonParamHandler;
import com.hb.xframework.util.MapUtils;


@Controller
@RequestMapping("/**/opts")
public class OptionsController {

	@Autowired
	private SortCodeService service;
	
	@Autowired
	private QuotaStandardService  quotaService;
	
	//config: [{selectId:'',valkey:'',txtkey:'',pn:'',pv:'',logic:''}]
	@RequestMapping("/load")
	public void load(String jsonConfig,HttpServletResponse response){
		if(StringUtils.isEmpty(jsonConfig)){
			JsonOutUtils.returnError(response, "缺少参数");
			return;
		}
		List<Map<String,Object>> qparams=JsonParamHandler.getListMapParams(jsonConfig);
		Map<String,Object> rstCodes=new HashMap<String,Object>(qparams.size());
		List<Map<String,Object>> optionsData=null;
		for(Map<String,Object> itm : qparams){
			if("quotaType".equals(itm.get("selectId"))){  //定额类型特殊处理
				optionsData=getQuotaTypes();
			}
			else if("quotaAlternatorId".equals(itm.get("selectId"))){  //发电机型号
				optionsData=getAllAlternators();
			}
			else{
				optionsData=service.loadCodes((String)itm.get("pn"), (String)itm.get("pv"),  (String)itm.get("logic"),  (String)itm.get("valkey"),  (String)itm.get("txtkey"));
			}
			rstCodes.put((String)itm.get("selectId"), optionsData);
			
		}
		JsonOutUtils.returnOkWithData(response, rstCodes);
	}
	
	private List<Map<String,Object>>  getQuotaTypes(){
		List<Map<String,Object>>  rst=new ArrayList<Map<String,Object>>();
		rst.add(MapUtils.build("val",QuotaType.TRAVEL.getName(),"txt",QuotaType.TRAVEL.getName()));
		rst.add(MapUtils.build("val",QuotaType.WORK.getName(),"txt",QuotaType.WORK.getName()));
		rst.add(MapUtils.build("val",QuotaType.ENGINE.getName(),"txt",QuotaType.ENGINE.getName()));
		rst.add(MapUtils.build("val",QuotaType.WARM.getName(),"txt",QuotaType.WARM.getName()));
		return rst;
	}
	
	//
	private List<Map<String,Object>> getAllAlternators(){
		List<Map<String,Object>>  records=quotaService.loadOtherStandard();
		if(records==null||records.size()==0){
			return null;
		}
		List<Map<String,Object>>  rst=new ArrayList<Map<String,Object>>();
		for(Map<String,Object> itm : records){
			rst.add(MapUtils.build("val",itm.get("recordId"),"txt",itm.get("deviceModel")));
		}
		return rst;
	}
}
