package com.hb.proj.sysdata.controller;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.sysdata.service.OrgService;
import com.hb.proj.utils.DataAuthUtils;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.MySessionUser;
import com.hb.xframework.util.JsonParamHandler;
import com.hb.xframework.util.MapUtils;
import com.hb.xframework.util.SessionThreadLocal;
import com.hb.xframework.web.RequestUtils;

@Controller
@RequestMapping("/**/org")
public class OrgController {

	@Autowired
	private OrgService service;
	
	@RequestMapping("/loadAuthNode")
	public void loadAuthNode(String superId,String targetDepth,String noauth, HttpServletResponse response){
		MySessionUser su=(MySessionUser)SessionThreadLocal.getSessionUser();
		if(su==null){
			return ;
		}
		List<Map<String, Object>> nodes=null;
		if(StringUtils.isEmpty(superId)){  //管理员查看全部或者明确不需要权限控制
			if(su.isSysAdmin()||"1".equals(noauth)){
				nodes=service.queryOrgChildNodes("0");
			}
			else{
				nodes=service.queryOrgSelfNodes(DataAuthUtils.getUserAuthOrg(su));
			}
		}
		else{
			nodes=service.queryOrgChildNodes(superId);
		}
		//nodes=service.queryOrgChildNodes(StringUtils.isEmpty(superId)?"0":superId);  //测试用
		JsonOutUtils.preprocessTreeNodes(nodes, targetDepth,null);
		JsonOutUtils.writeToResponse(response, JsonParamHandler.getJson(nodes));
	}
	
	@RequestMapping("/add")
	public void add(HttpServletRequest request,HttpServletResponse response) {
		Map<String, Object> args = RequestUtils.getParams(request);
		if(StringUtils.isEmpty((String)args.get("orgName"))){
			JsonOutUtils.returnError(response, "缺少单位名称");
			return ;
		}
		args.remove("oldSuperId");
		if(StringUtils.isEmpty((String)args.get("superId"))){
			args.put("superId","0");
		}
		MapUtils.blankValToNull(args);
		String newAssistCode = service.generateLeveledCode((String)args.get("superId"));
		if(newAssistCode==null){
			JsonOutUtils.returnError(response, "指定的上级组织不存在");
			return ;
		}
		args.put("assistCode",newAssistCode);
		String newOrgId=service.addOrg(args);
		JsonOutUtils.returnOkWithData(response,newOrgId);
	}
	
	@RequestMapping("/update")
	public void update(HttpServletRequest request,HttpServletResponse response) {
		Map<String, Object> args = RequestUtils.getParams(request);
		if(StringUtils.isEmpty((String)args.get("orgName"))){
			JsonOutUtils.returnError(response, "缺少单位名称");
			return ;
		}
		MapUtils.blankValToNull(args);
		String oldSuperId=request.getParameter("oldSuperId");
		args.remove("oldSuperId");
		if (!oldSuperId.equals(args.get("superId"))) { // 隶属关系变动后，修正assistCode
			String newAssistCode =service.generateLeveledCode((String)args.get("superId"));
			if(newAssistCode==null){
				JsonOutUtils.returnError(response, "指定的上级组织不存在");
				return ;
			}
			service.updateOrg(args,newAssistCode);
		}
		else{
			service.updateOrg(args,null);
		}
		
		JsonOutUtils.returnOk(response);
	}
	
	@RequestMapping("/get")
	public void get(String orgId, HttpServletResponse response) {
		Map<String, Object> org = service.getOrgById(orgId);
		JsonOutUtils.returnOkWithData(response, org);
	}

	@RequestMapping("/delete")
	public void delete(String orgId,String delModel,HttpServletResponse response) {
		if("sub".equalsIgnoreCase(delModel)){
			service.deleteSubOrg(orgId);
		}
		else{
			service.deleteOrg(orgId);
		}
		JsonOutUtils.returnOk(response);
	}
	
	/**
	 * 自动排列下级节点的顺序
	 * 
	 * @param superId
	 * @param response
	 */
	@RequestMapping("/sort")
	public void sort(String superId, HttpServletResponse response) {
		try {
			service.autoSortChildOrg(superId);
			JsonOutUtils.returnOk(response);
		} catch (Exception e) {
			e.printStackTrace();
			JsonOutUtils.returnError(response, "自动排序服务出现错误");
		}

	}
}
