package com.hb.proj.sysdata.controller;

import java.io.File;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import com.hb.proj.car.quota.QuotaStandardSupport;
import com.hb.proj.car.quota.QuotaType;
import com.hb.proj.excel.ExcelUtils;
import com.hb.proj.sysdata.service.QuotaStandardService;
import com.hb.proj.sysdata.service.SortCodeService;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.LayGridJsonUtils;
import com.hb.proj.utils.RequestUtils;
import com.hb.xframework.dao.util.FieldAttrConverter;
import com.hb.xframework.dao.util.PageModel;
import com.hb.xframework.dao.util.UUIDHexGenerator;
import com.hb.xframework.util.JsonParamHandler;
import com.hb.xframework.util.MapUtils;
import com.hb.xframework.util.SessionThreadLocal;
import com.hb.xframework.util.SessionUser;
/**
 * 根据需求修改，此特指百公里油耗标准管理，其他定额标准另行处理
 */
@Controller
@RequestMapping("/**/standard")
public class QuotaStandardController {

	@Autowired
	private QuotaStandardService service;
	
	@Autowired
	private SortCodeService  codeService;
	
	/**
	 * 目前用于定额表格中作业区编码转换为名称
	 * @param fatherId
	 * @param keyField
	 * @param valField
	 * @param response
	 */
	@RequestMapping("/loadCodesForGrid")
	public void loadCodesForGrid(String fatherId,String keyField,String valField,HttpServletResponse response){
		try{
			if(StringUtils.isEmpty(fatherId)||StringUtils.isEmpty(keyField)||StringUtils.isEmpty(valField)){
				JsonOutUtils.returnError(response, "缺少必要参数");
				return;
			}
			keyField=FieldAttrConverter.getFieldName(keyField);
			valField=FieldAttrConverter.getFieldName(valField);
			JsonOutUtils.returnOkWithData(response, codeService.loadSortCodeMapping(fatherId,keyField,valField));
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.returnError(response, "服务出现错误");
		}
	}
	
	
	@RequestMapping("/query")
	public void query(HttpServletRequest request,Integer page,Integer limit,HttpServletResponse response){
		try{
			Map<String,Object> args=RequestUtils.getParams(request);
			args.put("quotaType",QuotaType.TRAVEL.getName()); //限定为查询百公里定额
			PageModel<Map<String,Object>> pagedData=service.queryStandard(args,page, limit);
			LayGridJsonUtils.wirte(response, pagedData.getData(),pagedData.getTotalRow());
		}
		catch(Exception e){
			e.printStackTrace();
			LayGridJsonUtils.error(response, "服务出现错误");
		}
		
	}
	
	/**
	 * 车辆基本信息增，改时选择设备类型等
	 * @param request
	 * @param page
	 * @param limit
	 * @param response
	 */
	@RequestMapping("/loadForCarInfo")
	public void loadForCarInfo(HttpServletRequest request,Integer page,Integer limit,HttpServletResponse response){
		try{
			Map<String,Object> args=RequestUtils.getParams(request);
			args.put("quotaType",QuotaType.TRAVEL.getName()); //限定为查询百公里定额
			PageModel<Map<String,Object>> pagedData=service.loadForCarInfo(args,page, limit);
			LayGridJsonUtils.wirte(response, pagedData.getData(),pagedData.getTotalRow());
		}
		catch(Exception e){
			e.printStackTrace();
			LayGridJsonUtils.error(response, "服务出现错误");
		}
	}
	
	@RequestMapping("/export")
	public void export(HttpServletRequest request,HttpServletResponse response){
		try{
			Map<String,Object> args=RequestUtils.getParams(request);
			args.put("quotaType",QuotaType.TRAVEL.getName()); //限定为查询百公里定额
			PageModel<Map<String,Object>> pagedData=service.queryStandard(args,1, 10000);
			if(pagedData==null||pagedData.getData()==null||pagedData.getData().size()==0){
				JsonOutUtils.writeToResponse(response, "<script>alert('暂时无数据可导出');window.close();</script>","text/html;charset=UTF-8");
				return;
			}
			String templateName=File.separator+"excel"+File.separator+"quotaExport.xls";
			Map<String,Object> context=new HashMap<String,Object>();
			context.put("ds", pagedData.getData());
			Workbook wb=QuotaStandardExportBound.bound(context,ExcelUtils.reader(templateName,request.getSession().getServletContext()));
			ExcelUtils.exportExcel(response, "车辆油耗定额(百公里油耗)", wb);
		}
		catch(Exception e){
			e.printStackTrace();
			JsonOutUtils.writeToResponse(response, "<script>alert('导出数据出现错误');window.close();</script>","text/html;charset=UTF-8");
		}
	}
	
	@RequestMapping("/get")
	public void get(String recordId,HttpServletResponse response) throws Exception{
		Map<String,Object> mapping=service.getStandard(recordId);
		JsonOutUtils.returnOkWithData(response, mapping);
	}
	
	@RequestMapping("/delete")
	public void delete(String recordId,HttpServletResponse response){
		try{
			   if(service.inUsing(recordId)){
				   JsonOutUtils.returnError(response, "该标准还在使用中，禁止操作");
				   return;
			   }
			   service.deleteStandard(recordId);
			   JsonOutUtils.returnOk(response);
		   }
		   catch(Exception e){
			   e.printStackTrace();
			   JsonOutUtils.returnError(response, "服务出现错误");
		   }
	}
	
	@RequestMapping("/update")
	public void update(HttpServletRequest request,HttpServletResponse response){
	   try{
		   
		   Map<String,Object>  data=RequestUtils.getParams(request);
		   /*if(StringUtils.isEmpty((String)data.get("initQuota"))||StringUtils.isEmpty((String)data.get("stepQuota"))){
			   JsonOutUtils.returnError(response, "缺少必要参数：初始定额或递增定额");
			   return;
		   }*/
		   if(service.checkSame100kmQuota((String)data.get("deviceModel"),(String)data.get("quotaPower"),(String)data.get("recordId"))){
			   JsonOutUtils.returnError(response, "同型号同功率同地区的定额已存在!");
			   return;
		   }
		   SessionUser su=SessionThreadLocal.getSessionUser();
		   data.put("modifier", su!=null?su.getUserName():"unknow");
		   MapUtils.blankValToNull(data);
		   service.updateStandard(data);
		   QuotaStandardSupport.loadQuota(service.loadQuotaStandardForCache()); //同步更新缓存
		   JsonOutUtils.returnOk(response);
	   }
	   catch(Exception e){
		   e.printStackTrace();
		   JsonOutUtils.returnError(response, "服务出现错误");
	   }
		
	}
	
	@RequestMapping("/save")
	public void save(HttpServletRequest request,HttpServletResponse response){
		try{
			   Map<String,Object>  data=RequestUtils.getParams(request);
			   
			  
			  /* if(StringUtils.isEmpty((String)data.get("initQuota"))||StringUtils.isEmpty((String)data.get("stepQuota"))){
				   JsonOutUtils.returnError(response, "缺少必要参数：初始定额或递增定额");
				   return;
			   }*/
			   SessionUser su=SessionThreadLocal.getSessionUser();
			   UUIDHexGenerator  uuid=UUIDHexGenerator.getInstance();
			   data.put("modifier", su!=null?su.getUserName():"unknow");
			   data.put("modifyTime", new Date());
			   data.put("quotaType",QuotaType.TRAVEL.getName()); //限定为查询百公里定额
			   data.put("recordId", uuid.generate());
			   MapUtils.blankValToNull(data);
			   
			   service.saveStandard(data);
			   
			   QuotaStandardSupport.loadQuota(service.loadQuotaStandardForCache()); //同步更新缓存
			   JsonOutUtils.returnOk(response);
		   }
		   catch(Exception e){
			   e.printStackTrace();
			   JsonOutUtils.returnError(response, "服务出现错误");
		   }
	}
	
	
	/**
	 * 其他定额标准(作业区、发电机工作、定额上浮)
	 * @param powerQuotas
	 * @param outerQuota
	 * @param workAreas
	 * @param response
	 */
	@RequestMapping("/saveOther")
	public void saveOther(String powerQuotas,String workAreas,HttpServletResponse response){
		try{
			   if(StringUtils.isNotEmpty(powerQuotas)){  //存储发电机定额到定额表
				   List<Map<String,Object>>  news=transferRecord(powerQuotas);
				   if(news.size()>0){
					   service.saveOtherStandard(news);
					   QuotaStandardSupport.loadQuota(service.loadQuotaStandardForCache()); //同步更新缓存 
				   }
				  
			   }
			   
			   if(StringUtils.isNotEmpty(workAreas)){  // 存储作业区设置到编码表
				   List<Map<String,Object>> workAreaList=JsonParamHandler.getListMapParams(workAreas);
				   codeService.saveWorkAreaCodes(workAreaList);
			   }
			 
			   JsonOutUtils.returnOk(response);
		   }
		   catch(Exception e){
			   e.printStackTrace();
			   JsonOutUtils.returnError(response, "服务出现错误");
		   }
	}
	
	/**
	 * 其它定额配置：发电机定额、作业定额、作业区定义、上浮定额配置
	 */
	@RequestMapping("/loadOther")
	public void loadOther(String workAreaFatherCode,HttpServletResponse response){
		try{
			   List<Map<String,Object>> workAreas=codeService.queryCodeNodes(workAreaFatherCode);
			   List<Map<String,Object>>  others=service.loadOtherStandard();
			   JsonOutUtils.returnOkWithData(response,MapUtils.build("otherQuota",others,"workAreas",workAreas));  //,"config",QuotaStandardSupport.getConfig()
		   }
		   catch(Exception e){
			   e.printStackTrace();
			   JsonOutUtils.returnError(response, "服务出现错误");
		   }
	}
	
	private List<Map<String,Object>> transferRecord(String powerQuota){
		   SessionUser su=SessionThreadLocal.getSessionUser();
		   //UUIDHexGenerator  uuid=UUIDHexGenerator.getInstance();
		   List<Map<String,Object>> powerQuotaRecords=JsonParamHandler.getListMapParams(powerQuota);
		   for(Map<String,Object> itm : powerQuotaRecords){
			   itm.put("modifier", su!=null?su.getUserName():"unknow");
			   itm.put("modifyTime", new Date());
			   //itm.put("recordId", uuid.generate());
			   itm.put("oilType", "柴油");
			   itm.put("deviceName", "发电机");
			   itm.put("quotaType", QuotaType.ENGINE.getName());
		   }
		  
		   return powerQuotaRecords;
	 
	}
	
}
