package com.hb.proj.sysdata.controller;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.RequestMapping;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.hb.proj.car.service.CarOilMonitorService;
import com.hb.proj.car.service.OilMeterService;
import com.hb.proj.model.OilAddLog;
import com.hb.proj.model.OilChangeBO;
import com.hb.proj.sysdata.service.VolumeRatioService;
import com.hb.proj.utils.DateEditor;
import com.hb.proj.utils.JsonOutUtils;
import com.hb.proj.utils.LayGridJsonUtils;
import com.hb.proj.utils.RequestUtils;
import com.hb.xframework.dao.util.PageModel;

@Controller
@RequestMapping("/**/vol")
public class VolumeRatioController {
	
	@Autowired
	private VolumeRatioService  service;
	
	@Autowired
	private CarOilMonitorService monitorService;
	
	@Autowired
	private OilMeterService  meterService;
	
	@InitBinder
	public void initBinderDate(WebDataBinder binder) {
		binder.registerCustomEditor(Date.class, new DateEditor());
	}

	@RequestMapping("/query")
	public void query(HttpServletRequest request,Integer page,Integer limit,HttpServletResponse response){
		try{
			Map<String,Object> args=RequestUtils.getParams(request);
			PageModel<Map<String,Object>> pagedData=service.queryOilAddLog(args,page, limit);
			LayGridJsonUtils.wirte(response, pagedData.getData(),pagedData.getTotalRow(),"yyyy-MM-dd HH:mm:ss");
		}
		catch(Exception e){
			e.printStackTrace();
			LayGridJsonUtils.error(response, "服务出现错误");
		}
		
	}

	@RequestMapping("/get")
	public void get(String id,HttpServletResponse response) throws Exception{
		OilAddLog rst=service.getOilAddLog(id);
		JsonOutUtils.returnOkWithData(response, rst,"yyyy-MM-dd HH:mm:ss");
	}
	
	@RequestMapping("/add")
	public void add(OilAddLog  addLog,HttpServletResponse response){
	   try{
		   getVolumeRatio(addLog);
		   service.addOilAddLog(addLog);
		   JsonOutUtils.returnOk(response);
		  }
	   catch(Exception e){
		   e.printStackTrace();
		   JsonOutUtils.returnError(response, "服务出现错误");
	   }
		
	}
	
	@RequestMapping("/update")
	public void update(OilAddLog  addLog,HttpServletResponse response){
	   try{
		   getVolumeRatio(addLog);
		   service.updateOilAddLog(addLog);
		   JsonOutUtils.returnOk(response);
	   }
	   catch(Exception e){
		   e.printStackTrace();
		   JsonOutUtils.returnError(response, "服务出现错误");
	   }
		
	}
	
	@RequestMapping("/delete")
	public void delete(String id,HttpServletResponse response){
		try{
			   service.deleteOilAddLog(id);
			   JsonOutUtils.returnOk(response);
		   }
		   catch(Exception e){
			   e.printStackTrace();
			   JsonOutUtils.returnError(response, "服务出现错误");
		   }
	}
	
	private void getVolumeRatio(OilAddLog  addLog){
		if(addLog.getAddCount()==null){
			return;
		}
		if(addLog.getMeterVol()==null&&addLog.getMeterVol2()==null){
			return;
		}
		if(addLog.getMeterVol()!=null&&addLog.getMeterVol2()==null){
			addLog.setVolumeRatio(addLog.getAddCount().doubleValue()/addLog.getMeterVol().doubleValue());
		}
		else if(addLog.getMeterVol()==null&&addLog.getMeterVol2()!=null){
			addLog.setVolumeRatio2(addLog.getAddCount().doubleValue()/addLog.getMeterVol2().doubleValue());
		}
		else{
			double ratio=(addLog.getMeterVol().doubleValue()+addLog.getMeterVol2().doubleValue())/addLog.getMeterVol().doubleValue();
			addLog.setVolumeRatio(addLog.getAddCount().doubleValue()*ratio/addLog.getMeterVol().doubleValue());
			addLog.setVolumeRatio2(addLog.getAddCount().doubleValue()*(1-ratio)/addLog.getMeterVol2().doubleValue());
		}
		
	}
	
	
	/**
	 * 加载实际加油记录对应的测量加油记录
	 * @param id
	 * @param response
	 */
	@RequestMapping("/loadMeterAddLog")
	public void loadMeterAddLog(String id,HttpServletResponse response){
		try{
			OilAddLog  log=service.getOilAddLog(id);
			if(log==null){
				LayGridJsonUtils.error(response, "记录已不存在");
				return ;
			}
			List<OilChangeBO> rst=monitorService.loadCarOilAdd(log);
			LayGridJsonUtils.wirte(response, rst,rst!=null?rst.size():0,"yyyy-MM-dd HH:mm:ss");
		}
		catch(Exception e){
			   e.printStackTrace();
			   LayGridJsonUtils.error(response, "服务出现错误");
		   }		 
	}
	
	
	/**
	 * 根据实际加油量和测量加油量计算系数，并设置到t_oil_meter
	 * @param realLogId
	 * @param meterLog  json
	 * @param response
	 */
	@RequestMapping("/calculateRatio")
	public void calculateRatio(String realLogId,String meterLog,HttpServletResponse response){
		if(StringUtils.isEmpty(meterLog)){
			JsonOutUtils.returnError(response, "未找到测量加油数据");
			return;
		}
		List<OilChangeBO> oilChanges=JSON.parseObject(meterLog,new TypeReference<List<OilChangeBO>>() {});
		if(oilChanges==null||oilChanges.size()==0){
			JsonOutUtils.returnError(response, "未找到测量加油数据");
			return;
		}
		boolean singleMeter=oilChanges.size()==1;
		
		OilAddLog  log=service.getOilAddLog(realLogId);
		if(log==null||log.getAddCount()==null){
			JsonOutUtils.returnError(response, "未找到实际加油数据");
			return;
		}
		
		boolean boxReverse="2".equals(oilChanges.get(0).getBoxNum());
		Map<String,Object> updateLog=new HashMap<String,Object>();
		updateLog.put("id", realLogId);
		if(singleMeter){
			double ratio=log.getAddCount().doubleValue()/oilChanges.get(0).getDiffVolume().doubleValue();
			if(boxReverse){
				updateLog.put("volumeRatio2", ratio);
				updateLog.put("meterVol2",oilChanges.get(0).getDiffVolume());
			}
			else{
				updateLog.put("volumeRatio", ratio);
				updateLog.put("meterVol",oilChanges.get(0).getDiffVolume());
			}
			
		}
		else{
			double ratio=oilChanges.get(0).getDiffVolume().doubleValue()+oilChanges.get(1).getDiffVolume().doubleValue();
			ratio=oilChanges.get(0).getDiffVolume().doubleValue()/ratio;
			double ratio1=log.getAddCount().doubleValue()*ratio/oilChanges.get(0).getDiffVolume().doubleValue();
			double ratio2=log.getAddCount().doubleValue()*(1-ratio)/oilChanges.get(1).getDiffVolume().doubleValue();
			if(boxReverse){
				updateLog.put("volumeRatio",ratio2);
				updateLog.put("volumeRatio2",ratio1);
				updateLog.put("meterVol",oilChanges.get(1).getDiffVolume());
				updateLog.put("meterVol2",oilChanges.get(0).getDiffVolume());
			}
			else{
				updateLog.put("volumeRatio",ratio1);
				updateLog.put("volumeRatio2",ratio2);
				updateLog.put("meterVol",oilChanges.get(0).getDiffVolume());
				updateLog.put("meterVol2",oilChanges.get(1).getDiffVolume());
			}
			
		}
		
		service.updateOilAddLog(updateLog);
		JsonOutUtils.returnOk(response);
	}
	
	/**
	 * 启用容积系数（将系数设置到t_oil_meter表）
	 * @param carNum
	 * @param calMode 系数计算模式
	 * @param response
	 */
	@RequestMapping("/usRatio")
	public void usRatio(String carNum,Integer calMode,HttpServletResponse response){
		if(StringUtils.isEmpty(carNum)||calMode==null){
			JsonOutUtils.returnError(response, "缺少必要参数");
			return;
		}
		try{
			List<OilAddLog>  logs=service.loadOilAddLogHadRatio(carNum, calMode);
			if(logs==null||logs.size()==0){
				JsonOutUtils.returnError(response, "没有标定系数可用");
				return;
			}
			double[] ratios=avgVolumeRatio(logs);
			meterService.updateCarVolumeRatio(logs.get(0).getCarId(), ratios[0], ratios[1]);
			JsonOutUtils.returnOk(response);
		}
		catch(Exception e){
			   e.printStackTrace();
			   JsonOutUtils.returnError(response, "服务出现错误");
		}	
	}
	
	private double[] avgVolumeRatio(List<OilAddLog>  logs){
		double r1=0,r2=0;
		int num1=0,num2=0;
		for(OilAddLog log : logs){
			if(log.getVolumeRatio()!=null){
				r1+=log.getVolumeRatio().doubleValue();
				num1++;
			}
			if(log.getVolumeRatio2()!=null){
				r2+=log.getVolumeRatio2().doubleValue();
				num2++;
			}
		}
		if(num1>0){
			r1=r1/num1;
		}
		if(num2>0){
			r2=r2/num2;
		}
		
		return  new double[]{r1,r2};
	}
}
