package com.hb.proj.sysdata.service;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.xframework.dao.core.SpringJdbcDAO;
import com.hb.xframework.dao.util.UUIDHexGenerator;

@Service
public class OrgService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	public Set<String> getChildOrgByMultiId(String orgIds){
		return null;
	}
	
	public Map<String,Object> loadOrgMapping(){
		String sql="select org_id,org_name from t_org";
		return dao.queryForMapping(sql, "org_name", "org_id");
	}
	
	public List<Map<String, Object>> queryOrgSelfNodes(String superIds) {
		StringBuilder sql=new StringBuilder();
		sql.append("select org_id id,org_name name,super_id p_id,'org' node_type,assist_code,");
		sql.append("(select count(1) from t_org g2 where g2.super_id=g.org_id) child_count");
		sql.append(" from t_org g where org_id in ("+superIds+")");
		return dao.queryForListMap(sql.toString());
	}
	
	public List<Map<String, Object>> queryOrgChildNodes(String superId) {
		StringBuilder sql=new StringBuilder();
		sql.append("select org_id id,org_name name,super_id p_id,'org' node_type,assist_code,");
		sql.append("(select count(1) from t_org g2 where g2.super_id=g.org_id) child_count");
		sql.append(" from t_org g where super_id=?");
		return dao.queryForListMap(sql.toString(), superId);
	}
	
	public List<Map<String, Object>> queryAuthOrgNodeBySuperId(String superId, String userId) {
		StringBuilder sql = new StringBuilder();
		sql.append("select org_id id,super_id p_id,org_name name,'org' node_type,assist_code,");
		sql.append("(select count(1) from t_org g where g.super_id=gtab.org_id) child_count ");

		sql.append("from (select distinct g.* from t_org g ");
		
		sql.append(" where exists(");
		sql.append(" select record_id from t_user_data ud where ud.user_id=? and ud.data_id=g.org_id");
		sql.append(" )");
		sql.append(" ) gtab");

		sql.append(" where gtab.super_id=? ");
		sql.append(" order by display_num");
		return dao.queryForListMap(sql.toString(), userId, superId);
	}

	
	public Map<String, Object> getOrgById(String orgId) {
		StringBuilder sql = new StringBuilder();
		sql.append("select g.*,supg.org_name super_org_name");
		sql.append(" from t_org g ");
		sql.append(" left join t_org supg on g.super_id=supg.org_id");
		sql.append(" where g.org_id=?");
		return dao.queryForSingleMap(sql.toString(), orgId);
	}
	
	public String addOrg(Map<String, Object> org){
		UUIDHexGenerator uuid = UUIDHexGenerator.getInstance();
		org.put("orgId", uuid.generate());
		dao.insert(org, "t_org");
		return (String)org.get("orgId");
	}
	
	public boolean updateOrg(Map<String, Object> org,String newAssistCode){
		if(StringUtils.isNotEmpty(newAssistCode)){
			updateAssistCode((String)org.get("assistCode"),newAssistCode);
			org.put("assistCode",newAssistCode);
		}
		dao.update(org, "t_org", "org_id");
		return true;
	}
	
	public boolean deleteOrg(String orgId){
		String sql = "delete from t_org where org_id=?";
		dao.getJdbcTemplate().update(sql, orgId);
		return true;
	}
	
	public boolean deleteSubOrg(String orgId){
		Map<String,Object> org=getOrgById(orgId);
		if(org==null||StringUtils.isEmpty((String)org.get("assistCode"))){
			return true;
		}
		
		dao.getJdbcTemplate().update("delete g1 from t_org g1 where assist_code like CONCAT('%',?,'%')", org.get("assistCode"));
		return true;
	}
	
	/**
	 * 获得指定级别下的最大assistCode
	 * @param superId
	 * @param superAssistCode
	 * @return
	 */
	public String generateLeveledCode(String superId) {
		String superAssistCode=null;
		if("0".equals(superId)){  //顶级父节点
			superAssistCode="C";
		}
		else{
			Map<String,Object> superMenu=getOrgById(superId);
			if(superMenu==null){
				return null;
			}
			superAssistCode=(String)superMenu.get("assistCode");
			if(StringUtils.isEmpty(superAssistCode)){
				superAssistCode="C";
			}
		}
		
		String subMaxAssistCode = getMaxSubAssistCode(superId);
		
		if (subMaxAssistCode == null || "null".equalsIgnoreCase(subMaxAssistCode)) { // 没有子编码时
			return superAssistCode + "000";
		}
		String subSerial = subMaxAssistCode.replaceFirst(superAssistCode, ""); // 子编码流水号

		int nextNum = Integer.parseInt(subSerial) + 1;
		subSerial = String.format("%1$03d", nextNum);
		return superAssistCode + subSerial;
	}
	
	
	private String getMaxSubAssistCode(String superId) {
		String sql = "select max(assist_code) assist_code from t_org where super_id=?";
		Map<String,Object> rst=dao.queryForSingleMap(sql,superId);
		if(rst==null||rst.size()==0){
			return null;
		}
		return (String)rst.get("assistCode");
	}
	
	
	private boolean updateAssistCode(String assistCode, String newAssistCode) {
		StringBuilder sql = new StringBuilder();
		sql.append("update t_org set assist_code=replace(assist_code,?,?)");
		sql.append(" where assist_code like ?");
		dao.getJdbcTemplate().update(sql.toString(), assistCode, newAssistCode, assistCode + "%");
		return true;
	}
	
	public boolean autoSortChildOrg(String superId){
		List<Map<String,Object>>  subs=queryOrgChildNodes(superId);
		if(subs==null||subs.size()==0){
			return true;
		}
		sort(subs);
		// 更新序号
		int i=1;
		for(Map<String,Object> itm : subs){
			updateOrgDisplayNo((String)itm.get("id"), i++);
		}
		return true;
	}	
		
	
	
	public boolean updateOrgDisplayNo(String orgId, int displayNo) {
		String sql = "update t_org set display_num=? where org_id=?";
		dao.getJdbcTemplate().update(sql, displayNo, orgId);
		return true;
	}

	
	public void sort(List<Map<String,Object>>  subs){
		    Collections.sort(subs, new Comparator<Map<String,Object>>(){
		    	
			@Override
			public int compare(Map<String, Object> obj1, Map<String, Object> obj2) {
				String name1=(String)obj1.get("name");
				String name2=(String)obj2.get("name");
				if(name1.equals(name2)){
					return 0;
				}
				name1=trans(name1);
				name2=trans(name2);
				Pattern p=Pattern.compile("\\D*(\\d+)\\D*");
				Matcher m1=p.matcher(name1);
				Matcher m2=p.matcher(name2);
				String s1=null;
				String s2=null;
				int idx1=0;
				int idx2=0;
				while(m1.find()){
					s1=m1.group(1);
					if(m2.find()){
						s2=m2.group(1);	
					}
					else{
						break;
					}
					int ind1=name1.indexOf(s1,idx1);
					int ind2=name2.indexOf(s2,idx2);
					idx1=ind1;
					idx2=ind2;
					if(ind1==ind2){
						String pre1=name1.substring(0,ind1);
						String pre2=name2.substring(0,ind2);
						if(pre1.equals(pre2)){
							long num1=Long.parseLong(s1);
							long num2=Long.parseLong(s2);
							if(num1==num2){
								continue;
							}
							else{
								return (int)(num1-num2);
							}
						}
						else{
							break;
						}
					}
					else{
						break;
					}
				}
				return name1.compareTo(name2);
			}
			
			
			private String trans(String str){
				String[] strnum={"一","二","三","四","五","六","七","八","九"};
				int i=1;
				for(String s : strnum){
					str=str.replaceAll(s, String.valueOf(i));
					i++;
				}
				return str;
			}
			
		});
	}
}
