package com.hb.proj.sysdata.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.hb.proj.car.quota.QuotaType;
import com.hb.proj.model.Quota;
import com.hb.proj.model.QuotaRulePO;
import com.hb.xframework.dao.core.SpringJdbcDAO;
import com.hb.xframework.dao.util.FieldAttrConverter;
import com.hb.xframework.dao.util.PageModel;
import com.hb.xframework.dao.util.UUIDHexGenerator;
import com.hb.xframework.util.JsonParamHandler;

@Service
public class QuotaStandardService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	/**
	 * 加载定额标准到缓存中
	 * 由于百公里油耗标准存储结构改变，需要特殊处理2020.8.25
	 * 采用新的定额标准存储结构及确认逻辑, 百公里和作业在一条记录，同用quotaType='百公里耗油' 2021-10
	 * @return
	 */
	public List<Quota>  loadQuotaStandardForCache(){
		List<QuotaRulePO> pos=dao.queryForListPojo("select * from t_quota_rule",QuotaRulePO.class);
		if(pos==null||pos.size()==0){
			return null;
		}
		List<Quota> rst=new ArrayList<Quota>();
		for(QuotaRulePO po : pos){
			rst.addAll(transferQuota(po));
		}
		pos=null;
		return rst;
	}
	
	//采用新的定额标准存储结构及确认逻辑2021-10,百公里定额上浮规则涉及到设备名称，所以定额标准对象里要存储设备名称
	//百公里定额两个上浮规则：1.冬季月份（用户指定月份）；2.车辆使用达到一定年限上浮 
	private List<Quota> transferQuota(QuotaRulePO po){
		
		//根据使用年限上浮百公里定额处理
		Map<Integer,Double> riseMap=new HashMap<Integer,Double>();
		riseMap.put(5, po.getAge5Rise());
		riseMap.put(10, po.getAge10Rise());
		riseMap.put(15, po.getAge15Rise());
		
		List<Quota>  rst=new ArrayList<Quota>();
		
		if(QuotaType.TRAVEL.getName().equals(po.getQuotaType())){       //百公里定额(含作业定额数据，要拆分)
			rst.add(new Quota(po.getRecordId(),null,QuotaType.WORK.getName(),po.getWorkHourQuota(),po.getDeviceName()));  //分解出作业定额
			
			Quota quota100km=null;
			String workAreaQuota=po.getWorkAreaQuota();
			if(StringUtils.isNotEmpty(workAreaQuota)){
				List<Map<String,Object>> quotas=JsonParamHandler.getListMapParams(workAreaQuota);  //分解各地区的百公里定额
				for(Map<String,Object> itm : quotas){
					quota100km=new Quota(po.getRecordId(),(String)itm.get("areaCode"),po.getQuotaType(),((Number)itm.get("quota")).doubleValue(),po.getDeviceName());
					quota100km.setRiseMap(riseMap);
					rst.add(quota100km);
				}
			}
		}
		
		else if(QuotaType.ENGINE.getName().equals(po.getQuotaType())){  //发电机
			rst.add(new Quota(po.getRecordId(),null,po.getQuotaType(),po.getWorkHourQuota(),po.getDeviceName()));
		}
		
		
		
		return rst;
	}
	
	/*
	private void transferQuota(Map<String,Object> po,List<Quota> rst){
		Quota quota=new Quota((String)po.get("deviceName"),(String)po.get("deviceModel"),(Integer)po.get("deviceAge"),(String)po.get("quotaType"),(Number)po.get("quotaCount"));
		if(QuotaType.TRAVEL.getName().equals(quota.getQuotaType())){ //如果是百公里油耗,转换数据
			Quota itm=null;
			for(int i=0;i<20;i++){ //2021.7调整为20年
				itm=new Quota(quota.getDeviceName(),quota.getDeviceModel(),quota.getDeviceAge(),quota.getQuotaType(),quota.getQuotaCount());
				itm.setDeviceAge(i+1);
				itm.setQuotaCount((Number)po.get("age"+(i+1)+"Quota"));
				rst.add(itm);
			}
			quota=null;
		}
		else{
			rst.add(quota);
		}
		po.clear();
		po=null;
	}*/
	
	
	
	public PageModel<Map<String,Object>> queryStandard(Map<String,Object> args,int pageNo,int pageSize){
		List<Object> sqlParams=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder(100);
		sql.append("select s.* from t_quota_rule s ");
		sql.append(" where 1=1");
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("deviceName"))){
			String[] dns=((String)args.get("deviceName")).split("[,，;；\\-~\\|\\s]+");
			if(dns.length>1){ //两个参数
				sql.append(" and ((device_name like ?  and device_model like ?) or (device_name like ?  and device_model like ?))");
				sqlParams.add("%"+dns[0]+"%");
				sqlParams.add("%"+dns[1]+"%");
				sqlParams.add("%"+dns[1]+"%");
				sqlParams.add("%"+dns[0]+"%");
			}
			else{
				sql.append(" and (device_name like ? or device_model like ?)");
				sqlParams.add("%"+dns[0]+"%");
				sqlParams.add("%"+dns[0]+"%");
			}
		}
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("quotaType"))){
			sql.append(" and quota_type = ?");
			sqlParams.add(args.get("quotaType"));
		}
		//客户端指定排序规则
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("sortField"))){
			String sortField=FieldAttrConverter.getFieldName((String)args.get("sortField"));
			sql.append(" order by "+sortField+" "+(args.get("sortAction")==null?"asc":args.get("sortAction")));
		}
		else{
			sql.append(" order by device_name,quota_type ");
		}			
		Object[] sqlArgs=sqlParams.size()>0?sqlParams.toArray():null;
		return dao.queryForPagedListMap(sql.toString(),pageNo, pageSize,sqlArgs);
	}
	
	
	public PageModel<Map<String,Object>> loadForCarInfo(Map<String,Object> args,int pageNo,int pageSize){
		List<Object> sqlParams=new ArrayList<Object>();
		StringBuilder sql=new StringBuilder(100);
		sql.append("select record_id, device_name,device_model,quota_power,oil_type,quota_type from t_quota_rule s ");
		sql.append(" where 1=1");
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("deviceName"))){
			String[] dns=((String)args.get("deviceName")).split("[,，;；\\-~\\|\\s]+");
			if(dns.length>1){ //两个参数
				sql.append(" and ((device_name like ?  and device_model like ?) or (device_name like ?  and device_model like ?))");
				sqlParams.add("%"+dns[0]+"%");
				sqlParams.add("%"+dns[1]+"%");
				sqlParams.add("%"+dns[1]+"%");
				sqlParams.add("%"+dns[0]+"%");
			}
			else{
				sql.append(" and (device_name like ? or device_model like ?)");
				sqlParams.add("%"+dns[0]+"%");
				sqlParams.add("%"+dns[0]+"%");
			}
		}
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("quotaType"))){
			sql.append(" and quota_type = ?");
			sqlParams.add(args.get("quotaType"));
		}
		//客户端指定排序规则
		if(args!=null&&StringUtils.isNotEmpty((String)args.get("sortField"))){
			String sortField=FieldAttrConverter.getFieldName((String)args.get("sortField"));
			sql.append(" order by "+sortField+" "+(args.get("sortAction")==null?"asc":args.get("sortAction")));
		}
		else{
			sql.append(" order by device_name,quota_type ");
		}			
		Object[] sqlArgs=sqlParams.size()>0?sqlParams.toArray():null;
		return dao.queryForPagedListMap(sql.toString(),pageNo, pageSize,sqlArgs);
	}
	
	public void saveStandard(Map<String,Object>  data){
		dao.insert(data, "t_quota_rule");
	}
	
	public void saveStandard(List<Map<String,Object>> datas){
		if(datas==null||datas.size()==0){
			return ;
		}
		if(datas.size()==1){
			dao.insert(datas.get(0), "t_quota_rule");
		}
		else{
			dao.batchInsert(datas,"t_quota_rule");
		}
	}
	
	public Map<String,Object> getStandard(String recordId){
		String sql="select * from t_quota_rule where record_id=?";
		return dao.queryForSingleMap(sql, recordId);
	}
	
	public void deleteStandard(String recordId){
		dao.getJdbcTemplate().update("delete from t_quota_rule where record_id=?", recordId);
	}
	
	
	public void updateStandard(Map<String,Object> data){
		/*Map<String,Object> oldStandard=getStandard((String)data.get("recordId"));
		if(oldStandard==null){
			return;
		}
		boolean hadChange=!oldStandard.get("deviceName").equals(data.get("deviceName"));
		hadChange=hadChange||!oldStandard.get("deviceModel").equals(data.get("deviceModel"));
		hadChange=hadChange||!oldStandard.get("quotaPower").equals(data.get("quotaPower"));
		
		//名称或型号有变化的，需要同步到车辆基本信息中
		if(oldStandard!=null&&(!oldStandard.get("deviceName").equals(data.get("deviceName")) || ! oldStandard.get("deviceModel").equals(data.get("deviceModel")))){
			String sql="update t_car set device_name=?,device_model=? where device_name=? and device_model=?";
			dao.getJdbcTemplate().update(sql,data.get("deviceName"),data.get("deviceModel"),oldStandard.get("deviceName"),oldStandard.get("deviceModel"));
		}*/
		data.put("modifyTime", new Date());
		dao.update(data, "t_quota_rule", "record_id");
	}
	
	//t_car表新加quota_rule_id关联到t_quota_rule表
	public boolean inUsing(String recordId){
		//是否在用判断
		String sql="select c.car_num from t_car c where c.quota_rule_id=?";
		List<Map<String,Object>>  rst=dao.queryForListMap(sql, recordId);
		if(rst!=null&&rst.size()>0){
			return true;
		}
		return false;
		//同类唯一标准判断
		/*sql="select q.device_name from t_quota_standard q where q.quota_type=(select quota_type from t_quota_standard s where s.record_id=?)";
		rst=dao.queryForListMap(sql, recordId);
		if(rst!=null&&rst.size()<=1){
			return true;
		}
		return false;*/
	}
	
	private Map<String,Object>  getOtherQuotaByModel(String deviceModel,String quotaType){
		return dao.queryForSingleMap("select * from t_quota_rule where device_model=? and quota_type=?", deviceModel,quotaType);
	}
	
	/**
	 * 2021.9.23 只需要存储发电机定额(已有定额不能更改记录的recordId，否在和车辆的关联断开了)
	 * @param datas
	 */
	public void saveOtherStandard(List<Map<String,Object>> datas){
		if(datas==null||datas.size()==0){
			return ;
		}
		Map<String,Object>  oldQuota=null;
		UUIDHexGenerator  uuid=UUIDHexGenerator.getInstance();
		for(Map<String,Object> itm : datas){
			if(StringUtils.isBlank((String)itm.get("recordId"))){  //可能是前端新加，或删除后恢复的
				oldQuota=getOtherQuotaByModel((String)itm.get("deviceModel"),QuotaType.ENGINE.getName());
				if(oldQuota!=null&&oldQuota.size()>0){  //删除后恢复的
					itm.put("recordId", oldQuota.get("recordId"));  //使用旧的recordId，避免产生新的recordId，断开了关联
				}
				else{ //新加的
					itm.put("recordId", uuid.generate());
				}
				
			}
		}
		
		//先删除同类记录
		List<Map<String,Object>> olddatas=dao.queryForListMap("select * from t_quota_rule where quota_type=?", QuotaType.ENGINE.getName());
		if(olddatas!=null&&olddatas.size()>0){
			dao.getJdbcTemplate().update("insert into t_data_log(con,create_time,table_name,data_action) values(?,sysdate(),'t_quota_rule','delete')",JSON.toJSONString(olddatas));
		}
		dao.getJdbcTemplate().update("delete from t_quota_rule where quota_type=?",QuotaType.ENGINE.getName());
		dao.batchInsert(datas,"t_quota_rule");
		
	}
	
	/**
	 * 2021.9.21作业定额已经合并到百公里定额记录中,其它定额中只有发电机，无取暖
	 * @return
	 */
	public List<Map<String,Object>> loadOtherStandard(){
		String types="'"+QuotaType.ENGINE.getName()+"'";//"'"+QuotaType.ENGINE.getName()+"','"+QuotaType.WARM.getName()+"','"+QuotaType.WORK.getName()+"'";
		String sql="select * from t_quota_rule where quota_type in("+types+")";
		return dao.queryForListMap(sql);
	}
	
	/**
	 * 同型号、功率的设备在指定作业区的定额是否存在(新的定额标准：由型号、功率、作业区共同决定)
	 * @param deviceModel
	 * @param quotaPower
	 * @return
	 */
	public boolean checkSame100kmQuota(String deviceModel,String quotaPower,String recordId){
		String sql=null;
		Map<String,Object> rst=null;
		if(StringUtils.isNotEmpty(recordId)){
			sql="select record_id from t_quota_rule where device_model=? and quota_power=? and record_id <> ?";
			rst=dao.queryForSingleMap(sql, deviceModel,quotaPower,recordId);
		}
		else{
			sql="select record_id from t_quota_rule where device_model=? and quota_power=?";
			rst=dao.queryForSingleMap(sql, deviceModel,quotaPower);
		}
		return rst!=null&&rst.size()>0;
	}
	
}
