package com.hb.proj.sysdata.service;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hb.proj.utils.CodeConstant;
import com.hb.xframework.dao.core.SpringJdbcDAO;
import com.hb.xframework.dao.util.FieldAttrConverter;
import com.hb.xframework.dao.util.UUIDHexGenerator;

@Service
public class SortCodeService {

	@Autowired
	private SpringJdbcDAO  dao;
	
	
	//专为保存作业区编码,其他定额设置页面
	public void saveWorkAreaCodes(List<Map<String,Object>> codes){  //{codeId,codeName,fatherCodeId}
		Map<String,Object> idx=new HashMap<String,Object>();
		for(Map<String,Object> itm : codes){
			idx.put(itm.get("fatherCodeId")+"_"+itm.get("codeName"), 1);
			if(StringUtils.isEmpty((String)itm.get("codeId"))){  //可能是前端新加，或删除后恢复的
				if(existsCode((String)itm.get("fatherCodeId"),(String)itm.get("codeName"))){   //删除后恢复的
					continue;
				}
				else{
					addSortCode(itm);
				}
			}
			else{
				dao.update(itm, "t_sort_code", "code_id");
			}
		}
		//删除多余的
		List<Map<String, Object>> oldWorkAreas=queryCodeNodes(CodeConstant.SORT_WORK_AREA);
		Set<String>  delCodeIds=new HashSet<String>();
		String idxKey=null;
		for(Map<String, Object>  itm : oldWorkAreas){
			idxKey=itm.get("pId")+"_"+itm.get("name");
			if(!idx.containsKey(idxKey)){
				delCodeIds.add((String)itm.get("id"));
			}
		}
		
		if(delCodeIds.size()==0){
			return;
		}
		dao.getJdbcTemplate().update("delete from t_sort_code where code_id in ('"+StringUtils.join(delCodeIds,"','")+"')");
		
	}
	
	public Map<String,Object> loadSortCodeMapping(String fatherId,String keyField,String valField){
		String sql="select "+keyField+","+valField+" from t_sort_code where father_code_id=?";
		return dao.queryForMapping(sql, keyField, valField,fatherId);
	}
	
	public Map<String,Object> loadSortCodeMapping(String fatherId){
		String sql="select code_id,code_name from t_sort_code where father_code_id=?";
		return dao.queryForMapping(sql, "code_name", "code_id",fatherId);
	}
	
	public List<Map<String, Object>> loadCodes(String field,String val,String logic,String valkey,String txtkey) {
		field=FieldAttrConverter.getFieldName(field);
		valkey=FieldAttrConverter.getFieldName(valkey);
		txtkey=FieldAttrConverter.getFieldName(txtkey);
		if(StringUtils.isEmpty(logic)){
			logic="=";
		}
		StringBuilder sql = new StringBuilder();
		sql.append("select "+valkey+" val, "+txtkey+"  txt from t_sort_code d");
		sql.append(" where "+field+" "+logic+"'"+val+"'");
		return dao.queryForListMap(sql.toString());
	}
	
	public List<Map<String, Object>> queryCodeNodes(String fatherCodeId) {
		StringBuilder sql = new StringBuilder();
		sql.append("select code_id id,code_name name,father_code_id p_id,'code' node_type,");
		sql.append("(select count(1) from t_sort_code sub where sub.father_code_id=d.code_id) child_count");
		sql.append(" from t_sort_code d");
		sql.append(" where d.father_code_id=?");
		sql.append(" order by d.display_num");
		return dao.queryForListMap(sql.toString(), fatherCodeId);
	}
	
	public Map<String,Object> getSortCode(String codeId) {
		StringBuilder sql = new StringBuilder();
		sql.append("select d.*,ifnull(supd.code_name,'') father_code_name from t_sort_code d");
		sql.append(" left join t_sort_code supd on d.father_code_id=supd.code_id");
		sql.append(" where d.code_id=?");
		return dao.queryForSingleMap(sql.toString(), codeId);
	}
	
	public String addSortCode(Map<String, Object> code) {
		Map<String,Object> superCode = getSortCode((String) code.get("fatherCodeId"));
		String nextAssistCode = generateLeveledCode((String) code.get("fatherCodeId"), superCode != null ? (String)superCode.get("assistCode") : null);
		UUIDHexGenerator  uuid=UUIDHexGenerator.getInstance();
		code.put("codeId", uuid.generate());
		code.put("assistCode", nextAssistCode);
		code.put("createTime", new Date());
		dao.insert(code, "t_sort_code");
		return (String)code.get("codeId");
	}
	
	
	
	public boolean updateSortCode(Map<String, Object> code, String oldFatherCodeId) {
		if (!oldFatherCodeId.equals(code.get("fatherCodeId"))) { // 隶属关系变动后，修正assistCode,两个dao方法顺序固定这样
			updateAssistCode((String) code.get("assistCode"), (String) code.get("fatherCodeId"));
			code.remove("assistCode");  //updateAssistCode  方法中已经更新为新父节点的assistCode，此处的为旧值，不再参与更新操作
		}
		dao.update(code, "t_sort_code", "code_id");
		return true;
	}
	
	public boolean deleteSortCode(String codeId) {
		StringBuilder sql = new StringBuilder(100);
		sql.append("delete from t_sort_code  where code_id=?");
		dao.getJdbcTemplate().update(sql.toString(), codeId);
		return true;
	}
	
	
	private String generateLeveledCode(String fatherCodeId, String superAssistCode) {
		String subMaxSortCode = getMaxAssistCode(fatherCodeId);;
		String superSortCode = superAssistCode != null ? superAssistCode : "C";
		
		if (subMaxSortCode == null || "null".equalsIgnoreCase(subMaxSortCode)) { // 没有子编码时
			return superSortCode + "000";
		}
		String subSerial = subMaxSortCode.replaceFirst(superSortCode, ""); // 子编码流水号

		int nextNum = Integer.parseInt(subSerial) + 1;
		subSerial = String.format("%1$03d", nextNum);
		return superSortCode + subSerial;
	}

	
	
	public boolean existsCode(String fatherCodeId, String codeName) {
		String sql = "select count(1) from t_sort_code where code_name=? and father_code_id=?";
		int num = dao.getJdbcTemplate().queryForObject(sql, Integer.class,codeName, fatherCodeId);
		return num > 0;
	}
	
	public boolean existsDomainCode(String fatherCodeId, String domainCode) {
		String sql = "select count(1) from t_sort_code where domain_code=? and father_code_id=?";
		int num = dao.getJdbcTemplate().queryForObject(sql,Integer.class, domainCode, fatherCodeId);
		return num > 0;
	}
	
	
	private boolean updateAssistCode(String assistCode, String newFatherCodeId) {
		Map<String,Object> superCode = getSortCode(newFatherCodeId);
		if (superCode == null) {
			return true;
		}
		String newAssistCode = generateLeveledCode(newFatherCodeId, (String)superCode.get("assistCode"));
		
		StringBuilder sql = new StringBuilder();
		sql.append("update t_sort_code set assist_code=replace(assist_code,?,?)");
		sql.append(" where assist_code like ?");
		dao.getJdbcTemplate().update(sql.toString(), assistCode, newAssistCode, assistCode + "%");
		return true;
	}
	
	private String getMaxAssistCode(String fatherCodeId) {
		String sql = "select max(assist_code) assist_code from t_sort_code where father_code_id=?";
		Map<String,Object> rst=dao.queryForSingleMap(sql, fatherCodeId);
		return rst!=null?(String)rst.get("assistCode"):null;
	}
}
