package com.hb.proj.utils;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang.StringUtils;

import com.hb.proj.sysdata.service.OrgService;
import com.hb.xframework.util.ApplicationContextUtils;

/**
 * 数据权限统一处理类
 * 处理流程：用户个人数据权限（所属组织+权限组织）--》组织下的车辆数据（所属、所在组织）
 * authOrgs只能存储子节点全选中的父节点或者将所有子节点存储
 *
 */
public class DataAuthUtils {

	
	/**
	 * 用户个人数据权限
	 * @return
	 */
	public static String getUserAuthSQL(MySessionUser us){
		String orgId=us.getOrgId();
		String authOrg=us.getAuthOrgs();
		if(StringUtils.isNotEmpty(authOrg)){
			orgId="'"+orgId+"','"+authOrg.replaceAll("[,;\\s]+", "','")+"'";
		}
		else{
			orgId="'"+orgId+"'";
		}
		return getInOrgAuthSQL(orgId);
	}
	
	/**
	 * 根据起始组织获取按组织及下级过滤sql
	 * @param inOrg
	 * @return
	 */
	public static String getInOrgAuthSQL(String inOrg){
		StringBuilder sql=new StringBuilder();
		sql.append(" in (select org_id from t_org  start with department_id in ("+inOrg+")");
		sql.append(" connect by prior department_id=parent_id)");
		return sql.toString();
	}
	
	/**
	 * 在权限内找指定组织机构
	 * @param us
	 * @param orgId
	 * @return
	 */
	public static Set<String>  getChildOrgIds(MySessionUser us,String orgId){
		Set<String> orgs=getChildOrgIds(us);
		Set<String> orgs2=getChildOrgIds("'"+orgId+"'");
		orgs.retainAll(orgs2);
		return orgs;
	}
	
	/**
	 * 根据用户信息获取下级组织
	 * @param startOrgIds
	 * @return
	 */
	public static Set<String>  getChildOrgIds(MySessionUser us){
		String orgId=us.getOrgId();
		String authOrg=us.getAuthOrgs();
		if(StringUtils.isNotEmpty(authOrg)){
			orgId="'"+orgId+"','"+authOrg.replaceAll("[,;\\s]+", "','")+"'";
		}
		else{
			orgId="'"+orgId+"'";
		}
		return getChildOrgIds(orgId);
	}
	
	public static Set<String>  getChildOrgIds(String startOrgIds){
		OrgService service=ApplicationContextUtils.getBean("orgService", OrgService.class);
		return service.getChildOrgByMultiId(startOrgIds);
	}
	
	
	public static String getUserAuthOrg(MySessionUser us){
		String orgId=us.getOrgId();
		String authOrg=us.getAuthOrgs();
		if(StringUtils.isNotEmpty(authOrg)){
			orgId="'"+orgId+"','"+authOrg.replaceAll("[,;\\s]+", "','")+"'";
		}
		else{
			orgId="'"+orgId+"'";
		}
		return orgId;
	}
	
	public static Set<String> getAuthOrgAssistCode(MySessionUser us){
		if(us==null||us.isSysAdmin()){
			return null;
		}
		String orgAssistCode=us.getOrgAssistCode();
		//String authOrg=us.getAuthOrgs();
		Set<String> codes=new HashSet<String>();
		/*if(StringUtils.isNotEmpty(authOrg)){
			codes.addAll(Arrays.asList((authOrg.split("[,;\\s]+"))));
		}*/
		if(StringUtils.isNotEmpty(orgAssistCode)){
			codes.add(orgAssistCode);
		}
		return codes.size()>0?codes:null;
	}
}
