package com.hb.proj.utils;

import java.util.Map;
import java.util.concurrent.TimeUnit;

import okhttp3.Call;
import okhttp3.FormBody;
import okhttp3.FormBody.Builder;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

public class OkhttpUtils {

	private static OkHttpClient singleton;
	
	
	private OkhttpUtils(){

    }
	
	public static OkHttpClient getInstance() {
        if (singleton == null)
        {
            synchronized (OkhttpUtils.class)
            {
                if (singleton == null)
                {
                    singleton = new OkHttpClient.Builder().connectTimeout(10, TimeUnit.SECONDS).readTimeout(10, TimeUnit.SECONDS).build();
                }
            }
        }
        return singleton;
    }
	
	
	public static String get(String url){
		try{
			OkHttpClient client=OkhttpUtils.getInstance();
			Request req=new Request.Builder()
			.url(url)
			.get()
			.build();
			Response response = client.newCall(req).execute();
		    if (response.isSuccessful()) {
		        return response.body().string();
		    } else {
		        return null;
		    }

	
		}
		catch (Exception e) {
		      e.printStackTrace();
		      return null;
	     }
		
	}
	
	public static String post(String url,Map<String,String> args){
		return post(url,args,null);
	}
	
	
	public static String post(String url,Map<String,String> args,Map<String,String> headers){
	  try {
		
		OkHttpClient client=OkhttpUtils.getInstance();
		Builder b=new FormBody.Builder();
		for(String key : args.keySet()){
			b.add(key, args.get(key));
		}
		
		okhttp3.Request.Builder rb=new Request.Builder();
		if(headers!=null&&headers.size()>0){
			for(String h : headers.keySet()){
				rb.addHeader(h, headers.get(h));
			}
		}
		
		Request req=rb.url(url)
        .post(b.build())
        .build();
        final Call call = client.newCall(req);
        Response rmtResp=call.execute();
        return rmtResp.isSuccessful()?rmtResp.body().string():null;
	    } catch (Exception e) {
	      e.printStackTrace();
	      return null;
        }
       
	}
	
	

}
