package com.hb.proj.utils;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.hb.proj.auth.controller.AutoLoginUtils;
import com.hb.xframework.util.SessionThreadLocal;
import com.hb.xframework.util.SessionUser;
import com.hb.xframework.web.RequestUtils;



public class SessionTimeoutFilter implements Filter {

	private  static Logger logger=LoggerFactory.getLogger(SessionTimeoutFilter.class);
	
	private FilterConfig filterConfig;
	private String[] extPaths; //需要排除的路径
	private String[] autoLoginIPs;  //可以自动登陆ip列表
	
	
	
	@Override
	public void destroy() {
		filterConfig=null;
		extPaths=null;
		autoLoginIPs=null;
	}

	@Override
	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain filterChain) throws IOException, ServletException {
		HttpServletRequest httpRequest=(HttpServletRequest)request;
		HttpSession session=httpRequest.getSession(true);
		
		if(session.getAttribute(MVCConstants.CUR_USER_SESSION)==null)
		{   
			String cookieUsId=getCookieUsId(httpRequest.getCookies());
			String requestURI=httpRequest.getRequestURI();
			logger.debug("请求地址为："+requestURI);
			if(requestURI.endsWith(httpRequest.getContextPath()+"/")){
				filterChain.doFilter(request,response);
			}
			else if(isExtendPath(requestURI))
			{
				filterChain.doFilter(request,response);
			}
			else if(StringUtils.isNotEmpty(cookieUsId)){  //自动登录
				SessionUser su=AutoLoginUtils.getSessionUser(cookieUsId);
				session.setAttribute(MVCConstants.CUR_USER_SESSION,su);
				SessionThreadLocal.setSessionUser(su);
				filterChain.doFilter(request,response);
			}
			else
			{    logger.debug("no auth to access");
				 String contextPath=httpRequest.getContextPath();
				 String loginPage=filterConfig.getInitParameter("loginPage");
				 loginPage=loginPage.startsWith("/")?loginPage:"/"+loginPage;
				 loginPage=contextPath+loginPage+"?needLogin=1";  //未登录标志
				((HttpServletResponse)response).sendRedirect(loginPage);
			   
			}
		}
		else
		{
			SessionUser su=(SessionUser)session.getAttribute(MVCConstants.CUR_USER_SESSION);
			SessionThreadLocal.setSessionUser(su);
			filterChain.doFilter(request,response);

		}
	}
	
	private String  getCookieUsId(Cookie[] cookies){
		if(cookies==null){
			return null;
		}
		for(Cookie c  : cookies){
			if("usId".equals(c.getName())){
				return c.getValue();
			}
		}
		return null;
	}

	@Override
	public void init(FilterConfig filterConfig) throws ServletException {
		this.filterConfig=filterConfig;
		String val=filterConfig.getInitParameter("extpaths");
		if(val!=null)
		{
			extPaths=val.replaceAll("\\t|\\r|\\n|\\u3000|\\s", "").split(",");
			logger.debug("the following path will be extended:"+filterConfig.getInitParameter("extpaths"));
		}
		val=filterConfig.getInitParameter("autoIP");
		if(val!=null)
		{
			autoLoginIPs=val.replaceAll("\\t|\\r|\\n|\\u3000|\\s", "").split(",");
		}
		logger.debug("SessionTimeOutFilter success init");

	}
	
	private boolean isExtendPath(String path)
	{
		if(extPaths==null||extPaths.length==0){
			return false;
		}
		for(String pth : extPaths)
		{
			if(path.indexOf(pth)>=0)
			{
				logger.debug("该请求地址为排除地址："+path);
				return true;
			}
		}
		return false;
	}
	
	/*private boolean isAutoLogin(String ip){
		if(autoLoginIPs==null||autoLoginIPs.length==0){
			return false;
		}
		for(String p : autoLoginIPs)
		{
			if(ip.indexOf(p)>=0)
			{
				return true;
			}
		}
		return false;
	}*/
}
