/**
 * 页面报表公式支持，目前只支持：基于js的算术运算,公式中单元格的引用都是明确的显示的。
 * 支持引用页面中的其他DOM元素的值参与公式计算。基于jquery的$("#id").val()/text()方法
 * 支持页面编辑单元格的值，且引用了此单元格的公式会重新计算
 * 
 * 2020.7.18 
 * 1.取消单元格值更改的事件监听，不需要重新计算公式，报表无编辑功能
 * 
 * 2.增加对公式sum(C1:F1),sum(C1,C2)的支持
 * 
 * 2020.11.12
 * 1.对有固定行列功能的报表提供公式支持
 * 
 * 2020.11.23
 * 公式启用前对单元格进行索引编号（excle方式）,便于公式的计算
 */
;layui.define(["jquery"],function(exports){
	var $=layui.jquery;
	
	//根据excel表格中的单元格列名称转为表格索引（从0开始）
	var getIndexNum=function(str){
 	   var num=0;
 	   var len=str.length;
 	   for(var i=len-1;i>=0;i--){
 		   num+=(str.charAt(i).charCodeAt()-64)*Math.pow(26,len-i-1);
 	   }
 	   return num-1;
    };
    
    //sum公式替换为具体值后的表达式,适用sum(C1:H2)
    var getSumRstExp=function(dom,startIdx,endIdx){
    	var $tab=$(dom).parents("table");
    	var str=null,str2=null;
    	str=startIdx.replace(/[A-Za-z]+/g,""),str2=endIdx.replace(/[A-Za-z]+/g,"");
    	var r1=parseInt(str),r2=parseInt(str2);
    	str=startIdx.replace(/\d+/g,""),str2=endIdx.replace(/\d+/g,"");
    	var c1=getIndexNum(str),c2=getIndexNum(str2);
    	var sumExp="",val;
    	for(;r1<=r2;r1++){
    		for(var c=c1;c<=c2;c++){
    			val=$tab.find("tr:eq("+(r1-1)+") td:eq("+c+")").text();
    			val=val.replace(/\s+/g,"");
    			sumExp+="+"+(val!=null&&val!=""?val:0);
    		}
    	}
    	
    	return sumExp.length>0?sumExp.substr(1):"";
    };
    
  //sum公式替换为具体值后的表达式,适用sum(C1,H2)  cells=[C1,B2...]
    var getSumRstExp2=function(dom,cells){
    	var $tab=$(dom).parents("table");
    	var str=null,cellIdx=0,rowIdx=0;
    	var sumExp="",val;
    	for(var i=0,len=cells.length;i<len;i++){
    		str=cells[i].replace(/\d+/g,"");
    		cellIdx=getIndexNum(str);
    		str=cells[i].replace(/[A-Za-z]+/g,"");
    		rowIdx=parseInt(str)-1;
    		val=$tab.find("tr:eq("+(r1-1)+") td:eq("+c1+")").text();
    		sumExp+="+"+(val!=null&&val!=""?val:0);
    	}
    	
    	return sumExp.length>0?sumExp.substr(1):"";
    };
    
    var exeExpress=function(dom,rstExpress){
    	$me=$(dom);
    	var $fillDOM=$me;
    	if($fillDOM.find(".rpt-table-cell").length>0){
    		$fillDOM=$fillDOM.find(".rpt-table-cell");
    	}
    	var decimal=$me.attr("decimal")||"0";
   	    var zero=$me.attr("zero")||"false";  //0值是否显示为空白
    	try{
      	   var rst=eval(rstExpress);
      	   if(isFinite(rst)){
      		   if(rst==0&&zero=="true"){
      			   rst="";
      		   }
      		   else{
      			   rst=rst.toFixed(parseInt(decimal));
      		   }
      		   $fillDOM.text(rst);
 	        	
 	        }
         }
         catch(err){
      	   if(window.console){
      		   console.log("公式错误："+rstExpress+" "+err.message);
      	   }
      	   $fillDOM.text("#");
         }
    };
  
 //公式计算入口
 var exeFormula=function(dom){
	 var $me=$(dom);
	 var express=$me.attr("formula");
	 if(/^sum.+/.test(express)){
		 sumFormula(dom);
	 }
	 else{
		 simpleFormula(dom);
	 }
 };
 
 
 var sumFormula=function(dom){  //对sum(C1:F1) 公式的计算
	 var express=$(dom).attr("formula");
	 express=express.replace(/\s+/,"");
	 var reg1=/^sum\(([A-Z]+\d+(,[A-Z]+\d+){1,})\)$/g;
	 var reg2=/^sum\(([A-Z]+\d+):([A-Z]+\d+)\)$/g;
	 var rstExp=null;
	 if(reg1.test(express)){
		 //console.log(RegExp.$1)	;
		 express=RegExp.$1;
		 rstExp=getSumRstExp2(dom,express.split(","));
	 }
	 else if(reg2.test(express)){
		 rstExp=getSumRstExp(dom,RegExp.$1,RegExp.$2);
	 }
	 exeExpress(dom,rstExp);
 };
 
 
 
 
 var simpleFormula=function(dom,express){  //对C1+B2+#name 公式的计算
 	   var $me=$(dom);
 	   var $fillDOM=$me;
 	   if($fillDOM.find(".rpt-table-cell").length>0){
 	  		$fillDOM=$fillDOM.find(".rpt-table-cell");
 	   }
 	   var decimal=$me.attr("decimal")||"0";
 	   var format=$me.attr("format");
 	   var zero=$me.attr("zero")||"false";  //0值是否显示为空白
        var express=express||$me.attr("formula");
        var reg=/(#\w+|[A-Za-z]{1,2}\d+)/ig;
        //var expVars=[];
        var expDom={}; //公式每部分关联到公式单元格
        var arr;
        while((arr = reg.exec(express)) != null){
     	  // expVars.push(arr[0]);
     	   expDom[arr[0]]=dom;
        }
        var vals=null;
        var str=null;
        var cidx=null;
        var ridx=null;
        var $tab=$me.parents("table");
        var $td=null;
        for(var expVar in expDom){
     	   if(/^#\w+\d*/.test(expVar)){ //获取引用页面其他DOM的值 $("#id").val()
     		   vals=($(expVar).val()||$(expVar).text());
     	   }
     	   else if(/^[A-Za-z]{1,2}\d+/.test(expVar)){  //获取单元格的值，如：A1,B2
     		   /*str=expVar.replace(/\d+/g,"");
	           cidx=getIndexNum(str);
	           str=expVar.replace(/[A-Za-z]+/g,"");
     		   ridx=parseInt(str)-1;  //标记是用的excle中的行号，执行的时候是在js中，从0开始计数，所以减1
     		   $td=$tab.find("tr:eq("+ridx+") td:eq("+cidx+")");
     		   vals=$td.find(".rpt-table-cell").length>0?$td.find(".rpt-table-cell").text():$td.text(); //td有子元素的用子元素*/
     		   $td=$tab.find("[excel-index='"+expVar+"']");
     		   vals=$td.find(".rpt-table-cell").length>0?$td.find(".rpt-table-cell").text():$td.text(); //td有子元素的用子元素*/
     	   }
     	   express=express.replace(new RegExp(expVar+"(?!\\d)",'gm'),vals!=null&&vals!=""?vals:0); //C1+B2  公式中的单元格替换成单元格的值
     	  
        }
        var rst=0;
        try{
     	   rst=eval(express);
     	   if(isFinite(rst)){
     		   if(rst==0&&zero=="true"){
     			   rst="";
     		   }
     		   else if(format==".0%"){
     			  rst=(rst*100).toFixed(1)+"%";
     		   }
     		   else if(format==".00%"){
     			  rst=(rst*100).toFixed(2)+"%";
     		   }
     		   else{
     			   rst=rst.toFixed(parseInt(decimal));
     		   }
     		   
     		  $fillDOM.text(rst);
	        	 
	       }
        }
        catch(err){
     	   if(window.console){
     		   console.log("公式错误："+express+" "+err.message);
     	   }
     	  $fillDOM.text("#");
        }
       return expDom;
    };
    var addUseTag=function(tagMap,cell,rowidx,clmidx){ 
    	var rowspan=cell.rowSpan,colspan=cell.colSpan;
    	for(var i=0;i<rowspan;i++){
    		for(var j=0;j<colspan;j++){
    			tagMap[(rowidx+i)+"_"+(clmidx+j)]=true;
    		}
    	}
    }
    
    var getClmCharName=function(idx){ //idx从0开始的
    	var h=Math.floor(idx/26);
    	if(h>0){
    		return String.fromCharCode(h+64)+String.fromCharCode(idx%26+65);
    	}
    	else{
    		return String.fromCharCode(idx+65);
    	}
    };
    //计算公式前先按excel中的行列索引重新标定td（有合并单元格，索引还是按合并前的编号，而不是html中合并后对后续索引有影响）
    var beforFormula=function($tab){
    	var tagMap={};
    	var clmCount=0; //一行列数(合并格按拆分后算列数)
    	$tab.find("tr:first-child").find("td").each(function(idx,td){
    		if($(td).is("[patch]")){ //表头最后是补丁列不计算在内
    			return true;
    		}
    		clmCount+=$(td).attr("colspan")?parseInt($(td).attr("colspan")):1;
    	});
    	var clmidx=0,rowidx=0; //单元格列索引
    	var tds=$tab.find("td");
    	for(var i=0,len=tds.length;i<len;){
    		if(clmidx>=clmCount){//换行
    			rowidx+=1;
    			clmidx=0;
    		}
    		if(tagMap[rowidx+"_"+clmidx]){ //已经使用过
    			clmidx++;
    			continue;
    		}
    		$(tds[i]).attr("excel-index",getClmCharName(clmidx)+""+(rowidx+1));
    		addUseTag(tagMap,tds[i],rowidx,clmidx);
    		clmidx+=tds[i].colSpan;
    		i++;
    	}
    };
    
	var dom={
			start:function($tab){
				if(!$tab){
					$tab=$(".rpt-table-main>.rpt-table");
				}
				beforFormula($tab);
				$("td[formula]").each(function(){
		        	  exeFormula(this);
		        });
			}
	};
	
	exports('formula2',dom);
});